/*
 * JMGPU driver
 *
 * Copyright (c) 2020 ChangSha JingJiaMicro Electronics Co., Ltd.
 * All rights reserved. 
 *
 * Author:
 *      wj <jjwgpu@jingjiamicro.com>
 * 
 * The software and information contained herein is proprietary and 
 * confidential to JingJiaMicro Electronics. This software can only be 
 * used by JingJiaMicro Electronics Corporation. Any use, reproduction, 
 * or disclosure without the written permission of JingJiaMicro 
 * Electronics Corporation is strictly prohibited.
 *
 */
 





#if J9_OSTEODERMIS

#if J9_CHITTERLINGS

#include "jmgpu_relational.h"

#define J9_FUTTAH 0

typedef enum _USC_NN_TYPE
{
    USC_NN_TYPE_V6,
    USC_NN_TYPE_V7,
    USC_NN_TYPE_V8,
}
USC_NN_TYPE;
j9_duopoly
_InitializeUSC_NNCommands(
    IN jmk_HARDWARE Hardware,
    IN USC_NN_TYPE hw_type,
    IN jmtUINT32 kernelAddress,
    IN jmtUINT32 inImageAddress,
    IN jmtUINT32 outImageAddress,
    IN OUT jmtUINT8_PTR data_type,
    IN OUT jmtUINT32_PTR item_size,
    IN OUT jmtUINT32_PTR core_count,
    IN OUT jmtSIZE_T_PTR patchBufferSizes,
    IN OUT jmtUINT32_PTR* nnCommands
    )
{
    j9_duopoly status = J9_FLUTTERING;
    jmk_OS os = Hardware->os;

    jmtUINT32_PTR command = *nnCommands;

    jmtUINT8 kernelDataType = *data_type;
    jmtUINT8 inImageDataType = *data_type;
    jmtUINT8 outImageDataType = *data_type;

    jmtUINT32 kernelsPerCore = 1;

    jmtUINT32 inImageXSize = 3;
    jmtUINT32 inImageYSize = 2;

    jmtUINT32 outImageXSize = 2;
    jmtUINT32 outImageYSize = 1;
    jmtUINT32 outImageZSize = 1;

    jmtUINT32 kernelXYSize = 2;
    jmtUINT32 kernelZSize = 1;

    jmtUINT32 nn_layer_flush = 1, noZOffset = 0, size = J9_NONPROS(jmtUINT32) * ((hw_type == USC_NN_TYPE_V6)?16:32);
    jmtUINT32 imageEndAddress = 2048, post_shift = 0, post_shift_bit56 = 0;
    jmtUINT8 coefZP = 0, outputZP = 0;
    jmsFEATURE_DATABASE * db = (jmsFEATURE_DATABASE *)Hardware->featureDatabase;
    jmtUINT32 config = 0, index = 0;
    jmtUINT32 configuration[][3] = {

        {1, 0x2, db->j9_unpersuasible},
        {2, 0x4, db->j9_discrimination},
        {2, 0x1, db->j9mirror_transmutes},
        {2, 0x7, db->j9maths_aurivorous},
    };

    *patchBufferSizes = size;

    if (command == NULL)
    {
        j9_recaution(jmkOS_Allocate(os, size, (jmtPOINTER*)(&command)));
        j9_recaution(jmkOS_ZeroMemory(command, size));

        *nnCommands = command;
    }

    JMM_kASSERT(command != NULL);
    j9_bronchotyphus(0, 0, 0, 0, 0);

    for (index = 0; index < J9_SANDBARS(configuration); index ++)
    {
        if(configuration[index][2] > 0)
            break;
    }

    if (index == J9_SANDBARS(configuration))
         JMM_kASSERT("Hardware not support NN!");
#if 0
    index = 1;
#endif
#if J9_FUTTAH
    jmmkPRINT("The hardware(0x%0x) support %d\n", db->j9_nascent, index);
    jmmkPRINT("\tcore int8  %d\n\tcore int16 %d\n\tcore fp16  %d\n\tcore bfp16 %d\n",
        db->j9_unpersuasible, db->j9_discrimination, db->j9mirror_transmutes, db->j9maths_aurivorous);

#endif
    *item_size = configuration [index][0];
    *data_type = (jmtUINT8)configuration[index][1];
    *core_count = configuration[index][2];

    kernelDataType = *data_type;
    inImageDataType = *data_type;
    outImageDataType = *data_type;

    switch (hw_type)
    {
    case USC_NN_TYPE_V8:
        noZOffset = 1;
#if 0
        outputZP = (*data_type == 0x2 && (jmkHARDWARE_IsFeatureAvailable(Hardware, J9_HANDLE_ATTRIBUTE_MASKALONGE)?(-128):0));
#else
        outputZP = 0;
#endif
        post_shift = (jmkHARDWARE_IsFeatureAvailable(Hardware, J9_HANDLE_ATTRIBUTE_LITTERMATE))?0x1f:0;
        post_shift_bit56 = (jmkHARDWARE_IsFeatureAvailable(Hardware, J9_HANDLE_ATTRIBUTE_LITTERMATE))?3:0;

        break;
    case USC_NN_TYPE_V7:
    case USC_NN_TYPE_V6:
        post_shift = (*data_type == 0x2)?15:0;
        break;
    default:
        break;
    }


    j9_uncoherentness(command,
                      ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 0:0) - (0 ?
 0:0) + 1))))))) << (0 ?
 0:0))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 0:0) - (0 ? 0:0) + 1))))))) << (0 ? 0:0)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 1:1) - (0 ?
 1:1) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 1:1) - (0 ?
 1:1) + 1))))))) << (0 ?
 1:1))) | (((jmtUINT32) ((jmtUINT32) (noZOffset) & ((jmtUINT32) ((((1 ?
 1:1) - (0 ?
 1:1) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 1:1) - (0 ? 1:1) + 1))))))) << (0 ? 1:1)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 5:2) - (0 ?
 5:2) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 5:2) - (0 ?
 5:2) + 1))))))) << (0 ?
 5:2))) | (((jmtUINT32) ((jmtUINT32) (kernelXYSize) & ((jmtUINT32) ((((1 ?
 5:2) - (0 ?
 5:2) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 5:2) - (0 ? 5:2) + 1))))))) << (0 ? 5:2)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 19:6) - (0 ?
 19:6) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 19:6) - (0 ?
 19:6) + 1))))))) << (0 ?
 19:6))) | (((jmtUINT32) ((jmtUINT32) ((kernelZSize & 0x3FFF)) & ((jmtUINT32) ((((1 ?
 19:6) - (0 ?
 19:6) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 19:6) - (0 ? 19:6) + 1))))))) << (0 ? 19:6)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 26:20) - (0 ?
 26:20) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 26:20) - (0 ?
 26:20) + 1))))))) << (0 ?
 26:20))) | (((jmtUINT32) ((jmtUINT32) (kernelsPerCore) & ((jmtUINT32) ((((1 ?
 26:20) - (0 ?
 26:20) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 26:20) - (0 ? 26:20) + 1))))))) << (0 ? 26:20)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 28:27) - (0 ?
 28:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 28:27) - (0 ?
 28:27) + 1))))))) << (0 ?
 28:27))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 28:27) - (0 ?
 28:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 28:27) - (0 ? 28:27) + 1))))))) << (0 ? 28:27)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 29:29) - (0 ?
 29:29) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 29:29) - (0 ?
 29:29) + 1))))))) << (0 ?
 29:29))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 29:29) - (0 ?
 29:29) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 29:29) - (0 ? 29:29) + 1))))))) << (0 ? 29:29)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 30:30) - (0 ?
 30:30) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 30:30) - (0 ?
 30:30) + 1))))))) << (0 ?
 30:30))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 30:30) - (0 ?
 30:30) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 30:30) - (0 ? 30:30) + 1))))))) << (0 ? 30:30)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:31) - (0 ?
 31:31) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:31) - (0 ?
 31:31) + 1))))))) << (0 ?
 31:31))) | (((jmtUINT32) ((jmtUINT32) (nn_layer_flush) & ((jmtUINT32) ((((1 ?
 31:31) - (0 ?
 31:31) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:31) - (0 ? 31:31) + 1))))))) << (0 ? 31:31))));


    config =   ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 18:6) - (0 ?
 18:6) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 18:6) - (0 ?
 18:6) + 1))))))) << (0 ?
 18:6))) | (((jmtUINT32) ((jmtUINT32) (inImageXSize) & ((jmtUINT32) ((((1 ?
 18:6) - (0 ?
 18:6) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 18:6) - (0 ? 18:6) + 1))))))) << (0 ? 18:6)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:19) - (0 ?
 31:19) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:19) - (0 ?
 31:19) + 1))))))) << (0 ?
 31:19))) | (((jmtUINT32) ((jmtUINT32) (inImageYSize) & ((jmtUINT32) ((((1 ?
 31:19) - (0 ?
 31:19) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:19) - (0 ? 31:19) + 1))))))) << (0 ? 31:19)))

                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 1:1) - (0 ?
 1:1) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 1:1) - (0 ?
 1:1) + 1))))))) << (0 ?
 1:1))) | (((jmtUINT32) ((jmtUINT32) (kernelDataType >> 1) & ((jmtUINT32) ((((1 ?
 1:1) - (0 ?
 1:1) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 1:1) - (0 ? 1:1) + 1))))))) << (0 ? 1:1)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 3:3) - (0 ?
 3:3) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 3:3) - (0 ?
 3:3) + 1))))))) << (0 ?
 3:3))) | (((jmtUINT32) ((jmtUINT32) (inImageDataType >> 1) & ((jmtUINT32) ((((1 ?
 3:3) - (0 ?
 3:3) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 3:3) - (0 ? 3:3) + 1))))))) << (0 ? 3:3)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 5:5) - (0 ?
 5:5) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 5:5) - (0 ?
 5:5) + 1))))))) << (0 ?
 5:5))) | (((jmtUINT32) ((jmtUINT32) (outImageDataType >> 1) & ((jmtUINT32) ((((1 ?
 5:5) - (0 ?
 5:5) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 5:5) - (0 ? 5:5) + 1))))))) << (0 ? 5:5)))

                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 0:0) - (0 ?
 0:0) + 1))))))) << (0 ?
 0:0))) | (((jmtUINT32) ((jmtUINT32) (kernelDataType & 0x1) & ((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 0:0) - (0 ? 0:0) + 1))))))) << (0 ? 0:0)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 2:2) - (0 ?
 2:2) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 2:2) - (0 ?
 2:2) + 1))))))) << (0 ?
 2:2))) | (((jmtUINT32) ((jmtUINT32) (inImageDataType & 0x1) & ((jmtUINT32) ((((1 ?
 2:2) - (0 ?
 2:2) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 2:2) - (0 ? 2:2) + 1))))))) << (0 ? 2:2)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:4) - (0 ?
 4:4) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:4) - (0 ?
 4:4) + 1))))))) << (0 ?
 4:4))) | (((jmtUINT32) ((jmtUINT32) (outImageDataType & 0x1) & ((jmtUINT32) ((((1 ?
 4:4) - (0 ?
 4:4) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:4) - (0 ? 4:4) + 1))))))) << (0 ? 4:4)));


    j9_uncoherentness(command, config);


    j9_uncoherentness(command,
                      ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 24:24) - (0 ?
 24:24) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 24:24) - (0 ?
 24:24) + 1))))))) << (0 ?
 24:24))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 24:24) - (0 ?
 24:24) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 24:24) - (0 ? 24:24) + 1))))))) << (0 ? 24:24)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:25) - (0 ?
 25:25) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:25) - (0 ?
 25:25) + 1))))))) << (0 ?
 25:25))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 25:25) - (0 ?
 25:25) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:25) - (0 ? 25:25) + 1))))))) << (0 ? 25:25)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 2:0) - (0 ?
 2:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 2:0) - (0 ?
 2:0) + 1))))))) << (0 ?
 2:0))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 2:0) - (0 ?
 2:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 2:0) - (0 ? 2:0) + 1))))))) << (0 ? 2:0)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 5:3) - (0 ?
 5:3) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 5:3) - (0 ?
 5:3) + 1))))))) << (0 ?
 5:3))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 5:3) - (0 ?
 5:3) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 5:3) - (0 ? 5:3) + 1))))))) << (0 ? 5:3)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 26:26) - (0 ?
 26:26) + 1))))))) << (0 ?
 26:26))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 26:26) - (0 ? 26:26) + 1))))))) << (0 ? 26:26)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 7:7) - (0 ?
 7:7) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 7:7) - (0 ?
 7:7) + 1))))))) << (0 ?
 7:7))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 7:7) - (0 ?
 7:7) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 7:7) - (0 ? 7:7) + 1))))))) << (0 ? 7:7)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 23:8) - (0 ?
 23:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 23:8) - (0 ?
 23:8) + 1))))))) << (0 ?
 23:8))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 23:8) - (0 ?
 23:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 23:8) - (0 ? 23:8) + 1))))))) << (0 ? 23:8)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) ((jmtUINT32) (post_shift) & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27))));


     j9_uncoherentness(command,
                      ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 18:6) - (0 ?
 18:6) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 18:6) - (0 ?
 18:6) + 1))))))) << (0 ?
 18:6))) | (((jmtUINT32) ((jmtUINT32) (outImageXSize) & ((jmtUINT32) ((((1 ?
 18:6) - (0 ?
 18:6) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 18:6) - (0 ? 18:6) + 1))))))) << (0 ? 18:6)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:19) - (0 ?
 31:19) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:19) - (0 ?
 31:19) + 1))))))) << (0 ?
 31:19))) | (((jmtUINT32) ((jmtUINT32) (outImageYSize) & ((jmtUINT32) ((((1 ?
 31:19) - (0 ?
 31:19) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:19) - (0 ? 31:19) + 1))))))) << (0 ? 31:19)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 2:2) - (0 ?
 2:2) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 2:2) - (0 ?
 2:2) + 1))))))) << (0 ?
 2:2))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 2:2) - (0 ?
 2:2) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 2:2) - (0 ? 2:2) + 1))))))) << (0 ? 2:2)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 3:3) - (0 ?
 3:3) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 3:3) - (0 ?
 3:3) + 1))))))) << (0 ?
 3:3))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 3:3) - (0 ?
 3:3) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 3:3) - (0 ? 3:3) + 1))))))) << (0 ? 3:3))));


    j9_uncoherentness(command,
                      ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 13:0) - (0 ?
 13:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 13:0) - (0 ?
 13:0) + 1))))))) << (0 ?
 13:0))) | (((jmtUINT32) ((jmtUINT32) (outImageZSize) & ((jmtUINT32) ((((1 ?
 13:0) - (0 ?
 13:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 13:0) - (0 ? 13:0) + 1))))))) << (0 ? 13:0)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:14) - (0 ?
 15:14) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:14) - (0 ?
 15:14) + 1))))))) << (0 ?
 15:14))) | (((jmtUINT32) ((jmtUINT32) (0x0) & ((jmtUINT32) ((((1 ?
 15:14) - (0 ?
 15:14) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:14) - (0 ? 15:14) + 1))))))) << (0 ? 15:14)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 24:18) - (0 ?
 24:18) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 24:18) - (0 ?
 24:18) + 1))))))) << (0 ?
 24:18))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 24:18) - (0 ?
 24:18) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 24:18) - (0 ? 24:18) + 1))))))) << (0 ? 24:18)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:25) - (0 ?
 31:25) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:25) - (0 ?
 31:25) + 1))))))) << (0 ?
 31:25))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 31:25) - (0 ?
 31:25) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:25) - (0 ? 31:25) + 1))))))) << (0 ? 31:25)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 16:16) - (0 ?
 16:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 16:16) - (0 ?
 16:16) + 1))))))) << (0 ?
 16:16))) | (((jmtUINT32) ((jmtUINT32) ((0 >> 3) & 0x1) & ((jmtUINT32) ((((1 ?
 16:16) - (0 ?
 16:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 16:16) - (0 ? 16:16) + 1))))))) << (0 ? 16:16)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 17:17) - (0 ?
 17:17) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 17:17) - (0 ?
 17:17) + 1))))))) << (0 ?
 17:17))) | (((jmtUINT32) ((jmtUINT32) ((0 >> 3) & 0x1) & ((jmtUINT32) ((((1 ?
 17:17) - (0 ?
 17:17) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 17:17) - (0 ? 17:17) + 1))))))) << (0 ? 17:17))));



    j9_uncoherentness(command,
                      ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:26) - (0 ?
 31:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:26) - (0 ?
 31:26) + 1))))))) << (0 ?
 31:26))) | (((jmtUINT32) ((jmtUINT32) (((kernelZSize >> 14) & 0x3F)) & ((jmtUINT32) ((((1 ?
 31:26) - (0 ?
 31:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:26) - (0 ? 31:26) + 1))))))) << (0 ? 31:26)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:0) - (0 ?
 25:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:0) - (0 ?
 25:0) + 1))))))) << (0 ?
 25:0))) | (((jmtUINT32) ((jmtUINT32) ((kernelAddress >> 6)) & ((jmtUINT32) ((((1 ?
 25:0) - (0 ?
 25:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:0) - (0 ? 25:0) + 1))))))) << (0 ? 25:0))));


    j9_uncoherentness(command, inImageAddress);

    j9_uncoherentness(command, outImageAddress);

    j9_uncoherentness(command,
    j9_uncoherentness(command, 0);


    j9_uncoherentness(command, 0);


    j9_uncoherentness(command, 0);


    j9_uncoherentness(command, 0);


    j9_uncoherentness(command, 0);


    j9_uncoherentness(command, imageEndAddress);



    j9_uncoherentness(command,
                      ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 1:0) - (0 ?
 1:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 1:0) - (0 ?
 1:0) + 1))))))) << (0 ?
 1:0))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 1:0) - (0 ?
 1:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 1:0) - (0 ? 1:0) + 1))))))) << (0 ? 1:0)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 17:2) - (0 ?
 17:2) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 17:2) - (0 ?
 17:2) + 1))))))) << (0 ?
 17:2))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 17:2) - (0 ?
 17:2) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 17:2) - (0 ? 17:2) + 1))))))) << (0 ? 17:2)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 19:19) - (0 ?
 19:19) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 19:19) - (0 ?
 19:19) + 1))))))) << (0 ?
 19:19))) | (((jmtUINT32) ((jmtUINT32) (kernelDataType >> 2) & ((jmtUINT32) ((((1 ?
 19:19) - (0 ?
 19:19) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 19:19) - (0 ? 19:19) + 1))))))) << (0 ? 19:19)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 20:20) - (0 ?
 20:20) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 20:20) - (0 ?
 20:20) + 1))))))) << (0 ?
 20:20))) | (((jmtUINT32) ((jmtUINT32) (inImageDataType >> 2) & ((jmtUINT32) ((((1 ?
 20:20) - (0 ?
 20:20) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 20:20) - (0 ? 20:20) + 1))))))) << (0 ? 20:20)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 21:21) - (0 ?
 21:21) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 21:21) - (0 ?
 21:21) + 1))))))) << (0 ?
 21:21))) | (((jmtUINT32) ((jmtUINT32) (outImageDataType >> 2) & ((jmtUINT32) ((((1 ?
 21:21) - (0 ?
 21:21) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 21:21) - (0 ? 21:21) + 1))))))) << (0 ? 21:21)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 27:22) - (0 ?
 27:22) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 27:22) - (0 ?
 27:22) + 1))))))) << (0 ?
 27:22))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 27:22) - (0 ?
 27:22) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 27:22) - (0 ? 27:22) + 1))))))) << (0 ? 27:22)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 29:28) - (0 ?
 29:28) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 29:28) - (0 ?
 29:28) + 1))))))) << (0 ?
 29:28))) | (((jmtUINT32) ((jmtUINT32) (post_shift_bit56) & ((jmtUINT32) ((((1 ?
 29:28) - (0 ?
 29:28) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 29:28) - (0 ? 29:28) + 1))))))) << (0 ? 29:28))));


    if (hw_type == USC_NN_TYPE_V7 || hw_type == USC_NN_TYPE_V8)
    {

        j9_uncoherentness(command,
                      ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (inImageXSize * (*item_size)) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:16) - (0 ?
 31:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:16) - (0 ?
 31:16) + 1))))))) << (0 ?
 31:16))) | (((jmtUINT32) ((jmtUINT32) (inImageYSize) & ((jmtUINT32) ((((1 ?
 31:16) - (0 ?
 31:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:16) - (0 ? 31:16) + 1))))))) << (0 ? 31:16))));


        j9_uncoherentness(command, ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (outImageXSize * (*item_size)) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:24) - (0 ?
 31:24) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:24) - (0 ?
 31:24) + 1))))))) << (0 ?
 31:24))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 31:24) - (0 ?
 31:24) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:24) - (0 ? 31:24) + 1))))))) << (0 ? 31:24))));



        j9_uncoherentness(command,
                      ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:0) - (0 ?
 25:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:0) - (0 ?
 25:0) + 1))))))) << (0 ?
 25:0))) | (((jmtUINT32) ((jmtUINT32) (0 >> 6) & ((jmtUINT32) ((((1 ?
 25:0) - (0 ?
 25:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:0) - (0 ? 25:0) + 1))))))) << (0 ? 25:0)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 26:26) - (0 ?
 26:26) + 1))))))) << (0 ?
 26:26))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 26:26) - (0 ? 26:26) + 1))))))) << (0 ? 26:26)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 29:29) - (0 ?
 29:29) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 29:29) - (0 ?
 29:29) + 1))))))) << (0 ?
 29:29))) | (((jmtUINT32) ((jmtUINT32) ((0 >> 4) & 0x1) & ((jmtUINT32) ((((1 ?
 29:29) - (0 ?
 29:29) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 29:29) - (0 ? 29:29) + 1))))))) << (0 ? 29:29)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 30:30) - (0 ?
 30:30) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 30:30) - (0 ?
 30:30) + 1))))))) << (0 ?
 30:30))) | (((jmtUINT32) ((jmtUINT32) ((0 >> 4) & 0x1) & ((jmtUINT32) ((((1 ?
 30:30) - (0 ?
 30:30) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 30:30) - (0 ? 30:30) + 1))))))) << (0 ? 30:30)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 28:28) - (0 ?
 28:28) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 28:28) - (0 ?
 28:28) + 1))))))) << (0 ?
 28:28))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 28:28) - (0 ?
 28:28) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 28:28) - (0 ? 28:28) + 1))))))) << (0 ? 28:28)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:31) - (0 ?
 31:31) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:31) - (0 ?
 31:31) + 1))))))) << (0 ?
 31:31))) | (((jmtUINT32) ((jmtUINT32) (kernelDataType >> 3) & ((jmtUINT32) ((((1 ?
 31:31) - (0 ?
 31:31) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:31) - (0 ? 31:31) + 1))))))) << (0 ? 31:31))));


         j9_uncoherentness(command,
                      ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:0) - (0 ?
 25:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:0) - (0 ?
 25:0) + 1))))))) << (0 ?
 25:0))) | (((jmtUINT32) ((jmtUINT32) (0xFFFFFFFF >> 6) & ((jmtUINT32) ((((1 ?
 25:0) - (0 ?
 25:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:0) - (0 ? 25:0) + 1))))))) << (0 ? 25:0)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 30:30) - (0 ?
 30:30) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 30:30) - (0 ?
 30:30) + 1))))))) << (0 ?
 30:30))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 30:30) - (0 ?
 30:30) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 30:30) - (0 ? 30:30) + 1))))))) << (0 ? 30:30)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:31) - (0 ?
 31:31) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:31) - (0 ?
 31:31) + 1))))))) << (0 ?
 31:31))) | (((jmtUINT32) ((jmtUINT32) (inImageDataType >> 3) & ((jmtUINT32) ((((1 ?
 31:31) - (0 ?
 31:31) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:31) - (0 ? 31:31) + 1))))))) << (0 ? 31:31))));


        j9_uncoherentness(command, ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:0) - (0 ?
 25:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:0) - (0 ?
 25:0) + 1))))))) << (0 ?
 25:0))) | (((jmtUINT32) ((jmtUINT32) (0 >> 6) & ((jmtUINT32) ((((1 ?
 25:0) - (0 ?
 25:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:0) - (0 ? 25:0) + 1))))))) << (0 ? 25:0)))
                        | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 26:26) - (0 ?
 26:26) + 1))))))) << (0 ?
 26:26))) | (((jmtUINT32) ((jmtUINT32) (outImageDataType >> 3) & ((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 26:26) - (0 ? 26:26) + 1))))))) << (0 ? 26:26))));


        j9_uncoherentness(command, ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:0) - (0 ?
 25:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:0) - (0 ?
 25:0) + 1))))))) << (0 ?
 25:0))) | (((jmtUINT32) ((jmtUINT32) (0xFFFFFFFF >> 6) & ((jmtUINT32) ((((1 ?
 25:0) - (0 ?
 25:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:0) - (0 ? 25:0) + 1))))))) << (0 ? 25:0))));


        j9_uncoherentness(command,
                      ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 7:0) - (0 ?
 7:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 7:0) - (0 ?
 7:0) + 1))))))) << (0 ?
 7:0))) | (((jmtUINT32) ((jmtUINT32) (coefZP) & ((jmtUINT32) ((((1 ?
 7:0) - (0 ?
 7:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 7:0) - (0 ? 7:0) + 1))))))) << (0 ? 7:0)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:8) - (0 ?
 15:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:8) - (0 ?
 15:8) + 1))))))) << (0 ?
 15:8))) | (((jmtUINT32) ((jmtUINT32) (outputZP) & ((jmtUINT32) ((((1 ?
 15:8) - (0 ?
 15:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:8) - (0 ? 15:8) + 1))))))) << (0 ? 15:8)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 16:16) - (0 ?
 16:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 16:16) - (0 ?
 16:16) + 1))))))) << (0 ?
 16:16))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 16:16) - (0 ?
 16:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 16:16) - (0 ? 16:16) + 1))))))) << (0 ? 16:16)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 17:17) - (0 ?
 17:17) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 17:17) - (0 ?
 17:17) + 1))))))) << (0 ?
 17:17))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 17:17) - (0 ?
 17:17) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 17:17) - (0 ? 17:17) + 1))))))) << (0 ? 17:17)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:18) - (0 ?
 25:18) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:18) - (0 ?
 25:18) + 1))))))) << (0 ?
 25:18))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 25:18) - (0 ?
 25:18) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:18) - (0 ? 25:18) + 1))))))) << (0 ? 25:18))));


        j9_uncoherentness(command, 0);


        j9_uncoherentness(command,
                      ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 3:0) - (0 ?
 3:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 3:0) - (0 ?
 3:0) + 1))))))) << (0 ?
 3:0))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 3:0) - (0 ?
 3:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 3:0) - (0 ? 3:0) + 1))))))) << (0 ? 3:0)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:4) - (0 ?
 31:4) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:4) - (0 ?
 31:4) + 1))))))) << (0 ?
 31:4))) | (((jmtUINT32) ((jmtUINT32) (0 >> 4) & ((jmtUINT32) ((((1 ?
 31:4) - (0 ?
 31:4) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:4) - (0 ? 31:4) + 1))))))) << (0 ? 31:4))));


        j9_uncoherentness(command,
                      ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 3:0) - (0 ?
 3:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 3:0) - (0 ?
 3:0) + 1))))))) << (0 ?
 3:0))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 3:0) - (0 ?
 3:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 3:0) - (0 ? 3:0) + 1))))))) << (0 ? 3:0)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:4) - (0 ?
 31:4) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:4) - (0 ?
 31:4) + 1))))))) << (0 ?
 31:4))) | (((jmtUINT32) ((jmtUINT32) (0 >> 4) & ((jmtUINT32) ((((1 ?
 31:4) - (0 ?
 31:4) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:4) - (0 ? 31:4) + 1))))))) << (0 ? 31:4))));
    }

OnError:
    return status;
}

j9_duopoly
j9_staxis(
    IN jmtUINT8_PTR* base,
    IN jmtUINT32 value,
    IN jmtUINT32_PTR offset,
    IN jmtUINT length)
{
    j9_duopoly status = J9_FLUTTERING;
    jmtUINT32_PTR msb = (jmtUINT32_PTR)(*base) + 1, lsb = (jmtUINT32_PTR)(*base);

    JMM_kASSERT(*offset <= 32 && length <= 32);

    if ((*offset) < 32)
    {
        jmtUINT32 end = (*offset) + length, data = *lsb;

        if (end < 32)
        {
            data  = (*lsb & ((1 << *offset) - 1));
            data |= (*lsb & ~((1 << end) - 1));
            data |= (value << *offset);

            *lsb = data;
            *offset = end;
        }
        else if (end < 64)
        {
            jmtUINT32 length_m = end - 32;
            jmtUINT32 data_l = (*lsb & ((1 << *offset) - 1));
            jmtUINT32 data_m = (*msb & ~((1 << length_m) - 1));

            data_l |= (value << *offset);
            data_m |= (value >> length_m);

            *lsb = data_l;

            if (end > 32)
                *msb = data_m;

            *offset = length_m;

            *base = (jmtUINT8_PTR)msb;
        }

    }

    return status;
}

j9_duopoly
_InitializeUSC_NNKernel(
    IN jmk_HARDWARE Hardware,
    IN USC_NN_TYPE hw_type,
    IN jmtUINT8 data_type,
    IN jmtUINT32 item_size,
    IN jmtUINT32 core_count,
    IN OUT jmtUINT32_PTR nnKernels
    )
{
    j9_duopoly status = J9_FLUTTERING;

    jmtUINT8_PTR kernels = (jmtUINT8_PTR)nnKernels;
    jmtUINT8_PTR kernel_stream_size_ptr = 0;
    jmtUINT32 filterTotalCount = 1, filterSize = 2 * 2 * 1 * item_size, biasSize = 4;


    if (USC_NN_TYPE_V8 == hw_type)
    {
        jmtUINT32 i = 0, offset = 0;

        jmtUINT8 rlt[][18] = {{0}, {1, 1, 0, 1}, {7, 1}, {0}, {3, 1, 0, 1}, {0}, {0}, {1, 1, 0, 1} };
        jmtUINT8 map[][9] = {
            {1, 8, 7, 0, 4, 5, 6, 2, 3},
            {1, 5, 0, 7, 8, 2, 6, 3, 4},
            {1, 0, 7, 8, 4, 5, 6, 2, 3},
        };
        jmtBOOL bit16 = (data_type == 0x4) || data_type == (0x1) || data_type == (0x7);
        jmtBOOL fp16 = (data_type == 0x1);
        jmtUINT32 index = (data_type == 0x1)?1:((data_type == 0x7)?2:0);

        j9_recaution(j9_staxis(&kernels, 0, &offset, 1));
        j9_recaution(j9_staxis(&kernels, bit16, &offset, 1));
        j9_recaution(j9_staxis(&kernels, fp16, &offset, 1));
        j9_recaution(j9_staxis(&kernels, 0, &offset, 1));
        j9_recaution(j9_staxis(&kernels, 1, &offset, 4));
        j9_recaution(j9_staxis(&kernels, 4, &offset, 8));

        for(i = 0; i < 18; i++)
            j9_recaution(j9_staxis(&kernels, rlt[data_type][i], &offset, 8));

        for(i = 0; i < 4; i++)
        {
            j9_recaution(j9_staxis(&kernels, (map[index][2 * i + 1] << 4) + map[index][2 * i], &offset, 8));
        }

        j9_recaution(j9_staxis(&kernels, 0, &offset, 16));

        j9_recaution(j9_staxis(&kernels, 0, &offset, 16));

        kernel_stream_size_ptr = kernels;
        for (i = 0; i < core_count; i ++)
            j9_recaution(j9_staxis(&kernels, 0, &offset, 32));

        kernels = (jmtUINT8_PTR)nnKernels + J9_UNFLEETING((jmtUINT32)((jmtUINT32_PTR)kernels - nnKernels), 64);

        switch (data_type)
        {
            case 0x4:
                j9_recaution(j9_staxis(&kernels, 0x04058000, &offset, 32));
                j9_recaution(j9_staxis(&kernels, 0x640101fc, &offset, 32));
                j9_recaution(j9_staxis(&kernels, 0x00001200, &offset, 32));

                j9_recaution(j9_staxis(&kernel_stream_size_ptr, 0x0000006d, &offset, 32));


                break;
            case 0x0:
            case 0x2:
                j9_recaution(j9_staxis(&kernels, 0xec000038, &offset, 32));

                j9_recaution(j9_staxis(&kernel_stream_size_ptr, 0x35, &offset, 32));

                break;
            case 0x1:
                j9_recaution(j9_staxis(&kernels, 0x0009db68, &offset, 32));
                j9_recaution(j9_staxis(&kernels, 0x000006c0, &offset, 32));
                j9_recaution(j9_staxis(&kernels, 0x000001f0, &offset, 32));
                j9_recaution(j9_staxis(&kernels, 0x00000900, &offset, 32));
                j9_recaution(j9_staxis(&kernels, 0x00024000, &offset, 32));

                j9_recaution(j9_staxis(&kernel_stream_size_ptr, 0x000000a3, &offset, 32));

                break;
            case 0x7:
                j9_recaution(j9_staxis(&kernels, 0x0007fff8, &offset, 32));
                j9_recaution(j9_staxis(&kernels, 0x7f00fdfc, &offset, 32));
                j9_recaution(j9_staxis(&kernels, 0xc0397f00, &offset, 32));
                j9_recaution(j9_staxis(&kernels, 0x0900001f, &offset, 32));
                j9_recaution(j9_staxis(&kernels, 0x40000000, &offset, 32));
                j9_recaution(j9_staxis(&kernels, 0x00000002, &offset, 32));

                j9_recaution(j9_staxis(&kernel_stream_size_ptr, 0x000000b2, &offset, 32));

                break;
            default:
                JMM_kASSERT("Huffman encode not support this format! Please check!");
                break;
        }


    }
    else
    {
        jmtBOOL zero_all = J9_YARELY;
        jmtUINT8 zrl = 0;
        jmtUINT16 vznum = 1;
        jmtUINT32 bias = 0;
        jmtUINT32 total_size = J9_UNFLEETING((filterTotalCount * (filterSize  +  biasSize +  3) + 3), 64);

        jmkOS_ZeroMemory(kernels, total_size + 64);

        *((jmtUINT32_PTR)kernels) = total_size;
        kernels += total_size;
        if (zero_all)
            *((jmtUINT32_PTR)kernels) = (vznum << (8 * item_size));
        else
#if 0
        {
            jmtUINT8 value = 1;
            *((jmtUINT32_PTR)kernels) = zrl | (vznum << (8 * item_size)) | (value << (3 * 8 * item_size));
            *((jmtUINT32_PTR)kernels + 1) = bias;
            *((jmtUINT32_PTR)kernels + 2) = value | (value << (1 * 8 * item_size)) | (value << (2 * 8 * item_size));
        }
#else
        {
            jmtINT16 value = (data_type == 0x1)?0x3c00:1;
            jmtUINT32 i = 0, offset = 0;

            j9_staxis(&kernels, zrl, &offset, 8);
            j9_staxis(&kernels, vznum, &offset, 16);
            j9_staxis(&kernels, value, &offset, 8 * item_size);
            j9_staxis(&kernels, bias, &offset, 32);
            if (data_type == 0x3 || data_type == 0x4)
                j9_staxis(&kernels, 0, &offset, 16);

            for (i = 1; i < filterSize/item_size; i ++)
                j9_staxis(&kernels, value, &offset, 8 * item_size);
        }

#endif
    }
OnError:
    return status;
}
#if 1
static j9_duopoly j9_handle_j9min_activation(IN jmk_HARDWARE    Hardware,
                                            IN USC_NN_TYPE    hw_type,
                                            IN jmtUINT32_PTR  flushCommands,
                                            IN jmtUINT32      CmdAddress,
                                            IN jmtUINT32      SramRemapAddress,
                                            OUT jmtUINT32_PTR Bytes)
{
    j9_duopoly status = J9_FLUTTERING;
    jmsFEATURE_DATABASE *db = (jmsFEATURE_DATABASE *)(Hardware->featureDatabase);
    jmtUINT32 idx = 0, kernel_brust_size = db->J9_HANDLE__APHIDICIDE;
    jmtINT32 disableZDPN = 1, disableSWTiling = 1, smallBatch = 1, ddrBurstSize = 0;
    jmtBOOL enableNNStride = J9_YARELY;

    disableZDPN = (db->J9_MOOP || db->J9_DEXY) ? 0 : 1;

    enableNNStride = db->J9_SUSPENDIBILITY;
    disableSWTiling = enableNNStride ? 0 : 1;

    switch(kernel_brust_size)
    {
    case 256:
        ddrBurstSize = 0x2;
        break;
    case 64:
        ddrBurstSize = 0x0;
        break;
    default:
        break;
    }

    if (Hardware->identity.chipModel == 0x8000 && Hardware->identity.j9_exquisite == 0x7120 &&
            (Hardware->identity.j9_nascent == 0x80 || Hardware->identity.j9_nascent == 0x92))
        smallBatch = 0x0;
    else
        smallBatch = (db->J9_HANDLE_INCULPATED && db->J9_HANDLE_J9MENU_ATTRIBUTE_DEMOLISHED)
                     ? 0x0 : 0x1;

    if (hw_type == USC_NN_TYPE_V6)
    {
        flushCommands[idx++] = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x006B) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)));
        flushCommands[idx++] = 0;

        flushCommands[idx++] = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x0E03) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)));
        flushCommands[idx++] = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 6:6) - (0 ?
 6:6) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 6:6) - (0 ?
 6:6) + 1))))))) << (0 ?
 6:6))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 6:6) - (0 ?
 6:6) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 6:6) - (0 ? 6:6) + 1))))))) << (0 ? 6:6)));
    }

    flushCommands[idx++] = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x0E4E) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)));
    flushCommands[idx++] = SramRemapAddress;

    flushCommands[idx++] = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x0E4F) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)));
    flushCommands[idx++] = 0x00000000;

    flushCommands[idx++] = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x0E50) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)));
    flushCommands[idx++] = 0x00000000;

    flushCommands[idx++] = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x0E03) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)));
    flushCommands[idx++] = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 6:6) - (0 ?
 6:6) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 6:6) - (0 ?
 6:6) + 1))))))) << (0 ?
 6:6))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 6:6) - (0 ?
 6:6) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 6:6) - (0 ? 6:6) + 1))))))) << (0 ? 6:6)));

    flushCommands[idx++] = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x0E4C) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)));

    flushCommands[idx++] = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 2:2) - (0 ?
 2:2) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 2:2) - (0 ?
 2:2) + 1))))))) << (0 ?
 2:2))) | (((jmtUINT32) ((jmtUINT32) (disableZDPN) & ((jmtUINT32) ((((1 ?
 2:2) - (0 ?
 2:2) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 2:2) - (0 ? 2:2) + 1))))))) << (0 ? 2:2)))
              |  ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 3:3) - (0 ?
 3:3) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 3:3) - (0 ?
 3:3) + 1))))))) << (0 ?
 3:3))) | (((jmtUINT32) ((jmtUINT32) (disableSWTiling) & ((jmtUINT32) ((((1 ?
 3:3) - (0 ?
 3:3) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 3:3) - (0 ? 3:3) + 1))))))) << (0 ? 3:3)))
              |  ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:4) - (0 ?
 4:4) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:4) - (0 ?
 4:4) + 1))))))) << (0 ?
 4:4))) | (((jmtUINT32) ((jmtUINT32) (smallBatch) & ((jmtUINT32) ((((1 ?
 4:4) - (0 ?
 4:4) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:4) - (0 ? 4:4) + 1))))))) << (0 ? 4:4)))
              |  ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 6:5) - (0 ?
 6:5) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 6:5) - (0 ?
 6:5) + 1))))))) << (0 ?
 6:5))) | (((jmtUINT32) ((jmtUINT32) (ddrBurstSize) & ((jmtUINT32) ((((1 ?
 6:5) - (0 ?
 6:5) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 6:5) - (0 ? 6:5) + 1))))))) << (0 ? 6:5)))
              |  ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 12:12) - (0 ?
 12:12) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 12:12) - (0 ?
 12:12) + 1))))))) << (0 ?
 12:12))) | (((jmtUINT32) ((jmtUINT32) (0x0) & ((jmtUINT32) ((((1 ?
 12:12) - (0 ?
 12:12) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 12:12) - (0 ? 12:12) + 1))))))) << (0 ? 12:12)));


    flushCommands[idx++] = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x0E54) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)));

    flushCommands[idx++] = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 0:0) - (0 ?
 0:0) + 1))))))) << (0 ?
 0:0))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 0:0) - (0 ? 0:0) + 1))))))) << (0 ? 0:0)))
          | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 1:1) - (0 ?
 1:1) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 1:1) - (0 ?
 1:1) + 1))))))) << (0 ?
 1:1))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 1:1) - (0 ?
 1:1) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 1:1) - (0 ? 1:1) + 1))))))) << (0 ? 1:1)));


    flushCommands[idx++] = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x0428) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)));

    flushCommands[idx++] = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:6) - (0 ?
 31:6) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:6) - (0 ?
 31:6) + 1))))))) << (0 ?
 31:6))) | (((jmtUINT32) ((jmtUINT32) ((CmdAddress >> 6)) & ((jmtUINT32) ((((1 ?
 31:6) - (0 ?
 31:6) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:6) - (0 ? 31:6) + 1))))))) << (0 ? 31:6)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:0) - (0 ?
 4:0) + 1))))))) << (0 ?
 4:0))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:0) - (0 ? 4:0) + 1))))))) << (0 ? 4:0)));

    flushCommands[idx++] = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x0429) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)));

    flushCommands[idx++] = 0;

    flushCommands[idx++] = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x0E03) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)));

    flushCommands[idx++] = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 5:5) - (0 ?
 5:5) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 5:5) - (0 ?
 5:5) + 1))))))) << (0 ?
 5:5))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 5:5) - (0 ?
 5:5) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 5:5) - (0 ? 5:5) + 1))))))) << (0 ? 5:5)));

    *Bytes = idx * 4;


    return status;
}
#endif
#endif

#endif




