/*
 * JMGPU driver
 *
 * Copyright (c) 2020 ChangSha JingJiaMicro Electronics Co., Ltd.
 * All rights reserved. 
 *
 * Author:
 *      wj <jjwgpu@jingjiamicro.com>
 * 
 * The software and information contained herein is proprietary and 
 * confidential to JingJiaMicro Electronics. This software can only be 
 * used by JingJiaMicro Electronics Corporation. Any use, reproduction, 
 * or disclosure without the written permission of JingJiaMicro 
 * Electronics Corporation is strictly prohibited.
 *
 */
 





#if J9MATHS_PROMULGING

#define J9_HORRIPILATING         1


#define J9_ABLEPHARON            0
#define J9_CENTRARCHID            1
#define J9_ANTHODIUM            2

#include "jmgpu_relational.h"

#define J9_HANDLE_J9MENU_ATTRIBUTE_COENOTYPIC ~0U

#define J9_HANDSMOOTH ((1 << 0) | (1 << 1) | (1 << 2) | (1 << 3))
#define J9_NONRESIDENT(X, Y, Z, W) ((1 << (X)) | (1 << (Y)) | (1 << (Z)) | (1 << (W)))
#define J9_CRYSTALITIC(X) (1 << (X))
#define J9_PRESURROUND(X, Y) ((1 << (X)) | (1 << (Y)))
#define J9_COPPERSMITH(X, Y, Z) ((1 << (X)) | (1 << (Y)) | (1 << (Z)))
#define J9_EMPIRICALLY (0 | (1 << 2) | (2 << 4) | (3 << 6))
#define J9_RESTRINGENCY(X) ((X) | ((X) << 2) | ((X) << 4) | ((X) << 6))
#define J9_ANTIFEMINIST(X, Y) ((X) | ((Y) << 2) | ((Y) << 4) | ((Y) << 6))
#define J9_RIDICULOUSLY(X, Y, Z, W) ((X) | ((Y) << 2) | ((Z) << 4) | ((W) << 6))


#define J9_DTB(data, position) (((data) >> (position)) & 0x1)

#define J9LPC_(data, position, value)   (\
            ((data) & (~((1ULL) << position)))   \
             |  \
             (((value) << position) & ((1ULL) << position))   \
             )

#define J9FLU_(reg_field)       (0 ? reg_field)
#define _END(reg_field)         (1 ? reg_field)
#define J9_QUIPO(reg_field)     (_END(reg_field) - J9FLU_(reg_field) + 1)
#define J9AES_(data, reg_field) ((jmtUINT32)(data) << J9FLU_(reg_field))
#define J9NYP(reg_field)        ((J9_QUIPO(reg_field) == 32) \
                                    ?  ~0 \
                                    : (jmtUINT32)(~((jmtUINT64)(~0) << J9_QUIPO(reg_field))))

#define J9_AVOIDER(data, reg, field, value) \
(\
    ((jmtUINT32)(data) & ~J9AES_(J9NYP(reg##_##field), reg##_##field)) \
        | \
    J9AES_((jmtUINT32)(value) & J9NYP(reg##_##field), reg##_##field) \
)

#define J9_PREINSINUATE(data, reg, field, value) \
(\
    ((jmtUINT32)(data) & ~J9AES_(J9NYP(reg##_##field), reg##_##field)) \
        | \
    J9AES_(reg##_##field##_##value & J9NYP(reg##_##field), reg##_##field) \
)

jmtUINT32 HwFunc_SETBITS(
    IN jmtUINT32 Data,
    IN const unsigned int Start,
    IN const unsigned int End,
    IN const jmtUINT32 Value
    )
{
    jmtUINT32 data = Data;
    if (End >= Start)
    {
        jmtUINT32 _Mask =  ((~0ULL >> (63 - End + Start)) << Start);
        data &= ~_Mask;
        data |= ((Value) << Start) & _Mask;
        return data;
    }
    else
    {
        jmtUINT32 _Mask =  ((~0ULL >> (63 - Start + End)) << End);
        data &= ~_Mask;
        data |= ((Value) << End) & _Mask;
        return data;
    }
}

jmtUINT32 HwFunc_GETBITS(
    IN jmtUINT32 Data,
    IN const unsigned int Start,
    IN const unsigned int End
    )
{
    jmtUINT32 data = Data;
    if (End >= Start)
    {
        jmtUINT32 _Mask = (~0ULL >> (63 - (End - Start)));
        return (data >> Start) & _Mask;
    }
    else
    {
        jmtUINT32 _Mask = (~0ULL >> (63 - (Start - End)));
        return (data >> End) & _Mask;;
    }
}

j9_duopoly
_InitializePPU_SetImmediate(
    IN jmtUINT32                            Where,
    IN jmtUINT32                            Value,
    IN jmtUINT32                            Type,
    IN OUT jmtUINT32_PTR                        binary
    )
{
    j9_duopoly status = J9_FLUTTERING;

    j9_tympanichord("Where=0x%x", Where);

    switch (Where)
    {
        case 0:
            binary[1] = (((jmtUINT32)(binary[1]) & ~((jmtUINT32)(((((1 ?
 20:12) - (0 ?
 20:12) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 20:12) - (0 ?
 20:12) + 1))))) << (0 ?
 20:12))) | ((jmtUINT32)((jmtUINT32)(HwFunc_GETBITS(Value, 8, 0)) & ((((1 ?
 20:12) - (0 ?
 20:12) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 20:12) - (0 ? 20:12) + 1))))) << (0 ? 20:12)));
            binary[1] = (((jmtUINT32)(binary[1]) & ~((jmtUINT32)(((((1 ?
 29:22) - (0 ?
 29:22) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 29:22) - (0 ?
 29:22) + 1))))) << (0 ?
 29:22))) | ((jmtUINT32)((jmtUINT32)(HwFunc_GETBITS(Value, 16, 9)) & ((((1 ?
 29:22) - (0 ?
 29:22) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 29:22) - (0 ? 29:22) + 1))))) << (0 ? 29:22)));
            binary[1] = (((jmtUINT32)(binary[1]) & ~((jmtUINT32)(((((1 ?
 30:30) - (0 ?
 30:30) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 30:30) - (0 ?
 30:30) + 1))))) << (0 ?
 30:30))) | ((jmtUINT32)((jmtUINT32)(J9_DTB(Value, 17)) & ((((1 ?
 30:30) - (0 ?
 30:30) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 30:30) - (0 ? 30:30) + 1))))) << (0 ? 30:30)));
            binary[1] = (((jmtUINT32)(binary[1]) & ~((jmtUINT32)(((((1 ?
 31:31) - (0 ?
 31:31) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 31:31) - (0 ?
 31:31) + 1))))) << (0 ?
 31:31))) | ((jmtUINT32)((jmtUINT32)(J9_DTB(Value, 18)) & ((((1 ?
 31:31) - (0 ?
 31:31) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 31:31) - (0 ? 31:31) + 1))))) << (0 ? 31:31)));
            binary[2] = (((jmtUINT32)(binary[2]) & ~((jmtUINT32)(((((1 ?
 2:0) - (0 ?
 2:0) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 2:0) - (0 ?
 2:0) + 1))))) << (0 ?
 2:0))) | ((jmtUINT32)((jmtUINT32)(J9_DTB(Value, 19) | (Type << 1)) & ((((1 ?
 2:0) - (0 ?
 2:0) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 2:0) - (0 ? 2:0) + 1))))) << (0 ? 2:0)));
            binary[2] = (((jmtUINT32)(binary[2]) & ~((jmtUINT32)(((((1 ?
 5:3) - (0 ?
 5:3) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 5:3) - (0 ?
 5:3) + 1))))) << (0 ?
 5:3))) | ((jmtUINT32)((jmtUINT32)(0x7) & ((((1 ?
 5:3) - (0 ?
 5:3) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 5:3) - (0 ? 5:3) + 1))))) << (0 ? 5:3)));
            break;

        case 1:
            binary[2] = (((jmtUINT32)(binary[2]) & ~((jmtUINT32)(((((1 ?
 15:7) - (0 ?
 15:7) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 15:7) - (0 ?
 15:7) + 1))))) << (0 ?
 15:7))) | ((jmtUINT32)((jmtUINT32)(HwFunc_GETBITS(Value, 8, 0)) & ((((1 ?
 15:7) - (0 ?
 15:7) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 15:7) - (0 ? 15:7) + 1))))) << (0 ? 15:7)));
            binary[2] = (((jmtUINT32)(binary[2]) & ~((jmtUINT32)(((((1 ?
 24:17) - (0 ?
 24:17) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 24:17) - (0 ?
 24:17) + 1))))) << (0 ?
 24:17))) | ((jmtUINT32)((jmtUINT32)(HwFunc_GETBITS(Value, 16, 9)) & ((((1 ?
 24:17) - (0 ?
 24:17) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 24:17) - (0 ? 24:17) + 1))))) << (0 ? 24:17)));
            binary[2] = (((jmtUINT32)(binary[2]) & ~((jmtUINT32)(((((1 ?
 25:25) - (0 ?
 25:25) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 25:25) - (0 ?
 25:25) + 1))))) << (0 ?
 25:25))) | ((jmtUINT32)((jmtUINT32)(J9_DTB(Value, 17)) & ((((1 ?
 25:25) - (0 ?
 25:25) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 25:25) - (0 ? 25:25) + 1))))) << (0 ? 25:25)));
            binary[2] = (((jmtUINT32)(binary[2]) & ~((jmtUINT32)(((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 26:26) - (0 ?
 26:26) + 1))))) << (0 ?
 26:26))) | ((jmtUINT32)((jmtUINT32)(J9_DTB(Value, 18)) & ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 26:26) - (0 ? 26:26) + 1))))) << (0 ? 26:26)));
            binary[2] = (((jmtUINT32)(binary[2]) & ~((jmtUINT32)(((((1 ?
 29:27) - (0 ?
 29:27) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 29:27) - (0 ?
 29:27) + 1))))) << (0 ?
 29:27))) | ((jmtUINT32)((jmtUINT32)(J9_DTB(Value, 19) | (Type << 1)) & ((((1 ?
 29:27) - (0 ?
 29:27) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 29:27) - (0 ? 29:27) + 1))))) << (0 ? 29:27)));
            binary[3] = (((jmtUINT32)(binary[3]) & ~((jmtUINT32)(((((1 ?
 2:0) - (0 ?
 2:0) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 2:0) - (0 ?
 2:0) + 1))))) << (0 ?
 2:0))) | ((jmtUINT32)((jmtUINT32)(0x7) & ((((1 ?
 2:0) - (0 ?
 2:0) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 2:0) - (0 ? 2:0) + 1))))) << (0 ? 2:0)));
            break;

        case 2:
            binary[3] = (((jmtUINT32)(binary[3]) & ~((jmtUINT32)(((((1 ?
 12:4) - (0 ?
 12:4) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 12:4) - (0 ?
 12:4) + 1))))) << (0 ?
 12:4))) | ((jmtUINT32)((jmtUINT32)(HwFunc_GETBITS(Value, 8, 0)) & ((((1 ?
 12:4) - (0 ?
 12:4) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 12:4) - (0 ? 12:4) + 1))))) << (0 ? 12:4)));
            binary[3] = (((jmtUINT32)(binary[3]) & ~((jmtUINT32)(((((1 ?
 21:14) - (0 ?
 21:14) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 21:14) - (0 ?
 21:14) + 1))))) << (0 ?
 21:14))) | ((jmtUINT32)((jmtUINT32)(HwFunc_GETBITS(Value, 16, 9)) & ((((1 ?
 21:14) - (0 ?
 21:14) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 21:14) - (0 ? 21:14) + 1))))) << (0 ? 21:14)));
            binary[3] = (((jmtUINT32)(binary[3]) & ~((jmtUINT32)(((((1 ?
 22:22) - (0 ?
 22:22) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 22:22) - (0 ?
 22:22) + 1))))) << (0 ?
 22:22))) | ((jmtUINT32)((jmtUINT32)(J9_DTB(Value, 17)) & ((((1 ?
 22:22) - (0 ?
 22:22) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 22:22) - (0 ? 22:22) + 1))))) << (0 ? 22:22)));
            binary[3] = (((jmtUINT32)(binary[3]) & ~((jmtUINT32)(((((1 ?
 23:23) - (0 ?
 23:23) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 23:23) - (0 ?
 23:23) + 1))))) << (0 ?
 23:23))) | ((jmtUINT32)((jmtUINT32)(J9_DTB(Value, 18)) & ((((1 ?
 23:23) - (0 ?
 23:23) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 23:23) - (0 ? 23:23) + 1))))) << (0 ? 23:23)));
            binary[3] = (((jmtUINT32)(binary[3]) & ~((jmtUINT32)(((((1 ?
 27:25) - (0 ?
 27:25) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 27:25) - (0 ?
 27:25) + 1))))) << (0 ?
 27:25))) | ((jmtUINT32)((jmtUINT32)(J9_DTB(Value, 19) | (Type << 1)) & ((((1 ?
 27:25) - (0 ?
 27:25) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 27:25) - (0 ? 27:25) + 1))))) << (0 ? 27:25)));
            binary[3] = (((jmtUINT32)(binary[3]) & ~((jmtUINT32)(((((1 ?
 30:28) - (0 ?
 30:28) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 30:28) - (0 ?
 30:28) + 1))))) << (0 ?
 30:28))) | ((jmtUINT32)((jmtUINT32)(0x7) & ((((1 ?
 30:28) - (0 ?
 30:28) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 30:28) - (0 ? 30:28) + 1))))) << (0 ? 30:28)));
            break;
    }

    JMM_kFOOTER();
    return status;
}

j9_duopoly
_InitializePPU_SetInstructionType(
    IN jmtUINT32                            Type,
    OUT jmtUINT32_PTR                        binary
    )
{
    j9_duopoly status = J9_FLUTTERING;

    j9_tympanichord("Type=0x%x", Type);

    binary[1] = (((jmtUINT32)(binary[1]) & ~((jmtUINT32)(((((1 ?
 21:21) - (0 ?
 21:21) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 21:21) - (0 ?
 21:21) + 1))))) << (0 ?
 21:21))) | ((jmtUINT32)((jmtUINT32)(J9_DTB(Type, 0)) & ((((1 ?
 21:21) - (0 ?
 21:21) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 21:21) - (0 ? 21:21) + 1))))) << (0 ? 21:21)));
    binary[2] = (((jmtUINT32)(binary[2]) & ~((jmtUINT32)(((((1 ?
 31:30) - (0 ?
 31:30) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 31:30) - (0 ?
 31:30) + 1))))) << (0 ?
 31:30))) | ((jmtUINT32)((jmtUINT32)(HwFunc_GETBITS(Type, 2, 1)) & ((((1 ?
 31:30) - (0 ?
 31:30) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 31:30) - (0 ? 31:30) + 1))))) << (0 ? 31:30)));

    JMM_kFOOTER();
    return status;
}


j9_duopoly
_InitializePPU_IsEndOfBB(
    IN jmk_HARDWARE                            Hardware,
    IN jmtUINT32                            Opcode,
    OUT jmtUINT32_PTR                        binary
)
{
    j9_duopoly status = J9_FLUTTERING;
    j9_tympanichord("Opcode=0x%x", Opcode);

    if (binary != NULL)
    {
        if (((jmsFEATURE_DATABASE *)Hardware->featureDatabase)->J9_LIPPITUDO)
        {
            switch (Opcode)
            {
            case 0x09:
            case 0x56:
            case 0x0A:
            case 0x0B:
            case 0x0F:
            case 0x31:
            case 0x10:
                binary[1] = (((jmtUINT32)(binary[1]) & ~((jmtUINT32)(((((1 ?
 10:3) - (0 ?
 10:3) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 10:3) - (0 ?
 10:3) + 1))))) << (0 ?
 10:3))) | ((jmtUINT32)((jmtUINT32)(HwFunc_SETBITS(binary[1], 3, 3, 1)) & ((((1 ?
 10:3) - (0 ?
 10:3) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 10:3) - (0 ? 10:3) + 1))))) << (0 ? 10:3)));
                break;
            case 0x65:
            case 0x66:
            case 0x67:
            case 0x68:
            case 0x69:
            case 0x6A:
            case 0x6B:
            case 0x6C:
            case 0x46:
                binary[0] = (((jmtUINT32)(binary[0]) & ~((jmtUINT32)(((((1 ?
 10:6) - (0 ?
 10:6) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 10:6) - (0 ?
 10:6) + 1))))) << (0 ?
 10:6))) | ((jmtUINT32)((jmtUINT32)(HwFunc_SETBITS(binary[0], 2, 2, 1)) & ((((1 ?
 10:6) - (0 ?
 10:6) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 10:6) - (0 ? 10:6) + 1))))) << (0 ? 10:6)));
                break;
            case 0x32:
            case 0x39:
            case 0x33:
            case 0x3A:
            case 0x79:
            case 0x34:
            case 0x7A:
            case 0x35:
                binary[0] = (((jmtUINT32)(binary[0]) & ~((jmtUINT32)(((((1 ?
 10:6) - (0 ?
 10:6) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 10:6) - (0 ?
 10:6) + 1))))) << (0 ?
 10:6))) | ((jmtUINT32)((jmtUINT32)(HwFunc_SETBITS(binary[0], 2, 2, 1)) & ((((1 ?
 10:6) - (0 ?
 10:6) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 10:6) - (0 ? 10:6) + 1))))) << (0 ? 10:6)));
                break;
            default:
                if (Opcode != 0x16 &&
                    Opcode != 0x24 &&
                    Opcode != 0x14 &&
                    Opcode != 0x15 &&
                    Opcode != 0x17)
                    binary[0] = (((jmtUINT32)(binary[0]) & ~((jmtUINT32)(((((1 ?
 10:6) - (0 ?
 10:6) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 10:6) - (0 ?
 10:6) + 1))))) << (0 ?
 10:6))) | ((jmtUINT32)((jmtUINT32)(HwFunc_SETBITS(binary[0], 2, 2, 1)) & ((((1 ?
 10:6) - (0 ?
 10:6) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 10:6) - (0 ? 10:6) + 1))))) << (0 ? 10:6)));
                break;
            }
        }
    }



    JMM_kFOOTER();
    return status;
}
j9_duopoly
_InitializePPU_AddOpcode(
    IN jmk_HARDWARE                            Hardware,
    IN jmtUINT32                            Opcode,
    IN jmtUINT32                            Extended,
    IN jmtINT32                             Type,
    OUT jmtUINT32_PTR                        binary
    )
{
    j9_duopoly status = J9_FLUTTERING;

    j9_tympanichord("Opcode=0x%x", Opcode);

    binary[0] = (((jmtUINT32)(binary[0]) & ~((jmtUINT32)(((((1 ?
 5:0) - (0 ?
 5:0) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 5:0) - (0 ?
 5:0) + 1))))) << (0 ?
 5:0))) | ((jmtUINT32)((jmtUINT32)(HwFunc_GETBITS(Opcode, 5, 0)) & ((((1 ?
 5:0) - (0 ?
 5:0) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 5:0) - (0 ? 5:0) + 1))))) << (0 ? 5:0)));
    binary[2] = (((jmtUINT32)(binary[2]) & ~((jmtUINT32)(((((1 ?
 16:16) - (0 ?
 16:16) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 16:16) - (0 ?
 16:16) + 1))))) << (0 ?
 16:16))) | ((jmtUINT32)((jmtUINT32)(J9_DTB(Opcode, 6)) & ((((1 ?
 16:16) - (0 ?
 16:16) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 16:16) - (0 ? 16:16) + 1))))) << (0 ? 16:16)));
    if (Opcode == 0x7F)
    {
        j9_recaution(_InitializePPU_SetImmediate(2, Extended, 0x2, binary));
    }
    else if (Opcode == 0x45)
    {
        binary[0] = (((jmtUINT32)(binary[0]) & ~((jmtUINT32)(((((1 ?
 15:13) - (0 ?
 15:13) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 15:13) - (0 ?
 15:13) + 1))))) << (0 ?
 15:13))) | ((jmtUINT32)((jmtUINT32)(HwFunc_GETBITS(Extended, 2, 0)) & ((((1 ?
 15:13) - (0 ?
 15:13) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 15:13) - (0 ? 15:13) + 1))))) << (0 ? 15:13)));
        binary[0] = J9LPC_(binary[0], 31, J9_DTB(Extended, 3));
        binary[1] = HwFunc_SETBITS(binary[1], 1, 0, HwFunc_GETBITS(Extended, 5, 4));
    }
    else if (Opcode == 0x31 || Opcode == 0x09 || Opcode == 0x0F)
    {
        binary[0] = (((jmtUINT32)(binary[0]) & ~((jmtUINT32)(((((1 ?
 10:6) - (0 ?
 10:6) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 10:6) - (0 ?
 10:6) + 1))))) << (0 ?
 10:6))) | ((jmtUINT32)((jmtUINT32)(HwFunc_GETBITS(Extended, 4, 0)) & ((((1 ?
 10:6) - (0 ?
 10:6) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 10:6) - (0 ? 10:6) + 1))))) << (0 ? 10:6)));
    }

    if((jmtUINT32)Type != J9_HANDLE_J9MENU_ATTRIBUTE_COENOTYPIC)
        j9_recaution(_InitializePPU_SetInstructionType(Type, binary));

    j9_recaution(_InitializePPU_IsEndOfBB(Hardware, Opcode, binary));

OnError:

    JMM_kFOOTER();
    return status;
}

j9_duopoly
_InitializePPU_SetDestination(
    IN jmtUINT32                            Address,
    IN jmtUINT32                            Enable,
    IN jmtBOOL                              Saturate,
    IN OUT jmtUINT32_PTR                    binary
    )
{
    j9_duopoly status = J9_FLUTTERING;

    j9_tympanichord("Address=0x%x", Address);

    binary[0] = (((jmtUINT32)(binary[0]) & ~((jmtUINT32)(((((1 ?
 12:12) - (0 ?
 12:12) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 12:12) - (0 ?
 12:12) + 1))))) << (0 ?
 12:12))) | ((jmtUINT32)((jmtUINT32)(1) & ((((1 ?
 12:12) - (0 ?
 12:12) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 12:12) - (0 ? 12:12) + 1))))) << (0 ? 12:12)));
    binary[0] = (((jmtUINT32)(binary[0]) & ~((jmtUINT32)(((((1 ?
 22:16) - (0 ?
 22:16) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 22:16) - (0 ?
 22:16) + 1))))) << (0 ?
 22:16))) | ((jmtUINT32)((jmtUINT32)(Address) & ((((1 ?
 22:16) - (0 ?
 22:16) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 22:16) - (0 ? 22:16) + 1))))) << (0 ? 22:16)));
    binary[0] = (((jmtUINT32)(binary[0]) & ~((jmtUINT32)(((((1 ?
 26:23) - (0 ?
 26:23) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 26:23) - (0 ?
 26:23) + 1))))) << (0 ?
 26:23))) | ((jmtUINT32)((jmtUINT32)(Enable) & ((((1 ?
 26:23) - (0 ?
 26:23) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 26:23) - (0 ? 26:23) + 1))))) << (0 ? 26:23)));
    binary[0] = (((jmtUINT32)(binary[0]) & ~((jmtUINT32)(((((1 ?
 11:11) - (0 ?
 11:11) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 11:11) - (0 ?
 11:11) + 1))))) << (0 ?
 11:11))) | ((jmtUINT32)((jmtUINT32)(Saturate) & ((((1 ?
 11:11) - (0 ?
 11:11) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 11:11) - (0 ? 11:11) + 1))))) << (0 ? 11:11)));

    JMM_kFOOTER();
    return status;
}

#define J9_HANDSMOOTH ((1 << 0) | (1 << 1) | (1 << 2) | (1 << 3))

static jmtUINT32 j9_handle_j9_flimsilyst(jmtUINT32 format)
{
    jmtUINT32 pixel = 0;
    switch(format)
    {
    case 0x7:
        pixel = 15;
        break;
    case 0x3:
    case 0x6:
        pixel = 7;
        break;
    default:
        pixel = 15;
        break;
    }

    return pixel;
}

j9_duopoly
_InitializePPU_SetEVIS(
    IN jmtUINT32                            Start,
    IN jmtUINT32                            End,
    IN jmtUINT32                            Evis,
    IN OUT jmtUINT32_PTR                    binary
    )
{
    j9_duopoly status = J9_FLUTTERING;

    j9_tympanichord("Evis=0x%x", Evis);

    binary[0] = (((jmtUINT32)(binary[0]) & ~((jmtUINT32)(((((1 ?
 26:23) - (0 ?
 26:23) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 26:23) - (0 ?
 26:23) + 1))))) << (0 ?
 26:23))) | ((jmtUINT32)((jmtUINT32)(Start) & ((((1 ?
 26:23) - (0 ?
 26:23) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 26:23) - (0 ? 26:23) + 1))))) << (0 ? 26:23)));
    binary[0] = HwFunc_SETBITS(binary[0], 30, 27, End);
    binary[1] = HwFunc_SETBITS(binary[1], 10, 2, Evis);

    JMM_kFOOTER();
    return status;
}

#define j9_subsecurities(Where, Common_I, REL_ADR_I, TYPE_I, Address, Swizzle, Type, Negate, Absolute, Relative, Binary) \
    binary[Common_I] = J9_AVOIDER(binary[Common_I], AQ_INST, SRCWhere_VALID, 1); \
            binary[Common_I] = J9_AVOIDER(binary[Common_I], AQ_INST, SRC0_ADR, Address); \
            binary[Common_I] = J9_AVOIDER(binary[Common_I], AQ_INST, SRC0_SWIZZLE, Swizzle); \
            binary[Common_I] = J9_AVOIDER(binary[Common_I], AQ_INST, SRC0_MODIFIER_NEG, Negate); \
            binary[Common_I] = J9_AVOIDER(binary[Common_I], AQ_INST, SRC0_MODIFIER_ABS, Absolute); \
            binary[REL_ADR_I] = J9_AVOIDER(binary[REL_ADR_I], AQ_INST, SRC0_REL_ADR, Relative); \
            binary[TYPE_I] = J9_AVOIDER(binary[TYPE_I], AQ_INST, SRC0_TYPE, Type);

j9_duopoly
_InitializePPU_SetSource(
    IN jmtUINT32                            Where,
    IN jmtUINT32                            Address,
    IN jmtUINT32                            Swizzle,
    IN jmtUINT32                            Type,
    IN jmtBOOL                              Negate,
    IN jmtBOOL                              Absolute,
    IN jmtUINT32                            Relative,
    OUT jmtUINT32_PTR                        binary
    )
{
    j9_duopoly status = J9_FLUTTERING;

    j9_tympanichord("Where=0x%x", Where);
#if 1
    switch (Where)
    {
        case 0:
            binary[1] = (((jmtUINT32)(binary[1]) & ~((jmtUINT32)(((((1 ?
 11:11) - (0 ?
 11:11) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 11:11) - (0 ?
 11:11) + 1))))) << (0 ?
 11:11))) | ((jmtUINT32)((jmtUINT32)(1) & ((((1 ?
 11:11) - (0 ?
 11:11) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 11:11) - (0 ? 11:11) + 1))))) << (0 ? 11:11)));
            binary[1] = (((jmtUINT32)(binary[1]) & ~((jmtUINT32)(((((1 ?
 20:12) - (0 ?
 20:12) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 20:12) - (0 ?
 20:12) + 1))))) << (0 ?
 20:12))) | ((jmtUINT32)((jmtUINT32)(Address) & ((((1 ?
 20:12) - (0 ?
 20:12) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 20:12) - (0 ? 20:12) + 1))))) << (0 ? 20:12)));
            binary[1] = (((jmtUINT32)(binary[1]) & ~((jmtUINT32)(((((1 ?
 29:22) - (0 ?
 29:22) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 29:22) - (0 ?
 29:22) + 1))))) << (0 ?
 29:22))) | ((jmtUINT32)((jmtUINT32)(Swizzle) & ((((1 ?
 29:22) - (0 ?
 29:22) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 29:22) - (0 ? 29:22) + 1))))) << (0 ? 29:22)));
            binary[1] = (((jmtUINT32)(binary[1]) & ~((jmtUINT32)(((((1 ?
 30:30) - (0 ?
 30:30) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 30:30) - (0 ?
 30:30) + 1))))) << (0 ?
 30:30))) | ((jmtUINT32)((jmtUINT32)(Negate) & ((((1 ?
 30:30) - (0 ?
 30:30) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 30:30) - (0 ? 30:30) + 1))))) << (0 ? 30:30)));
            binary[1] = (((jmtUINT32)(binary[1]) & ~((jmtUINT32)(((((1 ?
 31:31) - (0 ?
 31:31) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 31:31) - (0 ?
 31:31) + 1))))) << (0 ?
 31:31))) | ((jmtUINT32)((jmtUINT32)(Absolute) & ((((1 ?
 31:31) - (0 ?
 31:31) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 31:31) - (0 ? 31:31) + 1))))) << (0 ? 31:31)));
            binary[2] = (((jmtUINT32)(binary[2]) & ~((jmtUINT32)(((((1 ?
 2:0) - (0 ?
 2:0) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 2:0) - (0 ?
 2:0) + 1))))) << (0 ?
 2:0))) | ((jmtUINT32)((jmtUINT32)(Relative) & ((((1 ?
 2:0) - (0 ?
 2:0) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 2:0) - (0 ? 2:0) + 1))))) << (0 ? 2:0)));
            binary[2] = (((jmtUINT32)(binary[2]) & ~((jmtUINT32)(((((1 ?
 5:3) - (0 ?
 5:3) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 5:3) - (0 ?
 5:3) + 1))))) << (0 ?
 5:3))) | ((jmtUINT32)((jmtUINT32)(Type) & ((((1 ?
 5:3) - (0 ?
 5:3) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 5:3) - (0 ? 5:3) + 1))))) << (0 ? 5:3)));
            break;

        case 1:
            binary[2] = (((jmtUINT32)(binary[2]) & ~((jmtUINT32)(((((1 ?
 6:6) - (0 ?
 6:6) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 6:6) - (0 ?
 6:6) + 1))))) << (0 ?
 6:6))) | ((jmtUINT32)((jmtUINT32)(1) & ((((1 ?
 6:6) - (0 ?
 6:6) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 6:6) - (0 ? 6:6) + 1))))) << (0 ? 6:6)));
            binary[2] = (((jmtUINT32)(binary[2]) & ~((jmtUINT32)(((((1 ?
 15:7) - (0 ?
 15:7) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 15:7) - (0 ?
 15:7) + 1))))) << (0 ?
 15:7))) | ((jmtUINT32)((jmtUINT32)(Address) & ((((1 ?
 15:7) - (0 ?
 15:7) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 15:7) - (0 ? 15:7) + 1))))) << (0 ? 15:7)));
            binary[2] = (((jmtUINT32)(binary[2]) & ~((jmtUINT32)(((((1 ?
 24:17) - (0 ?
 24:17) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 24:17) - (0 ?
 24:17) + 1))))) << (0 ?
 24:17))) | ((jmtUINT32)((jmtUINT32)(Swizzle) & ((((1 ?
 24:17) - (0 ?
 24:17) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 24:17) - (0 ? 24:17) + 1))))) << (0 ? 24:17)));
            binary[2] = (((jmtUINT32)(binary[2]) & ~((jmtUINT32)(((((1 ?
 25:25) - (0 ?
 25:25) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 25:25) - (0 ?
 25:25) + 1))))) << (0 ?
 25:25))) | ((jmtUINT32)((jmtUINT32)(Negate) & ((((1 ?
 25:25) - (0 ?
 25:25) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 25:25) - (0 ? 25:25) + 1))))) << (0 ? 25:25)));
            binary[2] = (((jmtUINT32)(binary[2]) & ~((jmtUINT32)(((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 26:26) - (0 ?
 26:26) + 1))))) << (0 ?
 26:26))) | ((jmtUINT32)((jmtUINT32)(Absolute) & ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 26:26) - (0 ? 26:26) + 1))))) << (0 ? 26:26)));
            binary[2] = (((jmtUINT32)(binary[2]) & ~((jmtUINT32)(((((1 ?
 29:27) - (0 ?
 29:27) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 29:27) - (0 ?
 29:27) + 1))))) << (0 ?
 29:27))) | ((jmtUINT32)((jmtUINT32)(Relative) & ((((1 ?
 29:27) - (0 ?
 29:27) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 29:27) - (0 ? 29:27) + 1))))) << (0 ? 29:27)));
            binary[3] = (((jmtUINT32)(binary[3]) & ~((jmtUINT32)(((((1 ?
 2:0) - (0 ?
 2:0) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 2:0) - (0 ?
 2:0) + 1))))) << (0 ?
 2:0))) | ((jmtUINT32)((jmtUINT32)(Type) & ((((1 ?
 2:0) - (0 ?
 2:0) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 2:0) - (0 ? 2:0) + 1))))) << (0 ? 2:0)));
            break;

        case 2:
            binary[3] = (((jmtUINT32)(binary[3]) & ~((jmtUINT32)(((((1 ?
 3:3) - (0 ?
 3:3) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 3:3) - (0 ?
 3:3) + 1))))) << (0 ?
 3:3))) | ((jmtUINT32)((jmtUINT32)(1) & ((((1 ?
 3:3) - (0 ?
 3:3) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 3:3) - (0 ? 3:3) + 1))))) << (0 ? 3:3)));
            binary[3] = (((jmtUINT32)(binary[3]) & ~((jmtUINT32)(((((1 ?
 12:4) - (0 ?
 12:4) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 12:4) - (0 ?
 12:4) + 1))))) << (0 ?
 12:4))) | ((jmtUINT32)((jmtUINT32)(Address) & ((((1 ?
 12:4) - (0 ?
 12:4) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 12:4) - (0 ? 12:4) + 1))))) << (0 ? 12:4)));
            binary[3] = (((jmtUINT32)(binary[3]) & ~((jmtUINT32)(((((1 ?
 21:14) - (0 ?
 21:14) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 21:14) - (0 ?
 21:14) + 1))))) << (0 ?
 21:14))) | ((jmtUINT32)((jmtUINT32)(Swizzle) & ((((1 ?
 21:14) - (0 ?
 21:14) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 21:14) - (0 ? 21:14) + 1))))) << (0 ? 21:14)));
            binary[3] = (((jmtUINT32)(binary[3]) & ~((jmtUINT32)(((((1 ?
 22:22) - (0 ?
 22:22) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 22:22) - (0 ?
 22:22) + 1))))) << (0 ?
 22:22))) | ((jmtUINT32)((jmtUINT32)(Negate) & ((((1 ?
 22:22) - (0 ?
 22:22) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 22:22) - (0 ? 22:22) + 1))))) << (0 ? 22:22)));
            binary[3] = (((jmtUINT32)(binary[3]) & ~((jmtUINT32)(((((1 ?
 23:23) - (0 ?
 23:23) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 23:23) - (0 ?
 23:23) + 1))))) << (0 ?
 23:23))) | ((jmtUINT32)((jmtUINT32)(Absolute) & ((((1 ?
 23:23) - (0 ?
 23:23) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 23:23) - (0 ? 23:23) + 1))))) << (0 ? 23:23)));
            binary[3] = (((jmtUINT32)(binary[3]) & ~((jmtUINT32)(((((1 ?
 27:25) - (0 ?
 27:25) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 27:25) - (0 ?
 27:25) + 1))))) << (0 ?
 27:25))) | ((jmtUINT32)((jmtUINT32)(Relative) & ((((1 ?
 27:25) - (0 ?
 27:25) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 27:25) - (0 ? 27:25) + 1))))) << (0 ? 27:25)));
            binary[3] = (((jmtUINT32)(binary[3]) & ~((jmtUINT32)(((((1 ?
 30:28) - (0 ?
 30:28) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 30:28) - (0 ?
 30:28) + 1))))) << (0 ?
 30:28))) | ((jmtUINT32)((jmtUINT32)(Type) & ((((1 ?
 30:28) - (0 ?
 30:28) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 30:28) - (0 ? 30:28) + 1))))) << (0 ? 30:28)));
            break;
    }
#else
    jmtUINT config[][3] = {
        {1, 2, 2},
        {2, 2, 3},
        {3, 3, 3},
    };

    j9_subsecurities(Where, config[Where][0], config[Where][1], config[Where][2], Address, Swizzle, Type, Negate, Absolute, Relative, binary);
#endif
    JMM_kFOOTER();
    return status;
}

static const jmtUINT32 NEGATE_FLAG   = 1 << 0;
static const jmtUINT32 ABSOLUTE_FLAG = 1 << 1;

j9_duopoly
_InitializePPU_SetUniform(
    IN jmtUINT32                            Where,
    IN jmtUINT32                            Address,
    IN jmtUINT32                            Swizzle,
    IN jmtUINT32                            Modifiers,
    OUT jmtUINT32_PTR                        binary
    )
{
    j9_duopoly status = J9_FLUTTERING;

    jmtBOOL negate   = (Modifiers & NEGATE_FLAG  ) ? J9_CUPPY : J9_YARELY;
    jmtBOOL absolute = (Modifiers & ABSOLUTE_FLAG) ? J9_CUPPY : J9_YARELY;
    j9_tympanichord("Where=0x%x", Where);

    j9_recaution(_InitializePPU_SetSource(Where, Address, Swizzle, 0x2, negate, absolute, 0, binary));

OnError:
    JMM_kFOOTER();
    return status;
}

j9_duopoly
_InitializePPU_SetTempReg(
    IN jmtUINT32                            Where,
    IN jmtUINT32                            Address,
    IN jmtUINT32                            Swizzle,
    IN jmtUINT32                            Modifiers,
    OUT jmtUINT32_PTR                        binary
    )
{
    j9_duopoly status = J9_FLUTTERING;

    jmtBOOL negate   = (Modifiers & NEGATE_FLAG  ) ? J9_CUPPY : J9_YARELY;
    jmtBOOL absolute = (Modifiers & ABSOLUTE_FLAG) ? J9_CUPPY : J9_YARELY;
    j9_tympanichord("Where=0x%x", Where);

    j9_recaution(_InitializePPU_SetSource(Where, Address, Swizzle, 0x0, negate, absolute, 0, binary));

OnError:
    JMM_kFOOTER();
    return status;
}


j9_duopoly
_InitializePPU_SetSourceBin(
    IN jmtUINT32                            SourceBin,
    OUT jmtUINT32_PTR                        binary
    )
{
    j9_duopoly status = J9_FLUTTERING;

    j9_tympanichord("SourceBin=0x%x", SourceBin);

    binary[1] = HwFunc_SETBITS(binary[1], 25, 22, SourceBin);

    JMM_kFOOTER();
    return status;
}


j9_duopoly
_InitializePPU_SetImmediateValue(
    IN jmtUINT32                            Where,
    IN jmtUINT32                            Value,
    OUT jmtUINT32_PTR                        binary
    )
{
    j9_duopoly status = J9_FLUTTERING;
    jmtINT32 raw = (((jmtUINT32)(0) & ~((jmtUINT32)(((((1 ?
 19:0) - (0 ?
 19:0) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 19:0) - (0 ?
 19:0) + 1))))) << (0 ?
 19:0))) | ((jmtUINT32)((jmtUINT32)(Value) & ((((1 ?
 19:0) - (0 ?
 19:0) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 19:0) - (0 ?
 19:0) + 1))))) << (0 ?
 19:0))) | (((jmtUINT32)(0) & ~((jmtUINT32)(((((1 ?
 21:20) - (0 ?
 21:20) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 21:20) - (0 ?
 21:20) + 1))))) << (0 ?
 21:20))) | ((jmtUINT32)(0x2 & ((((1 ?
 21:20) - (0 ?
 21:20) + 1) == 32) ?
 ~0 : (jmtUINT32)(~((jmtUINT64)(~0) << ((1 ?
 21:20) - (0 ? 21:20) + 1))))) << (0 ? 21:20)));

    j9_tympanichord("Where=0x%x", Where);

    j9_recaution(_InitializePPU_SetSource(Where, HwFunc_GETBITS(raw, 8, 0), HwFunc_GETBITS(raw, 16, 9), 0x7, J9_DTB(raw, 17), J9_DTB(raw, 18), HwFunc_GETBITS(raw, 21, 19), binary));

OnError:
    JMM_kFOOTER();
    return status;
}

j9_duopoly j9_ferroconcretor(
    IN jmk_HARDWARE            Hardware,
    IN jmtUINT32            Data_type,
    IN OUT jmtUINT32_PTR    binarys,
    OUT  jmtUINT32_PTR        command_count,
    OUT  jmtUINT32_PTR        reg_count
    )
{
    j9_duopoly status = J9_FLUTTERING;
    jmtUINT32 count = 0;
    jmtUINT32 Input1 = Data_type;
    jmtUINT32 Input2 = Data_type;
    jmtUINT32 Output = Data_type;

    j9_tympanichord("binarys=0x%x", binarys);





    j9_recaution(_InitializePPU_AddOpcode(Hardware, 0x79, 0, Input1, &binarys[count]));
    j9_recaution(_InitializePPU_SetDestination(1, J9_HANDSMOOTH, J9_YARELY, &binarys[count]));
    j9_recaution(_InitializePPU_SetEVIS(0, j9_handle_j9_flimsilyst(Input1), 1, &binarys[count]));
    j9_recaution(_InitializePPU_SetUniform(0, 0, J9_EMPIRICALLY, 0, &binarys[count]));
    j9_recaution(_InitializePPU_SetTempReg(1, 0, J9_ANTIFEMINIST(0, 1), 0, &binarys[count]));
    count += 4;


    j9_recaution(_InitializePPU_AddOpcode(Hardware, 0x79, 0, Input2, &binarys[count]));
    j9_recaution(_InitializePPU_SetDestination(2, J9_HANDSMOOTH, J9_YARELY, &binarys[count]));
    j9_recaution(_InitializePPU_SetEVIS(0, j9_handle_j9_flimsilyst(Input2), 1, &binarys[count]));
    j9_recaution(_InitializePPU_SetUniform(0, 0, J9_EMPIRICALLY, 0, &binarys[count]));
    j9_recaution(_InitializePPU_SetTempReg(1, 0, J9_ANTIFEMINIST(0, 1), 0, &binarys[count]));
    count += 4;


    j9_recaution(_InitializePPU_AddOpcode(Hardware, 0x45, 0x0B, Output, &binarys[count]));
    j9_recaution(_InitializePPU_SetDestination(1, J9_HANDSMOOTH, J9_YARELY, &binarys[count]));
    j9_recaution(_InitializePPU_SetEVIS(0, 7, (Input1 | (Input2 << 3)), &binarys[count]));
    j9_recaution(_InitializePPU_SetTempReg(0, 1, J9_EMPIRICALLY, 0, &binarys[count]));
    j9_recaution(_InitializePPU_SetTempReg(1, 2, J9_EMPIRICALLY, 0, &binarys[count]));
    j9_recaution(_InitializePPU_SetSource (2, 2, J9_EMPIRICALLY, 0x4, J9_YARELY, J9_YARELY, 0, &binarys[count]));
    count += 4;



    j9_recaution(_InitializePPU_AddOpcode(Hardware, 0x7A, 0, Output, &binarys[count]));
    j9_recaution(_InitializePPU_SetEVIS(0, j9_handle_j9_flimsilyst(Output), 1, &binarys[count]));
    j9_recaution(_InitializePPU_SetUniform(0, 1, J9_EMPIRICALLY, 0, &binarys[count]));
    j9_recaution(_InitializePPU_SetTempReg(1, 0, J9_ANTIFEMINIST(0, 1), 0, &binarys[count]));
    j9_recaution(_InitializePPU_SetTempReg(2, 1, J9_EMPIRICALLY, 0, &binarys[count]));
    count += 4;

    *command_count = count;
    *reg_count = 0x3;
OnError:
    JMM_kFOOTER();
    return status;
}

#endif




