/*
 * JMGPU driver
 *
 * Copyright (c) 2020 ChangSha JingJiaMicro Electronics Co., Ltd.
 * All rights reserved. 
 *
 * Author:
 *      wj <jjwgpu@jingjiamicro.com>
 * 
 * The software and information contained herein is proprietary and 
 * confidential to JingJiaMicro Electronics. This software can only be 
 * used by JingJiaMicro Electronics Corporation. Any use, reproduction, 
 * or disclosure without the written permission of JingJiaMicro 
 * Electronics Corporation is strictly prohibited.
 *
 */
 




#include "jmgpu_integrity.h"
#include "jmgpu_detect.h"
#include "jmgpu_protect.h"



#define J9_ACESODYNE    J9_UNDERSIGNALMEN



#define J9CGM_(reg)                                                            \
    j9pdn_(\
        Context, index, \
        reg ## _Address >> 2, \
        reg ## _ResetValue, \
        reg ## _Count, \
        J9_YARELY, J9_YARELY                                                     \
        )

#define J9_LUMINANCE(reg, count)                                               \
    j9pdn_(\
        Context, index, \
        reg ## _Address >> 2, \
        reg ## _ResetValue, \
        count, \
        J9_YARELY, J9_YARELY                                                     \
        )

#define J9MIRROR_CONGRUENCE(reg, offset, count)                                \
    j9pdn_(\
        Context, index, \
        (reg ## _Address >> 2) + offset, \
        reg ## _ResetValue, \
        count, \
        J9_YARELY, J9_YARELY                                                     \
        )

#define J9MIRROR_QUINTUPLET(reg, mirror, count)                                \
    j9_packboard(\
        Context, \
        reg ## _Address >> 2, \
        count, \
        mirror ## _Address >> 2                                                \
        )

#define J9_SAWBUCKS(reg)                                                       \
    j9pdn_(\
        Context, index, \
        reg ## _Address >> 2, \
        reg ## _ResetValue, \
        reg ## _Count, \
        J9_YARELY, J9_CUPPY                                                      \
        )

#define J9_SEMIRETICULATE(reg, block, count)                                   \
    j9pdn_(\
        Context, index, \
        (reg ## _Address >> 2) + (block << reg ## _BLK), \
        reg ## _ResetValue, \
        count, \
        J9_YARELY, J9_CUPPY                                                      \
        )

#define J9_HANDLE_J9M_CELIOSCOPY(reg, offset, count)                           \
    j9pdn_(\
        Context, index, \
        (reg ## _Address >> 2) + offset, \
        reg ## _ResetValue, \
        count, \
        J9_YARELY, J9_CUPPY                                                      \
        )

#define J9_OASAL(reg)                                                          \
    j9pdn_(\
        Context, index, \
        reg ## _Address >> 2, \
        reg ## _ResetValue, \
        reg ## _Count, \
        J9_CUPPY, J9_YARELY                                                      \
        )

#define J9_UNCHANGINGNESS(reg, value)                                          \
    j9pdn_(\
        Context, index, \
        reg ## _Address >> 2, \
        value, \
        reg ## _Count, \
        J9_YARELY, J9_YARELY                                                     \
        )

#define J9_HANDLE_J9M_PRECONCEAL(reg, offset, value)                           \
    j9pdn_(\
        Context, index, \
        (reg ## _Address >> 2) + offset, \
        value, \
        1, \
        J9_YARELY, J9_YARELY                                                     \
        )

#define J9_HANDLE_J9_ANTIRACISM(reg, value, block, count)                      \
    j9pdn_(\
        Context, index, \
        (reg ## _Address >> 2) + (block << reg ## _BLK), \
        value, \
        count, \
        J9_YARELY, J9_YARELY                                                     \
        )


#define J9_ARDENNITE()                                                         \
    j9_handle_congruence(Context, index)

#define J9_ICSH(reg, field)                                                    \
    do                                                                         \
    {                                                                          \
        if (J9_HANDLE_STEELINESS(data, reg, MASK_ ## field, ENABLED))           \
        {                                                                      \
            enable |= J9_DOWNSTREET(reg, field);                                \
        }                                                                      \
    }                                                                          \
    while (J9_YARELY)

#define J9_MAHARANEE(reg)                                                      \
    ((reg ## _Count) >> (reg ## _BLK))



#define J9_PRECONCEALS \
    (J9_MEGAPHONICALLY(0, AQ_COMMAND_NOP_COMMAND, OPCODE, NOP) | 0xC0FFEE)

#if !J9_HANDLE__UNFEMINISE || J9_DEMISSNESS
static jmtUINT32
j9_handle_congruence(
    IN jmk_CONTEXT Context,
    IN jmtUINT32 Index
    )
{
    jmtUINT32_PTR buffer;
    jmtUINT32 align;


    align = (Index & 1) ? 1 : 0;


    buffer = (Context->buffer == J9_CHYAK)
        ? J9_CHYAK
        : Context->buffer->logical;

    if (align && (buffer != J9_CHYAK))
    {
        buffer[Index] = 0xDEADDEAD;
    }


    Context->lastAddress = ~0U;


    return align;
}
#endif

#if !J9_HANDLE__UNFEMINISE
static jmtUINT32
j9_maad(
    IN jmk_CONTEXT Context,
    IN jmtUINT32 Index
    )
{
    if (Context->buffer != J9_CHYAK)
    {
        jmtUINT32_PTR buffer;


        buffer = Context->buffer->logical + Index;


        *buffer++ = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x03 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)));
        *buffer++ = 0;
    }


    return 2;
}
#endif

#if J9_DEMISSNESS
static jmtUINT32
j9_voltize(
    IN jmk_CONTEXT Context,
    IN jmtUINT32 Index,
    IN j9_rittingerite Pipe
    )
{
    jmtUINT32 flushSlots;
    jmtBOOL txCacheFix;
    jmtBOOL fcFlushStall;
    jmtBOOL iCacheInvalidate;
    jmtBOOL halti5;
    jmtBOOL snapPages;
    jmtBOOL hwTFB;
    jmtBOOL blt;
    jmtBOOL peTSFlush;
    jmtBOOL multiCluster;
    jmtBOOL computeOnly;

    txCacheFix
        = jmkHARDWARE_IsFeatureAvailable(Context->hardware, J9_HANDLE__ATTRIBUTE_EQUIPOISED);

    fcFlushStall
        = jmkHARDWARE_IsFeatureAvailable(Context->hardware, J9_HANDLE_J9MIN_PUZZLEHEAD);

    iCacheInvalidate
        = jmkHARDWARE_IsFeatureAvailable(Context->hardware, J9_HANDLE_ATTRIBUTE_BEHAVIOUR_REMODIFIED);

    halti5
        = jmkHARDWARE_IsFeatureAvailable(Context->hardware, J9MATHS_CORPORALCY);

    snapPages
        = jmkHARDWARE_IsFeatureAvailable(Context->hardware, J9_HANDLE_J9MATHS_ASSIGNABLY) &&
          jmkHARDWARE_IsFeatureAvailable(Context->hardware, J9_HANDLE_J9M_SEMESTRIAL);

    hwTFB
        = jmkHARDWARE_IsFeatureAvailable(Context->hardware, J9MATHS_TRANSMUTES);

    blt
        = jmkHARDWARE_IsFeatureAvailable(Context->hardware, J9_HANDLE_J_SNOBBINESS);
    multiCluster
        = jmkHARDWARE_IsFeatureAvailable(Context->hardware, J9_HANDLE_J9MA_CORROSIVES);

    peTSFlush
        = jmkHARDWARE_IsFeatureAvailable(Context->hardware, J9_HANDLE_J9MA_ATTRIBUTE_DISTORTION);

    computeOnly = jmkHARDWARE_IsFeatureAvailable(Context->hardware, J9_HANDLE_J9M_DELIQUESCE);

    flushSlots = blt ? 10 : 6;

    if (Pipe == J9_SYNAPTAI)
    {
        if (!txCacheFix)
        {

            flushSlots += blt ? 8 : 4;
        }


        flushSlots += 2;
    }

    if (fcFlushStall && !computeOnly)
    {

        flushSlots += blt ? ((!peTSFlush) ? 14 :10) : 6;
    }

    if (iCacheInvalidate && !halti5)
    {
        flushSlots += blt ? 16 : 12;
    }


    if (hwTFB)
    {
        flushSlots += 2;
    }


    if (snapPages)
    {
        flushSlots += 2;
    }

    if (Context->buffer != J9_CHYAK)
    {
        jmtUINT32_PTR buffer;


        buffer = Context->buffer->logical + Index;

        if (Pipe == J9_SYNAPTAI && !txCacheFix)
        {
            if (blt)
            {

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x502E) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 0:0) - (0 ?
 0:0) + 1))))))) << (0 ?
 0:0))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 0:0) - (0 ? 0:0) + 1))))))) << (0 ? 0:0)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x0E02) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:0) - (0 ?
 4:0) + 1))))))) << (0 ?
 4:0))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:0) - (0 ? 4:0) + 1))))))) << (0 ? 4:0)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 12:8) - (0 ?
 12:8) + 1))))))) << (0 ?
 12:8))) | (((jmtUINT32) (0x10 & ((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 12:8) - (0 ? 12:8) + 1))))))) << (0 ? 12:8)));


                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x09 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:0) - (0 ?
 4:0) + 1))))))) << (0 ?
 4:0))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:0) - (0 ? 4:0) + 1))))))) << (0 ? 4:0)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 12:8) - (0 ?
 12:8) + 1))))))) << (0 ?
 12:8))) | (((jmtUINT32) (0x10 & ((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 12:8) - (0 ? 12:8) + 1))))))) << (0 ? 12:8)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x502E) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 0:0) - (0 ?
 0:0) + 1))))))) << (0 ?
 0:0))) | (((jmtUINT32) (0x0 & ((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 0:0) - (0 ? 0:0) + 1))))))) << (0 ? 0:0)));
            }
            else
            {

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x0E02) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:0) - (0 ?
 4:0) + 1))))))) << (0 ?
 4:0))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:0) - (0 ? 4:0) + 1))))))) << (0 ? 4:0)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 12:8) - (0 ?
 12:8) + 1))))))) << (0 ?
 12:8))) | (((jmtUINT32) (0x07 & ((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 12:8) - (0 ? 12:8) + 1))))))) << (0 ? 12:8)));


                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x09 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:0) - (0 ?
 4:0) + 1))))))) << (0 ?
 4:0))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:0) - (0 ? 4:0) + 1))))))) << (0 ? 4:0)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 12:8) - (0 ?
 12:8) + 1))))))) << (0 ?
 12:8))) | (((jmtUINT32) (0x07 & ((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 12:8) - (0 ? 12:8) + 1))))))) << (0 ? 12:8)));
            }
        }


        *buffer++
            = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x0E03) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

        *buffer++
            = (Pipe == J9_GUNSMITH)
                ?
   ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 3:3) - (0 ?
 3:3) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 3:3) - (0 ?
 3:3) + 1))))))) << (0 ?
 3:3))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 3:3) - (0 ?
 3:3) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 3:3) - (0 ? 3:3) + 1))))))) << (0 ? 3:3)))
                :   ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 0:0) - (0 ?
 0:0) + 1))))))) << (0 ?
 0:0))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 0:0) - (0 ? 0:0) + 1))))))) << (0 ? 0:0)))
                  | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 1:1) - (0 ?
 1:1) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 1:1) - (0 ?
 1:1) + 1))))))) << (0 ?
 1:1))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 1:1) - (0 ?
 1:1) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 1:1) - (0 ? 1:1) + 1))))))) << (0 ? 1:1)))
                  | (multiCluster ?
 0 : ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 2:2) - (0 ?
 2:2) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 2:2) - (0 ?
 2:2) + 1))))))) << (0 ?
 2:2))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 2:2) - (0 ?
 2:2) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 2:2) - (0 ? 2:2) + 1))))))) << (0 ? 2:2))))
                  | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 5:5) - (0 ?
 5:5) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 5:5) - (0 ?
 5:5) + 1))))))) << (0 ?
 5:5))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 5:5) - (0 ?
 5:5) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 5:5) - (0 ? 5:5) + 1))))))) << (0 ? 5:5)))
                  | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 10:10) - (0 ?
 10:10) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 10:10) - (0 ?
 10:10) + 1))))))) << (0 ?
 10:10))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 10:10) - (0 ?
 10:10) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 10:10) - (0 ? 10:10) + 1))))))) << (0 ? 10:10)))
                  | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 11:11) - (0 ?
 11:11) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 11:11) - (0 ?
 11:11) + 1))))))) << (0 ?
 11:11))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 11:11) - (0 ?
 11:11) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 11:11) - (0 ? 11:11) + 1))))))) << (0 ? 11:11)));

        if (hwTFB)
        {
             *buffer++
                 = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                 | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x7003) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)))
                 | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)));

             *buffer++
                 = 0x1;
        }


        if (Pipe == J9_SYNAPTAI)
        {
            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x0E03) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

            *buffer++ = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:4) - (0 ?
 4:4) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:4) - (0 ?
 4:4) + 1))))))) << (0 ?
 4:4))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 4:4) - (0 ?
 4:4) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:4) - (0 ? 4:4) + 1))))))) << (0 ? 4:4)));
        }


        if (blt)
        {

            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x502E) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 0:0) - (0 ?
 0:0) + 1))))))) << (0 ?
 0:0))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 0:0) - (0 ? 0:0) + 1))))))) << (0 ? 0:0)));

            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x0E02) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:0) - (0 ?
 4:0) + 1))))))) << (0 ?
 4:0))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:0) - (0 ? 4:0) + 1))))))) << (0 ? 4:0)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 12:8) - (0 ?
 12:8) + 1))))))) << (0 ?
 12:8))) | (((jmtUINT32) (0x10 & ((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 12:8) - (0 ? 12:8) + 1))))))) << (0 ? 12:8)));


            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x09 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)));

            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:0) - (0 ?
 4:0) + 1))))))) << (0 ?
 4:0))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:0) - (0 ? 4:0) + 1))))))) << (0 ? 4:0)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 12:8) - (0 ?
 12:8) + 1))))))) << (0 ?
 12:8))) | (((jmtUINT32) (0x10 & ((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 12:8) - (0 ? 12:8) + 1))))))) << (0 ? 12:8)));

            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x502E) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 0:0) - (0 ?
 0:0) + 1))))))) << (0 ?
 0:0))) | (((jmtUINT32) (0x0 & ((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 0:0) - (0 ? 0:0) + 1))))))) << (0 ? 0:0)));
        }
        else
        {
            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x0E02) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:0) - (0 ?
 4:0) + 1))))))) << (0 ?
 4:0))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:0) - (0 ? 4:0) + 1))))))) << (0 ? 4:0)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 12:8) - (0 ?
 12:8) + 1))))))) << (0 ?
 12:8))) | (((jmtUINT32) (0x07 & ((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 12:8) - (0 ? 12:8) + 1))))))) << (0 ? 12:8)));


            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x09 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)));

            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:0) - (0 ?
 4:0) + 1))))))) << (0 ?
 4:0))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:0) - (0 ? 4:0) + 1))))))) << (0 ? 4:0)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 12:8) - (0 ?
 12:8) + 1))))))) << (0 ?
 12:8))) | (((jmtUINT32) (0x07 & ((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 12:8) - (0 ? 12:8) + 1))))))) << (0 ? 12:8)));
        }

        if (fcFlushStall && !computeOnly)
        {
            if (!peTSFlush && blt)
            {
                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x502E) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 0:0) - (0 ?
 0:0) + 1))))))) << (0 ?
 0:0))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 0:0) - (0 ? 0:0) + 1))))))) << (0 ? 0:0)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x502B) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 0:0) - (0 ?
 0:0) + 1))))))) << (0 ?
 0:0))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 0:0) - (0 ? 0:0) + 1))))))) << (0 ? 0:0)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x502E) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 0:0) - (0 ?
 0:0) + 1))))))) << (0 ?
 0:0))) | (((jmtUINT32) (0x0 & ((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 0:0) - (0 ? 0:0) + 1))))))) << (0 ? 0:0)));
            }
            else
            {
                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x0594) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 0:0) - (0 ?
 0:0) + 1))))))) << (0 ?
 0:0))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 0:0) - (0 ? 0:0) + 1))))))) << (0 ? 0:0)));
            }


            if (blt)
            {

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x502E) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 0:0) - (0 ?
 0:0) + 1))))))) << (0 ?
 0:0))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 0:0) - (0 ? 0:0) + 1))))))) << (0 ? 0:0)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x0E02) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:0) - (0 ?
 4:0) + 1))))))) << (0 ?
 4:0))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:0) - (0 ? 4:0) + 1))))))) << (0 ? 4:0)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 12:8) - (0 ?
 12:8) + 1))))))) << (0 ?
 12:8))) | (((jmtUINT32) (0x10 & ((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 12:8) - (0 ? 12:8) + 1))))))) << (0 ? 12:8)));


                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x09 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:0) - (0 ?
 4:0) + 1))))))) << (0 ?
 4:0))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:0) - (0 ? 4:0) + 1))))))) << (0 ? 4:0)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 12:8) - (0 ?
 12:8) + 1))))))) << (0 ?
 12:8))) | (((jmtUINT32) (0x10 & ((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 12:8) - (0 ? 12:8) + 1))))))) << (0 ? 12:8)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x502E) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 0:0) - (0 ?
 0:0) + 1))))))) << (0 ?
 0:0))) | (((jmtUINT32) (0x0 & ((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 0:0) - (0 ? 0:0) + 1))))))) << (0 ? 0:0)));
            }
            else
            {
                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x0E02) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:0) - (0 ?
 4:0) + 1))))))) << (0 ?
 4:0))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:0) - (0 ? 4:0) + 1))))))) << (0 ? 4:0)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 12:8) - (0 ?
 12:8) + 1))))))) << (0 ?
 12:8))) | (((jmtUINT32) (0x07 & ((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 12:8) - (0 ? 12:8) + 1))))))) << (0 ? 12:8)));


                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x09 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:0) - (0 ?
 4:0) + 1))))))) << (0 ?
 4:0))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:0) - (0 ? 4:0) + 1))))))) << (0 ? 4:0)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 12:8) - (0 ?
 12:8) + 1))))))) << (0 ?
 12:8))) | (((jmtUINT32) (0x07 & ((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 12:8) - (0 ? 12:8) + 1))))))) << (0 ? 12:8)));
            }
        }

        if (iCacheInvalidate && !halti5)
        {

            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x0218) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 0:0) - (0 ?
 0:0) + 1))))))) << (0 ?
 0:0))) | (((jmtUINT32) (0x0 & ((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 0:0) - (0 ? 0:0) + 1))))))) << (0 ? 0:0)));

            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x021A) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 0:0) - (0 ?
 0:0) + 1))))))) << (0 ?
 0:0))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 0:0) - (0 ? 0:0) + 1))))))) << (0 ? 0:0)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:4) - (0 ?
 4:4) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:4) - (0 ?
 4:4) + 1))))))) << (0 ?
 4:4))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 4:4) - (0 ?
 4:4) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:4) - (0 ? 4:4) + 1))))))) << (0 ? 4:4)));

            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x0218) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 0:0) - (0 ?
 0:0) + 1))))))) << (0 ?
 0:0))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 0:0) - (0 ? 0:0) + 1))))))) << (0 ? 0:0)));

            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x021A) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 0:0) - (0 ?
 0:0) + 1))))))) << (0 ?
 0:0))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 0:0) - (0 ? 0:0) + 1))))))) << (0 ? 0:0)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 5:5) - (0 ?
 5:5) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 5:5) - (0 ?
 5:5) + 1))))))) << (0 ?
 5:5))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 5:5) - (0 ?
 5:5) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 5:5) - (0 ? 5:5) + 1))))))) << (0 ? 5:5)));


            if (blt)
            {

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x502E) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 0:0) - (0 ?
 0:0) + 1))))))) << (0 ?
 0:0))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 0:0) - (0 ? 0:0) + 1))))))) << (0 ? 0:0)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x0E02) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:0) - (0 ?
 4:0) + 1))))))) << (0 ?
 4:0))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:0) - (0 ? 4:0) + 1))))))) << (0 ? 4:0)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 12:8) - (0 ?
 12:8) + 1))))))) << (0 ?
 12:8))) | (((jmtUINT32) (0x10 & ((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 12:8) - (0 ? 12:8) + 1))))))) << (0 ? 12:8)));


                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x09 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:0) - (0 ?
 4:0) + 1))))))) << (0 ?
 4:0))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:0) - (0 ? 4:0) + 1))))))) << (0 ? 4:0)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 12:8) - (0 ?
 12:8) + 1))))))) << (0 ?
 12:8))) | (((jmtUINT32) (0x10 & ((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 12:8) - (0 ? 12:8) + 1))))))) << (0 ? 12:8)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x502E) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 0:0) - (0 ?
 0:0) + 1))))))) << (0 ?
 0:0))) | (((jmtUINT32) (0x0 & ((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 0:0) - (0 ? 0:0) + 1))))))) << (0 ? 0:0)));
            }
            else
            {
                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x0E02) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:0) - (0 ?
 4:0) + 1))))))) << (0 ?
 4:0))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:0) - (0 ? 4:0) + 1))))))) << (0 ? 4:0)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 12:8) - (0 ?
 12:8) + 1))))))) << (0 ?
 12:8))) | (((jmtUINT32) (0x07 & ((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 12:8) - (0 ? 12:8) + 1))))))) << (0 ? 12:8)));


                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x09 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)));

                *buffer++
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:0) - (0 ?
 4:0) + 1))))))) << (0 ?
 4:0))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:0) - (0 ? 4:0) + 1))))))) << (0 ? 4:0)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 12:8) - (0 ?
 12:8) + 1))))))) << (0 ?
 12:8))) | (((jmtUINT32) (0x07 & ((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 12:8) - (0 ? 12:8) + 1))))))) << (0 ? 12:8)));
            }
        }

        if (snapPages)
        {
            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x13 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:0) - (0 ?
 4:0) + 1))))))) << (0 ?
 4:0))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:0) - (0 ? 4:0) + 1))))))) << (0 ? 4:0)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:0) - (0 ?
 4:0) + 1))))))) << (0 ?
 4:0))) | (((jmtUINT32) (0x02 & ((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:0) - (0 ? 4:0) + 1))))))) << (0 ? 4:0)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:0) - (0 ?
 4:0) + 1))))))) << (0 ?
 4:0))) | (((jmtUINT32) (0x04 & ((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:0) - (0 ? 4:0) + 1))))))) << (0 ? 4:0)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:0) - (0 ?
 4:0) + 1))))))) << (0 ?
 4:0))) | (((jmtUINT32) (0x08 & ((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:0) - (0 ? 4:0) + 1))))))) << (0 ? 4:0)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:0) - (0 ?
 4:0) + 1))))))) << (0 ?
 4:0))) | (((jmtUINT32) (0x10 & ((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:0) - (0 ? 4:0) + 1))))))) << (0 ? 4:0)));

            *buffer++
                = 0;
        }
    }


    return flushSlots;
}
#endif

#if J9_DEMISSNESS
static jmtUINT32
j9_ungroundable(
    IN jmk_CONTEXT Context,
    IN jmtUINT32 Index
    )
{
    jmtBOOL blt = jmkHARDWARE_IsFeatureAvailable(Context->hardware, J9_HANDLE_J_SNOBBINESS);
    if (Context->buffer != J9_CHYAK)
    {
        jmtUINT32_PTR buffer;


        buffer = Context->buffer->logical + Index;

        if (blt)
        {

            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x502E) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 0:0) - (0 ?
 0:0) + 1))))))) << (0 ?
 0:0))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 0:0) - (0 ? 0:0) + 1))))))) << (0 ? 0:0)));

            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x0E02) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:0) - (0 ?
 4:0) + 1))))))) << (0 ?
 4:0))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:0) - (0 ? 4:0) + 1))))))) << (0 ? 4:0)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 12:8) - (0 ?
 12:8) + 1))))))) << (0 ?
 12:8))) | (((jmtUINT32) (0x10 & ((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 12:8) - (0 ? 12:8) + 1))))))) << (0 ? 12:8)));


            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x09 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)));

            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:0) - (0 ?
 4:0) + 1))))))) << (0 ?
 4:0))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:0) - (0 ? 4:0) + 1))))))) << (0 ? 4:0)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 12:8) - (0 ?
 12:8) + 1))))))) << (0 ?
 12:8))) | (((jmtUINT32) (0x10 & ((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 12:8) - (0 ? 12:8) + 1))))))) << (0 ? 12:8)));

            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x502E) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

            *buffer
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 0:0) - (0 ?
 0:0) + 1))))))) << (0 ?
 0:0))) | (((jmtUINT32) (0x0 & ((jmtUINT32) ((((1 ?
 0:0) - (0 ?
 0:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 0:0) - (0 ? 0:0) + 1))))))) << (0 ? 0:0)));
        }
        else
        {

            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x0E02) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:0) - (0 ?
 4:0) + 1))))))) << (0 ?
 4:0))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:0) - (0 ? 4:0) + 1))))))) << (0 ? 4:0)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 12:8) - (0 ?
 12:8) + 1))))))) << (0 ?
 12:8))) | (((jmtUINT32) (0x07 & ((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 12:8) - (0 ? 12:8) + 1))))))) << (0 ? 12:8)));


            *buffer++
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x09 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)));

            *buffer
                = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:0) - (0 ?
 4:0) + 1))))))) << (0 ?
 4:0))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 4:0) - (0 ?
 4:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:0) - (0 ? 4:0) + 1))))))) << (0 ? 4:0)))
                | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 12:8) - (0 ?
 12:8) + 1))))))) << (0 ?
 12:8))) | (((jmtUINT32) (0x07 & ((jmtUINT32) ((((1 ?
 12:8) - (0 ?
 12:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 12:8) - (0 ? 12:8) + 1))))))) << (0 ? 12:8)));
        }
    }


    return (blt ? 8 : 4);
}
#endif

#if (J9_DEMISSNESS || J9_MONGRELISE)

#if J9_HANDLE_J9M_ASSIGNABLY
typedef struct {
    jmtUINT inputBase;
    jmtUINT count;
    jmtUINT outputBase;
}
jmsSTATEMIRROR;

const jmsSTATEMIRROR mirroredStates[] =
{
    {0x5800, 0x300, 0x5B00},
    {0x5600, 0x100, 0x5700},
    {0xD800, 0x140, 0xD000},
};

jmtUINT mirroredStatesCount = 0;

static j9_duopoly
j9_juberous(
    IN jmsSTATE_DELTA_PTR StateDelta
    )
{

    StateDelta->id += 1;


    if (StateDelta->id == 0)
    {

        jmkOS_ZeroMemory(J9_PROVINCIALISMS(StateDelta->mapEntryID), StateDelta->mapEntryIDSize);


        StateDelta->id += 1;
    }


    StateDelta->elementCount = 0;


    StateDelta->recordCount = 0;


    return J9_FLUTTERING;
}

static j9_duopoly
j9_subcordate(
    IN jmk_CONTEXT Context,
    IN jmsSTATE_DELTA_PTR delta
)
{
    jmtUINT_PTR mapEntryIndex = J9_PROVINCIALISMS(delta->mapEntryIndex);
    jmtUINT_PTR mapEntryID = J9_PROVINCIALISMS(delta->mapEntryID);
    jmsSTATE_DELTA_RECORD_PTR recordArray = J9_PROVINCIALISMS(delta->recordArray);
    j9_duopoly status = J9_FLUTTERING;

    j9_quincunx();


    if (mapEntryIndex != J9_CHYAK)
    {
        j9_recaution(j9_overorganizing(Context->os, mapEntryIndex));
    }


    if (mapEntryID != J9_CHYAK)
    {
        j9_recaution(j9_overorganizing(Context->os, mapEntryID));
    }


    if (recordArray != J9_CHYAK)
    {
        j9_recaution(j9_overorganizing(Context->os, recordArray));
    }

    j9_recaution(j9_overorganizing(Context->os, delta));

OnError:


    JMM_kFOOTER_NO();
    return J9_FLUTTERING;
}

static j9_duopoly
j9_phoniatrics(
    IN jmk_CONTEXT Context,
    OUT jmsSTATE_DELTA_PTR * Delta
    )
{
    jmk_CONTEXT context = Context;
    jmk_OS os = context->os;
    j9_duopoly status;
    jmsSTATE_DELTA_PTR delta = J9_CHYAK;
    jmtPOINTER pointer = J9_CHYAK;

    j9_quincunx();

    if (context->maxState == 0)
    {
         *Delta = NULL;
         JMM_kFOOTER_NO();
         return J9_FLUTTERING;
    }


    j9_recaution(jmkOS_Allocate(
        os, J9_NONPROS(j9_revalidating), (jmtPOINTER *) &delta
        ));


    jmkOS_ZeroMemory(delta, J9_NONPROS(j9_revalidating));

    if (context->maxState > 0)
    {

        jmtSIZE_T bytes = J9_NONPROS(jmtUINT) * context->maxState;


        j9_recaution(jmkOS_Allocate(
            os, bytes, &pointer
            ));

        delta->mapEntryID = J9_SUBAUDIBLENESS(pointer);


        delta->mapEntryIDSize = (jmtUINT32)bytes;


        jmkOS_ZeroMemory(J9_PROVINCIALISMS(delta->mapEntryID), bytes);


        j9_recaution(jmkOS_Allocate(
            os, bytes, &pointer
            ));

        delta->mapEntryIndex = J9_SUBAUDIBLENESS(pointer);

    }

    if (context->numStates > 0)
    {

        j9_recaution(jmkOS_Allocate(
            os,
            J9_NONPROS(j9_handle_j_coruscated) * context->numStates,
            &pointer
            ));

        delta->recordArray = J9_SUBAUDIBLENESS(pointer);
    }


    j9_juberous(delta);

    *Delta = delta;

    JMM_kFOOTER();
    return status;

OnError:
    if (delta)
    {
        j9_subcordate(Context, delta);
    }

    JMM_kFOOTER_NO();
    return status;
}
#endif

static jmtUINT32
j9_ginglymi(
    IN jmk_CONTEXT Context,
    IN jmtUINT32 Index,
    IN j9_rittingerite Pipe
    )
{
    jmtUINT32 slots = 2;

    if (Context->buffer != J9_CHYAK)
    {
        jmtUINT32_PTR buffer;


        buffer = Context->buffer->logical + Index;


        *buffer++
            = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x0E00) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)));

        *buffer
            = (Pipe == J9_GUNSMITH)
                ? 0x1
                : 0x0;
    }

    Context->pipeSelectBytes = slots * J9_NONPROS(jmtUINT32);

    return slots;
}
#endif

#if !J9_HANDLE__UNFEMINISE || J9_DEMISSNESS
static jmtUINT32
j9pdn_(
    IN jmk_CONTEXT Context,
    IN jmtUINT32 Index,
    IN jmtUINT32 Address,
    IN jmtUINT32 Value,
    IN jmtUINT32 Size,
    IN jmtBOOL FixedPoint,
    IN jmtBOOL Hinted
    )
{
    jmtUINT32_PTR buffer;
    jmtUINT32 align;
    jmtUINT32 i;


    align = (Index & 1) ? 1 : 0;


    buffer = (Context->buffer == J9_CHYAK)
        ? J9_CHYAK
        : Context->buffer->logical;

    if ((buffer == J9_CHYAK) && (Address + Size > Context->maxState))
    {

        Context->maxState = Address + Size;
    }

    if (buffer == J9_CHYAK)
    {

        Context->numStates += Size;
    }


    if ((Address != Context->lastAddress) || (FixedPoint != Context->lastFixed))
    {
        if (buffer != J9_CHYAK)
        {
            if (align)
            {

                buffer[Index++] = 0xDEADDEAD;
            }


            JMM_kASSERT((Index & 1) == 0);

            if (FixedPoint)
            {
                buffer[Index]
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 26:26) - (0 ?
 26:26) + 1))))))) << (0 ?
 26:26))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 26:26) - (0 ? 26:26) + 1))))))) << (0 ? 26:26)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (Size) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (Address) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));
            }
            else
            {
                buffer[Index]
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 26:26) - (0 ?
 26:26) + 1))))))) << (0 ?
 26:26))) | (((jmtUINT32) (0x0 & ((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 26:26) - (0 ? 26:26) + 1))))))) << (0 ? 26:26)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (Size) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                    | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (Address) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));
            }


            for (i = 0; i < (jmtUINT32)Size; i += 1)
            {

                buffer[Index + 1 + i] = Value;


                Context->map[Address + i].index = (jmtUINT)Index + 1 + i;
            }
        }


        Context->lastIndex   = (jmtUINT)Index;
        Context->lastAddress = Address + (jmtUINT32)Size;
        Context->lastSize    = Size;
        Context->lastFixed   = FixedPoint;


        return align + 1 + Size;
    }


    if (buffer != J9_CHYAK)
    {

        buffer[Context->lastIndex] =
            ((((jmtUINT32) (buffer[Context->lastIndex])) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (Context->lastSize + Size) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)));


        for (i = 0; i < (jmtUINT32)Size; i += 1)
        {

            buffer[Index + i] = Value;


            Context->map[Address + i].index = (jmtUINT)Index + i;
        }
    }


    Context->lastAddress += (jmtUINT32)Size;
    Context->lastSize    += Size;


    return Size;
}

static jmtUINT32
j9_packboard(
    IN jmk_CONTEXT Context,
    IN jmtUINT32 Address,
    IN jmtUINT32 Size,
    IN jmtUINT32 AddressMirror
    )
{
    jmtUINT32 i;


    if (Context->buffer != J9_CHYAK)
    {

        for (i = 0; i < Size; i++)
        {

            Context->map[Address + i].index =
                Context->map[AddressMirror + i].index;
        }
    }


    return Size;
}

static void
j9_lonesomenesses(
    IN jmk_CONTEXT Context,
    IN jmtUINT32 Address,
    IN jmtUINT32 Size,
    IN jmtUINT32 Count
    )
{
    jmtUINT base;
    jmtUINT nopCount;
    jmtUINT32_PTR nop;
    jmsCONTEXT_PTR buffer;
    jmsSTATE_MAP_PTR map;
    jmtUINT i;


    buffer = Context->buffer;


    map = Context->map;

    base = map[Address].index;

    if (Count > 1024)
    {
        buffer->logical[base - 1]
            = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 26:26) - (0 ?
 26:26) + 1))))))) << (0 ?
 26:26))) | (((jmtUINT32) (0x0 & ((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 26:26) - (0 ? 26:26) + 1))))))) << (0 ? 26:26)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1024) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (Address) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

        buffer->logical[base + 1024 + 1]
            = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 26:26) - (0 ?
 26:26) + 1))))))) << (0 ?
 26:26))) | (((jmtUINT32) (0x0 & ((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 26:26) - (0 ? 26:26) + 1))))))) << (0 ? 26:26)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (Count - 1024) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (Address + 1024) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));


        nopCount = (Size / 2) - (Count / 2);

        nop = &buffer->logical[base + (Count | 1) + 2];


        for (i = 0; i < nopCount; i += 1)
        {
            if (nop >= buffer->logical + Context->totalSize)
            {
                break;
            }


            *nop = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x03 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)));


            nop += 2;
        }
    }
    else
    {
        buffer->logical[base - 1]
            = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 26:26) - (0 ?
 26:26) + 1))))))) << (0 ?
 26:26))) | (((jmtUINT32) (0x0 & ((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 26:26) - (0 ? 26:26) + 1))))))) << (0 ? 26:26)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (Count) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (Address) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));


        nopCount = (Size / 2) - (Count / 2) + Size / 1024;


        nop = &buffer->logical[base + (Count | 1)];


        for (i = 0; i < nopCount; i += 1)
        {
            if (nop >= buffer->logical + Context->totalSize)
            {
                break;
            }


            *nop = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x03 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)));


            nop += 2;
        }
    }
}
#endif

#if (J9_DEMISSNESS || J9_MONGRELISE)

static j9_duopoly
j9_handle_j9_attribute_interester(
    IN jmk_CONTEXT Context
    )
{
    jmtUINT32_PTR buffer;
    jmtUINT32 index;

#if J9_DEMISSNESS
    jmtBOOL halti5;
    jmtBOOL hasSecurity;
    jmtBOOL hasRobustness;
    jmtBOOL multiCluster;
    jmtUINT clusterAliveMask;
#endif

    jmk_HARDWARE hardware;

    j9_quincunx();

    hardware = Context->hardware;

    j9maths_activation(hardware, J9_CONTRACEPTIVE);


    index = 0;


    Context->lastAddress = ~0U;


    buffer = (Context->buffer == J9_CHYAK)
        ? J9_CHYAK
        : Context->buffer->logical;





#if !J9_HANDLE__UNFEMINISE
    if (jmkHARDWARE_IsFeatureAvailable(hardware, J9MIRROR_RHYMESTERS))
    {

        index += j9_voltize(Context, index, J9_SYNAPTAI);


        index += j9_ginglymi(Context, index, J9_GUNSMITH);

        index += j9pdn_(Context, index, 0x00670 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);


        index += j9pdn_(Context, index, 0x01200 >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, 0x01204 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01208 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x0120C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01210 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01214 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01218 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x0121C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);


        index += j9pdn_(Context, index, 0x01220 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01224 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);


        index += j9pdn_(Context, index, 0x01228 >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, 0x0122C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01230 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01234 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);


        index += j9pdn_(Context, index, 0x01238 >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, 0x01240 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01244 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01248 >> 2, 0xFFFFFFFF, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x0124C >> 2, 0xFFFFFFFF, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01250 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01254 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x0123C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);


        index += j9pdn_(Context, index, 0x01258 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01800 >> 2, 0x00000000, 128, J9_YARELY, J9_YARELY);


        index += j9pdn_(Context, index, 0x01C00 >> 2, 0x00000000, 256, J9_YARELY, J9_YARELY);


        index += j9pdn_(Context, index, 0x0125C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01260 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01264 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01268 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01270 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01274 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x0126C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01278 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);


        index += j9pdn_(Context, index, 0x0127C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01280 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);


        if (jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_J9M_ABRIDGABLE))
        {
            index += j9pdn_(Context, index, 0x01284 >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
            index += j9pdn_(Context, index, 0x01288 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x0128C >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
            index += j9pdn_(Context, index, 0x01290 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x01298 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x0129C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012A0 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012A4 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012A8 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012AC >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012E4 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        }


        if (jmkHARDWARE_IsFeatureAvailable(hardware, J9MATHS_ACETARSONE))
        {
            index += j9pdn_(Context, index, 0x012B0 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012B4 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012B8 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012BC >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012C0 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012C4 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012C8 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012CC >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012D0 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012D4 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012D8 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012DC >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012E0 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x03400 >> 2, 0x00000000, 256, J9_YARELY, J9_YARELY);
        }


        if (jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_J9_ATTRIBUTE_BEHAVIOUR_BALLBUSTER))
        {
            index += j9pdn_(Context, index, 0x012E8 >> 2, 0xFFFFFFFF, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012EC >> 2, 0xFFFFFFFF, 1, J9_YARELY, J9_YARELY);
        }


        if (jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_ATTRIBUTE_POLYGAMIAN))
        {
            index += j9pdn_(Context, index, 0x02800 >> 2, 0x00000000, 128, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x02A00 >> 2, 0x00000000, 128, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012F0 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012F4 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012F8 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012FC >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x01300 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x01304 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        }

        index += J9_ARDENNITE();

        Context->linkIndex2D = index;
        index += j9_maad(Context, index);
    }
#endif

#if J9_DEMISSNESS



    halti5 = jmkHARDWARE_IsFeatureAvailable(hardware, J9MATHS_CORPORALCY);
    hasSecurity = jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_PREREJOICE);
    hasRobustness = jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_J_ECSTATICAL);
    multiCluster = jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_J9MA_CORROSIVES);
    clusterAliveMask = hardware->identity.clusterAvailMask & hardware->options.userClusterMask;


    Context->entryOffset3D = (jmtUINT)index * J9_NONPROS(jmtUINT32);


    index += j9_ginglymi(Context, index, J9_SYNAPTAI);

    if (multiCluster)
    {
        index += j9pdn_(Context, index, (0x03910 >> 2) + (0 << 2), ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 7:0) - (0 ?
 7:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 7:0) - (0 ?
 7:0) + 1))))))) << (0 ?
 7:0))) | (((jmtUINT32) ((jmtUINT32) (clusterAliveMask) & ((jmtUINT32) ((((1 ?
 7:0) - (0 ?
 7:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 7:0) - (0 ? 7:0) + 1))))))) << (0 ? 7:0))), 4, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x03908 >> 2, ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 2:0) - (0 ?
 2:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 2:0) - (0 ?
 2:0) + 1))))))) << (0 ?
 2:0))) | (((jmtUINT32) (0x2 & ((jmtUINT32) ((((1 ?
 2:0) - (0 ?
 2:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 2:0) - (0 ? 2:0) + 1))))))) << (0 ? 2:0))), 1, J9_YARELY, J9_YARELY);
    }


#if J9_NOONED && 1
    index += j9pdn_(Context, index, 0x03850 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
#endif

    index += j9_voltize(Context, index, J9_SYNAPTAI);


    if (hasSecurity)
    {
        index += j9pdn_(Context, index, 0x03900 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += J9_ARDENNITE();
        index += j9pdn_(Context, index, 0x03904 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    }

    if (halti5)
    {
        jmtUINT32 uscControl = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 20:16) - (0 ?
 20:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 20:16) - (0 ?
 20:16) + 1))))))) << (0 ?
 20:16))) | (((jmtUINT32) ((jmtUINT32) (2) & ((jmtUINT32) ((((1 ?
 20:16) - (0 ?
 20:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 20:16) - (0 ? 20:16) + 1))))))) << (0 ? 20:16)));
        index += j9pdn_(Context, index, 0x03888 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x038C0 >> 2, 0x00000000, 16, J9_YARELY, J9_YARELY);

        uscControl |= ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 2:0) - (0 ?
 2:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 2:0) - (0 ?
 2:0) + 1))))))) << (0 ?
 2:0))) | (((jmtUINT32) ((jmtUINT32) (hardware->options.uscL1CacheRatio) & ((jmtUINT32) ((((1 ?
 2:0) - (0 ?
 2:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 2:0) - (0 ? 2:0) + 1))))))) << (0 ? 2:0)));
        if (multiCluster)
        {
            uscControl |= ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 11:8) - (0 ?
 11:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 11:8) - (0 ?
 11:8) + 1))))))) << (0 ?
 11:8))) | (((jmtUINT32) ((jmtUINT32) (hardware->options.uscAttribCacheRatio) & ((jmtUINT32) ((((1 ?
 11:8) - (0 ?
 11:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 11:8) - (0 ? 11:8) + 1))))))) << (0 ? 11:8)));
        }
        index += j9pdn_(Context, index, 0x03884 >> 2, uscControl, 1, J9_YARELY, J9_YARELY);
    }
    else
    {
        index += j9pdn_(Context, index, 0x03820 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x03828 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x0382C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x03834 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x03838 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x03854 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    }

    index += J9_ARDENNITE();


    index += j9pdn_(Context, index, 0x01654 >> 2, 0x00200000, 1, J9_YARELY, J9_YARELY);

    if (hasSecurity || hasRobustness)
    {
        index += j9pdn_(Context, index, 0x001AC >> 2, ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 16:16) - (0 ?
 16:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 16:16) - (0 ?
 16:16) + 1))))))) << (0 ?
 16:16))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 16:16) - (0 ?
 16:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 16:16) - (0 ? 16:16) + 1))))))) << (0 ? 16:16))), 1, J9_YARELY, J9_YARELY);
    }


    index += j9_ungroundable(Context, index);
#endif




    Context->linkIndex3D = (jmtUINT)index;

    if (buffer != J9_CHYAK)
    {
        buffer[index + 0]
            = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x08 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

        buffer[index + 1]
            = 0;
    }

    index += 2;


    Context->bufferSize = index * J9_NONPROS(jmtUINT32);






    Context->entryOffsetXDFrom2D = (jmtUINT)index * J9_NONPROS(jmtUINT32);


    index += j9_ginglymi(Context, index, J9_SYNAPTAI);


    Context->linkIndexXD = (jmtUINT)index;

    if (buffer != J9_CHYAK)
    {
        buffer[index + 0]
            = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x08 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

        buffer[index + 1]
            = 0;
    }

    index += 2;





    Context->totalSize = index * J9_NONPROS(jmtUINT32);


    JMM_kFOOTER_NO();
    return J9_FLUTTERING;

}

static j9_duopoly
j9_handle_j9m_centermost(
    IN jmk_CONTEXT Context
    )
{
    jmtUINT32_PTR buffer = J9_CHYAK;
    jmtUINT32 index;

#if J9_DEMISSNESS
    jmtBOOL halti0, halti1, halti2, halti3, halti4, halti5;
    jmtUINT i;
    jmtUINT vertexUniforms, fragmentUniforms;
    jmtBOOL unifiedUniform;
    jmtBOOL hasGS, hasTS;
    jmtBOOL genericAttrib;
    jmtBOOL hasICache;
    jmtBOOL hasICachePrefetch;
    jmtUINT numRT = 0;
    jmtUINT numSamplers = 32;
    jmtBOOL hasTXdesc;
    jmtBOOL hasSecurity;
    jmtBOOL hasRobustness;
    jmtBOOL multiCluster;
    jmtBOOL smallBatch;
    jmtBOOL multiCoreBlockSetCfg2;
    jmtUINT clusterAliveMask[J9_HANDLE_J9M_PIPEFISHES];
    jmtBOOL hasPSCSThrottle;
    jmtBOOL hasMsaaFragOperation;
    jmtBOOL newGPipe;
    jmtBOOL computeOnly;
#if J9_PRISM && J9_ERRONEOUSLY
    jmtUINT PEFenceEndianControl;
#endif
#endif

    jmk_HARDWARE hardware;

    j9_quincunx();

    hardware = Context->hardware;

    j9maths_activation(hardware, J9_CONTRACEPTIVE);

    if (!hardware->options.hasShader)
    {
        return j9_handle_j9_attribute_interester(Context);
    }


    index = 0;


    Context->lastAddress = ~0U;


    buffer = (Context->buffer == J9_CHYAK)
        ? J9_CHYAK
        : Context->buffer->logical;





#if !J9_HANDLE__UNFEMINISE
    if (jmkHARDWARE_IsFeatureAvailable(hardware, J9MIRROR_RHYMESTERS))
    {

        index += j9_voltize(Context, index, J9_SYNAPTAI);


        index += j9_ginglymi(Context, index, J9_GUNSMITH);

        index += j9pdn_(Context, index, 0x00670 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);


        index += j9pdn_(Context, index, 0x01200 >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, 0x01204 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01208 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x0120C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01210 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01214 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01218 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x0121C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);


        index += j9pdn_(Context, index, 0x01220 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01224 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);


        index += j9pdn_(Context, index, 0x01228 >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, 0x0122C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01230 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01234 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);


        index += j9pdn_(Context, index, 0x01238 >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, 0x01240 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01244 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01248 >> 2, 0xFFFFFFFF, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x0124C >> 2, 0xFFFFFFFF, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01250 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01254 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x0123C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);


        index += j9pdn_(Context, index, 0x01258 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01800 >> 2, 0x00000000, 128, J9_YARELY, J9_YARELY);


        index += j9pdn_(Context, index, 0x01C00 >> 2, 0x00000000, 256, J9_YARELY, J9_YARELY);


        index += j9pdn_(Context, index, 0x0125C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01260 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01264 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01268 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01270 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01274 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x0126C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01278 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);


        index += j9pdn_(Context, index, 0x0127C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01280 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);


        if (jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_J9M_ABRIDGABLE))
        {
            index += j9pdn_(Context, index, 0x01284 >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
            index += j9pdn_(Context, index, 0x01288 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x0128C >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
            index += j9pdn_(Context, index, 0x01290 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x01298 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x0129C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012A0 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012A4 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012A8 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012AC >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012E4 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        }


        if (jmkHARDWARE_IsFeatureAvailable(hardware, J9MATHS_ACETARSONE))
        {
            index += j9pdn_(Context, index, 0x012B0 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012B4 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012B8 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012BC >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012C0 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012C4 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012C8 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012CC >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012D0 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012D4 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012D8 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012DC >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012E0 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x03400 >> 2, 0x00000000, 256, J9_YARELY, J9_YARELY);
        }


        if (jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_J9_ATTRIBUTE_BEHAVIOUR_BALLBUSTER))
        {
            index += j9pdn_(Context, index, 0x012E8 >> 2, 0xFFFFFFFF, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012EC >> 2, 0xFFFFFFFF, 1, J9_YARELY, J9_YARELY);
        }


        if (jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_ATTRIBUTE_POLYGAMIAN))
        {
            index += j9pdn_(Context, index, 0x02800 >> 2, 0x00000000, 128, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x02A00 >> 2, 0x00000000, 128, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012F0 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012F4 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012F8 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x012FC >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x01300 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x01304 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        }

        index += J9_ARDENNITE();

        Context->linkIndex2D = index;
        index += j9_maad(Context, index);
    }
#endif

#if J9_DEMISSNESS



    halti0 = jmkHARDWARE_IsFeatureAvailable(hardware, J9MATHS_POLYGRAPHY);
    halti1 = jmkHARDWARE_IsFeatureAvailable(hardware, J9MATHS_ENTONEMENT);
    halti2 = jmkHARDWARE_IsFeatureAvailable(hardware, J9MATHS_ROSANILINE);
    halti3 = jmkHARDWARE_IsFeatureAvailable(hardware, J9MATHS_TERNARIANT);
    halti4 = jmkHARDWARE_IsFeatureAvailable(hardware, J9MATHS_ABSTINENCE);
    halti5 = jmkHARDWARE_IsFeatureAvailable(hardware, J9MATHS_CORPORALCY);
    hasGS  = jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_J9MENU_PREPROMISE);
    hasTS  = jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_J9M_CONQUERING);
    genericAttrib = jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_J9MIN_FORGATHERS);
    hasICache = jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_ATTRIBUTE_BEHAVIOUR_REMODIFIED);
    hasTXdesc = jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_J9MA_DULCIGENIC);
    hasSecurity = jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_PREREJOICE);
    hasRobustness = jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_J_ECSTATICAL);
    hasICachePrefetch = jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_J9MA_ATTRIBUTE_COMMENCING);
    multiCluster = jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_J9MA_CORROSIVES);
    smallBatch = jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_J9_NONJURIDIC) && hardware->options.smallBatch;
    multiCoreBlockSetCfg2 = jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_ATTRIBUTE_BEHAVIOUR_OXAMETHANE);
    hasPSCSThrottle = jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_J9MA_HISTOLOGIC);
    hasMsaaFragOperation = jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_J9MA_ATTRIBUTE_BANDCUTTER);
    newGPipe = jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE__THERMOGENY);
    computeOnly = jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_J9M_DELIQUESCE);
#if J9_PRISM && J9_ERRONEOUSLY
    PEFenceEndianControl = jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_J9_NONMELODIC)
                                                           ? 0x3
                                                           : 0x2;
#endif

    for(i = 0; i < J9_HANDLE_J9M_PIPEFISHES; i++)
    {
        clusterAliveMask[i] = hardware->identity.clusterAvailMask & hardware->options.userClusterMasks[i];
    }


    if (Context->hardware->identity.chipModel == J9_AJOG &&
        Context->hardware->identity.j9_exquisite == 0x5124 &&
        Context->hardware->identity.j9_nascent == 0x103)
    {
        numRT = 16;
    }
    else if (halti2 ||
            ((Context->hardware->identity.chipModel == J9_BOSK) &&
            (Context->hardware->identity.j9_exquisite == 0x5250)))
    {
        numRT = 8;
    }
    else if (halti0)
    {
        numRT = 4;
    }
    else
    {
        numRT = 1;
    }

    if (hasGS && hasTS)
    {
        numSamplers = 80;
    }


    J9_HANDLE_J_DEEDHOLDER(Context->hardware->identity.chipModel,
                         Context->hardware->identity.j9_exquisite,
                         halti5,
                         smallBatch,
                         computeOnly,
                         Context->hardware->identity.numConstants,
                         unifiedUniform,
                         vertexUniforms,
                         fragmentUniforms);

#if !J9_HANDLE_J9MENU_MUSCOLOGIC
    if (Context->hardware->identity.numConstants > 256)
    {
        unifiedUniform = J9_CUPPY;
    }
    else
    {
        unifiedUniform = J9_YARELY;
    }
#endif


    Context->entryOffset3D = (jmtUINT)index * J9_NONPROS(jmtUINT32);


    index += j9_ginglymi(Context, index, J9_SYNAPTAI);

    if (multiCluster)
    {
        for (i = 0; i < hardware->kernel->device->coreNum; i++)
        {
            index += j9pdn_(Context, index, (0x03910 >> 2) + i, ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 7:0) - (0 ?
 7:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 7:0) - (0 ?
 7:0) + 1))))))) << (0 ?
 7:0))) | (((jmtUINT32) ((jmtUINT32) (clusterAliveMask[i]) & ((jmtUINT32) ((((1 ?
 7:0) - (0 ?
 7:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 7:0) - (0 ? 7:0) + 1))))))) << (0 ? 7:0))), 1, J9_YARELY, J9_YARELY);
        }

        index += j9pdn_(Context, index, 0x03908 >> 2, ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 2:0) - (0 ?
 2:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 2:0) - (0 ?
 2:0) + 1))))))) << (0 ?
 2:0))) | (((jmtUINT32) (0x2 & ((jmtUINT32) ((((1 ?
 2:0) - (0 ?
 2:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 2:0) - (0 ? 2:0) + 1))))))) << (0 ? 2:0))), 1, J9_YARELY, J9_YARELY);
    }


#if J9_NOONED && 1
    index += j9pdn_(Context, index, 0x03850 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
#endif

    index += j9_voltize(Context, index, J9_SYNAPTAI);


    if (hasSecurity)
    {
        index += j9pdn_(Context, index, 0x03900 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += J9_ARDENNITE();
        index += j9pdn_(Context, index, 0x03904 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    }

    index += j9pdn_(Context, index, 0x03814 >> 2, 0x00000001, 1, J9_YARELY, J9_YARELY);
    index += J9_ARDENNITE();
    index += j9pdn_(Context, index, 0x03818 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x0381C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);

    if (halti5)
    {
        jmtUINT32 uscControl = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 20:16) - (0 ?
 20:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 20:16) - (0 ?
 20:16) + 1))))))) << (0 ?
 20:16))) | (((jmtUINT32) ((jmtUINT32) (2) & ((jmtUINT32) ((((1 ?
 20:16) - (0 ?
 20:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 20:16) - (0 ? 20:16) + 1))))))) << (0 ? 20:16)));
        index += j9pdn_(Context, index, 0x03888 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x038C0 >> 2, 0x00000000, 16, J9_YARELY, J9_YARELY);

        uscControl |= ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 2:0) - (0 ?
 2:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 2:0) - (0 ?
 2:0) + 1))))))) << (0 ?
 2:0))) | (((jmtUINT32) ((jmtUINT32) (hardware->options.uscL1CacheRatio) & ((jmtUINT32) ((((1 ?
 2:0) - (0 ?
 2:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 2:0) - (0 ? 2:0) + 1))))))) << (0 ? 2:0)));
        if (multiCluster)
        {
            uscControl |= ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 11:8) - (0 ?
 11:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 11:8) - (0 ?
 11:8) + 1))))))) << (0 ?
 11:8))) | (((jmtUINT32) ((jmtUINT32) (hardware->options.uscAttribCacheRatio) & ((jmtUINT32) ((((1 ?
 11:8) - (0 ?
 11:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 11:8) - (0 ? 11:8) + 1))))))) << (0 ? 11:8)));
        }
        index += j9pdn_(Context, index, 0x03884 >> 2, uscControl, 1, J9_YARELY, J9_YARELY);
    }
    else
    {
        index += j9pdn_(Context, index, 0x03820 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x03828 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x0382C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x03834 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x03838 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x03854 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    }

    if (hasGS)
    {
        index += j9pdn_(Context, index, 0x0388C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    }

    index += j9pdn_(Context, index, 0x0384C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);


    if (halti5)
    {
        index += j9pdn_(Context, index, 0x17800 >> 2, 0x00000000, 32, J9_YARELY, J9_YARELY);
        index += J9_ARDENNITE();
        index += j9pdn_(Context, index, 0x007C4 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x007D0 >> 2, 0x00000000, 2, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x007D8 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x17A80 >> 2, 0x00000000, 32, J9_YARELY, J9_YARELY);
        if (genericAttrib || newGPipe)
        {
            index += j9pdn_(Context, index, 0x17880 >> 2, 0x00000000, 32, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x17900 >> 2, 0x00000000, 32, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x17980 >> 2, 0x00000000, 32, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x17A00 >> 2, 0x3F800000, 32, J9_YARELY, J9_YARELY);
        }
    }
    else
    {
        index += j9pdn_(Context, index, 0x00600 >> 2, 0x00000000, (halti0 ? 16 : 12), J9_YARELY, J9_YARELY);
        index += J9_ARDENNITE();
        if (genericAttrib)
        {
            index += j9pdn_(Context, index, 0x006C0 >> 2, 0x00000000, 16, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x00700 >> 2, 0x00000000, 16, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x00740 >> 2, 0x00000000, 16, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x00780 >> 2, 0x3F800000, 16, J9_YARELY, J9_YARELY);
        }
    }

    if (halti2 || (Context->hardware->identity.streamCount > 8))
    {
        index += j9pdn_(Context, index, 0x14600 >> 2, 0x00000000, 16, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x14640 >> 2, 0x00000000, 16, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x14680 >> 2, 0x00000000, 16, J9_YARELY, J9_YARELY);
    }
    else if (Context->hardware->identity.streamCount > 1)
    {
        index += j9pdn_(Context, index, 0x00680 >> 2, 0x00000000, 8, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, 0x006A0 >> 2, 0x00000000, 8, J9_YARELY, J9_YARELY);
    }
    else
    {
        index += j9pdn_(Context, index, 0x0064C >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, 0x00650 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);

    }
    index += j9pdn_(Context, index, 0x00644 >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
    index += j9pdn_(Context, index, 0x00648 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00674 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);

    if (halti1)
    {
        index += j9pdn_(Context, index, 0x00678 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x0067C >> 2, 0xFFFFFFFF, 1, J9_YARELY, J9_YARELY);
    }

    index += J9_ARDENNITE();

    if (hasRobustness)
    {
        index += j9pdn_(Context, index, 0x146C0 >> 2, 0x00000000, 16, J9_YARELY, J9_CUPPY);
        index += J9_ARDENNITE();
        index += j9pdn_(Context, index, 0x007F8 >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
        index += J9_ARDENNITE();
    }


    if (multiCluster)
    {

        index += j9pdn_(Context, index, 0x18404 >> 2, ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 1:0) - (0 ?
 1:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 1:0) - (0 ?
 1:0) + 1))))))) << (0 ?
 1:0))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 1:0) - (0 ?
 1:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 1:0) - (0 ? 1:0) + 1))))))) << (0 ? 1:0))), 1, J9_YARELY, J9_YARELY);
    }

    if (halti5)
    {
        index += j9pdn_(Context, index, 0x008B8 >> 2, ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:4) - (0 ?
 4:4) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:4) - (0 ?
 4:4) + 1))))))) << (0 ?
 4:4))) | (((jmtUINT32) ((jmtUINT32) (smallBatch ?
 0x0 : 0x1) & ((jmtUINT32) ((((1 ?
 4:4) - (0 ?
 4:4) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:4) - (0 ?
 4:4) + 1))))))) << (0 ?
 4:4))) | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 5:5) - (0 ?
 5:5) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 5:5) - (0 ?
 5:5) + 1))))))) << (0 ?
 5:5))) | (((jmtUINT32) ((jmtUINT32) (smallBatch ?
 0x0 : 0x1) & ((jmtUINT32) ((((1 ?
 5:5) - (0 ?
 5:5) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 5:5) - (0 ?
 5:5) + 1))))))) << (0 ?
 5:5))) | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 8:8) - (0 ?
 8:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 8:8) - (0 ?
 8:8) + 1))))))) << (0 ?
 8:8))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 8:8) - (0 ?
 8:8) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 8:8) - (0 ?
 8:8) + 1))))))) << (0 ?
 8:8))) | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 9:9) - (0 ?
 9:9) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 9:9) - (0 ?
 9:9) + 1))))))) << (0 ?
 9:9))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 9:9) - (0 ?
 9:9) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 9:9) - (0 ? 9:9) + 1))))))) << (0 ? 9:9))), 1, J9_YARELY, J9_YARELY);

        index += j9pdn_(Context, index, 0x15600 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    }
    else
    {
        index += j9pdn_(Context, index, 0x00860 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    }

    if (hasICache)
    {

        index += j9pdn_(Context, index, 0x00868 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x0086C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x0304C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01028 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += J9_ARDENNITE();

        if (hasICachePrefetch)
        {
            if (halti5)
            {
                index += j9pdn_(Context, index, 0x15604 >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
                index += j9pdn_(Context, index, 0x01094 >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
            }
            else
            {
                index += j9pdn_(Context, index, 0x00890 >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
                index += j9pdn_(Context, index, 0x0104C >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
            }
            index += J9_ARDENNITE();
        }
    }

    if (multiCluster)
    {

        index += j9pdn_(Context, index, 0x010A8 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    }


    index += j9pdn_(Context, index, 0x00804 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00808 >> 2, ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 5:0) - (0 ?
 5:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 5:0) - (0 ?
 5:0) + 1))))))) << (0 ?
 5:0))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 5:0) - (0 ?
 5:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 5:0) - (0 ? 5:0) + 1))))))) << (0 ? 5:0))), 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x0080C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00830 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);

    if (halti5)
    {
        index += j9pdn_(Context, index, 0x00898 >> 2, 0x00000000, 2, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x008A0 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x00870 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x008A8 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x008C0 >> 2, 0x00000000, 8, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x008E0 >> 2, 0x00000000, 8, J9_YARELY, J9_YARELY);
    }
    else
    {
        index += j9pdn_(Context, index, 0x00810 >> 2, 0x00000000, 4, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x00820 >> 2, 0x00000000, 4, J9_YARELY, J9_YARELY);
    }

    if (multiCluster)
    {
        index += j9pdn_(Context, index, 0x007FC >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x15608 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    }

    index += J9_ARDENNITE();


    if (hasGS)
    {
        index += j9pdn_(Context, index, 0x01100 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01104 >> 2, 0x00000001, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01108 >> 2, 0x01000001, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x0110C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01110 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01114 >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, 0x0111C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01140 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x0115C >> 2, 0x000000FF, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01144 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01148 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x0114C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01154 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01120 >> 2, 0x00000000, 8, J9_YARELY, J9_YARELY);
        index += J9_ARDENNITE();
    }


    if (hasTS)
    {
        index += j9pdn_(Context, index, 0x007C0 >> 2, 0x00000003, 1, J9_YARELY, J9_YARELY);

        index += j9pdn_(Context, index, 0x14A14 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x14A18 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x14A1C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x14A40 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x14A00 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x14A04 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x14A08 >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, 0x14A10 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x14A20 >> 2, 0x00000000, 8, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x14A44 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x14A4C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);

        index += J9_ARDENNITE();

        index += j9pdn_(Context, index, 0x14B18 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x14B1C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x14B20 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x14B04 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x14B08 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x14B0C >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, 0x14B14 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x14B40 >> 2, 0x00000000, 8, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x14B24 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x14B2C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x14B34 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);

        index += j9pdn_(Context, index, 0x14B00 >> 2, 0x00040000, 1, J9_YARELY, J9_YARELY);

    }

    index += J9_ARDENNITE();


    if (jmkHARDWARE_IsFeatureAvailable(Context->hardware, J9MATHS_TRANSMUTES))
    {
        index += j9pdn_(Context, index, 0x1C000 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x1C008 >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, (0x1C040 >> 2) + (0 << 4), 0x00000000, 4, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, 0x1C080 >> 2, 0x00000000, 4, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x1C0C0 >> 2, 0x00000000, 4, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x1C100 >> 2, 0x00000000, 4, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x1C800 >> 2, 0x00000000, 128*4, J9_YARELY, J9_YARELY);

        index += j9pdn_(Context, index, 0x1C014 >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
        index += J9_ARDENNITE();

    }


    index += j9pdn_(Context, index, 0x00A00 >> 2, 0x00000000, 1, J9_CUPPY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00A04 >> 2, 0x00000000, 1, J9_CUPPY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00A08 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00A0C >> 2, 0x00000000, 1, J9_CUPPY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00A10 >> 2, 0x00000000, 1, J9_CUPPY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00A14 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00A18 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00A1C >> 2, 0x3F000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00A28 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00A2C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00A30 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00A34 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00A38 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00A3C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00A80 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00A84 >> 2, 0x00000000, 1, J9_CUPPY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00A8C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00A88 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);

    if (halti5)
    {
        index += j9pdn_(Context, index, 0x00AA8 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x00A90 >> 2, 0x00000000, 4, J9_YARELY, J9_YARELY);
    }
    else
    {
        index += j9pdn_(Context, index, 0x00A40 >> 2, 0x00000000, Context->hardware->identity.varyingsCount, J9_YARELY, J9_YARELY);
    }

    if (multiCluster)
    {

        index += j9pdn_(Context, index, 0x00AAC >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
		index += j9pdn_(Context, index, 0x03A00 >> 2, ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 2:0) - (0 ?
 2:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 2:0) - (0 ?
 2:0) + 1))))))) << (0 ?
 2:0))) | (((jmtUINT32) (0x7 & ((jmtUINT32) ((((1 ?
 2:0) - (0 ?
 2:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 2:0) - (0 ?
 2:0) + 1))))))) << (0 ?
 2:0))) | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:28) - (0 ?
 31:28) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:28) - (0 ?
 31:28) + 1))))))) << (0 ?
 31:28))) | (((jmtUINT32) ((jmtUINT32) (0xf) & ((jmtUINT32) ((((1 ?
 31:28) - (0 ?
 31:28) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:28) - (0 ? 31:28) + 1))))))) << (0 ? 31:28))), 1, J9_YARELY, J9_YARELY);
    }
	else
	{
    	index += j9pdn_(Context, index, 0x03A00 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
	}

    index += j9pdn_(Context, index, 0x03A04 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x03A08 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);

    if (multiCoreBlockSetCfg2)
    {
        index += j9pdn_(Context, index, 0x03A0C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x03A10 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    }


    index += j9pdn_(Context, index, 0x00C00 >> 2, 0x00000000, 1, J9_CUPPY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00C04 >> 2, 0x00000000, 1, J9_CUPPY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00C08 >> 2, 0x45000000, 1, J9_CUPPY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00C0C >> 2, 0x45000000, 1, J9_CUPPY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00C10 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00C14 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00C18 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00C1C >> 2, 0x42000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00C20 >> 2, 0x00000000, 1, J9_CUPPY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00C24 >> 2, 0x00000000, 1, J9_CUPPY, J9_YARELY);


    index += j9pdn_(Context, index, 0x00E00 >> 2, 0x000000F1, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00E10 >> 2, 0x00000000, 4, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00E04 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00E40 >> 2, 0x00000000, 16, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00E08 >> 2, ((((jmtUINT32) (0x17000031)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 2:2) - (0 ?
 2:2) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 2:2) - (0 ?
 2:2) + 1))))))) << (0 ?
 2:2))) | (((jmtUINT32) ((jmtUINT32) (smallBatch ?
 0x0 : 0x1) & ((jmtUINT32) ((((1 ?
 2:2) - (0 ?
 2:2) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 2:2) - (0 ? 2:2) + 1))))))) << (0 ? 2:2))), 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00E24 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00E20 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);

    if (halti2)
    {
        index += j9pdn_(Context, index, 0x00E0C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    }

    if (halti5)
    {
        index += j9pdn_(Context, index, 0x00E34 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    }



    index += j9pdn_(Context, index, 0x01004 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x01008 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x0100C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x01010 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x01030 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x01034 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);

    if (halti2)
    {
        index += j9pdn_(Context, index, 0x01040 >> 2, 0x00000000, 2, J9_YARELY, J9_YARELY);
    }

    if (numRT == 16)
    {
        index += j9pdn_(Context, index, 0x0102C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x010C8 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x010CC >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    }
    else if (numRT == 8)
    {
        index += j9pdn_(Context, index, 0x0102C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01038 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    }

    if (hasMsaaFragOperation)
    {
        index += j9pdn_(Context, index, 0x01054 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01060 >> 2, 0x00000000, 8, J9_YARELY, J9_YARELY);
    }

    if (halti5)
    {
        index += j9pdn_(Context, index, 0x01080 >> 2, 0x00000000, 4, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01058 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01098 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    }

    if (hasPSCSThrottle)
    {
        index += j9pdn_(Context, index, 0x0109C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    }

    index += J9_ARDENNITE();


    if (hasTXdesc)
    {

        index += j9pdn_(Context, index, 0x14C40 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);

        if (smallBatch)
        {
            index += j9pdn_(Context, index, 0x010B0 >> 2, numSamplers, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x010B4 >> 2, numSamplers, 1, J9_YARELY, J9_YARELY);

            index += j9pdn_(Context, index, 0x16000 >> 2, 0x00000000, numSamplers, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x16200 >> 2, 0x00000000, numSamplers, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x16400 >> 2, 0x00000000, numSamplers, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x16600 >> 2, 0x00000000, numSamplers, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x16800 >> 2, 0x00000000, numSamplers, J9_YARELY, J9_YARELY);

            index += j9pdn_(Context, index, (0x15800 >> 2) + (0 << 0), 0x00000000, numSamplers, J9_YARELY, J9_CUPPY);
            index += j9pdn_(Context, index, 0x15A00 >> 2, 0x00000000, numSamplers, J9_YARELY, J9_YARELY);

            index += J9_ARDENNITE();
        }
        else
        {
            index += j9pdn_(Context, index, 0x16C00 >> 2, 0x00000000, numSamplers, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x16E00 >> 2, 0x00000000, numSamplers, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x17000 >> 2, 0x00000000, numSamplers, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x17200 >> 2, 0x00000000, numSamplers, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x17400 >> 2, 0x00000000, numSamplers, J9_YARELY, J9_YARELY);

            index += j9pdn_(Context, index, (0x15C00 >> 2) + (0 << 0), 0x00000000, numSamplers, J9_YARELY, J9_CUPPY);
            index += j9pdn_(Context, index, 0x15E00 >> 2, 0x00000000, numSamplers, J9_YARELY, J9_YARELY);

            index += J9_ARDENNITE();

            j9_packboard(Context, 0x16000 >> 2, numSamplers , 0x16C00 >> 2);
            j9_packboard(Context, 0x16200 >> 2, numSamplers , 0x16E00 >> 2);
            j9_packboard(Context, 0x16400 >> 2, numSamplers , 0x17000 >> 2);
            j9_packboard(Context, 0x16600 >> 2, numSamplers , 0x17200 >> 2);
            j9_packboard(Context, 0x16800 >> 2, numSamplers , 0x17400 >> 2);
            j9_packboard(Context, 0x15800 >> 2, numSamplers , 0x15C00 >> 2);
            j9_packboard(Context, 0x15A00 >> 2, numSamplers , 0x15E00 >> 2);
        }
    }
    else
    {
        index += j9pdn_(Context, index, 0x02000 >> 2, 0x00000000, 12, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x02040 >> 2, 0x00000000, 12, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x02080 >> 2, 0x00000000, 12, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x020C0 >> 2, 0x00000000, 12, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x02100 >> 2, 0x00000000, 12, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x02140 >> 2, 0x00000000, 12, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x02180 >> 2, 0x00000000, 12, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x021C0 >> 2, 0x00321000, 12, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x02200 >> 2, 0x00000000, 12, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x02240 >> 2, 0x00000000, 12, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, (0x02400 >> 2) + (0 << 4), 0x00000000, 12, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, (0x02440 >> 2) + (0 << 4), 0x00000000, 12, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, (0x02480 >> 2) + (0 << 4), 0x00000000, 12, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, (0x024C0 >> 2) + (0 << 4), 0x00000000, 12, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, (0x02500 >> 2) + (0 << 4), 0x00000000, 12, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, (0x02540 >> 2) + (0 << 4), 0x00000000, 12, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, (0x02580 >> 2) + (0 << 4), 0x00000000, 12, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, (0x025C0 >> 2) + (0 << 4), 0x00000000, 12, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, (0x02600 >> 2) + (0 << 4), 0x00000000, 12, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, (0x02640 >> 2) + (0 << 4), 0x00000000, 12, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, (0x02680 >> 2) + (0 << 4), 0x00000000, 12, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, (0x026C0 >> 2) + (0 << 4), 0x00000000, 12, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, (0x02700 >> 2) + (0 << 4), 0x00000000, 12, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, (0x02740 >> 2) + (0 << 4), 0x00000000, 12, J9_YARELY, J9_CUPPY);
        index += J9_ARDENNITE();

        if (jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_J9MIN_ANCHORITIC))
        {
            jmtUINT count = halti1 ? 14 : 1;

            for (i = 0; i < 12; i += 1)
            {
                index += j9pdn_(Context, index, (0x02C00 >> 2) + i * 16, 0x00000000, count, J9_YARELY, J9_YARELY);
            }
        }

        if (jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE__ATTRIBUTE_DOWAGERISM))
        {
            jmtUINT texBlockCount;
            jmtUINT mwv207regTXLogSizeResetValue;

            mwv207regTXLogSizeResetValue = ((((jmtUINT32) (0x00000000)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 29:29) - (0 ?
 29:29) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 29:29) - (0 ?
 29:29) + 1))))))) << (0 ?
 29:29))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 29:29) - (0 ?
 29:29) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 29:29) - (0 ? 29:29) + 1))))))) << (0 ? 29:29)));


            index += j9pdn_(Context, index, 0x10000 >> 2, 0x00000000, 32, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x10080 >> 2, 0x00000000, 32, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x10100 >> 2, mwv207regTXLogSizeResetValue, 32, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x10180 >> 2, 0x00000000, 32, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x10200 >> 2, 0x00000000, 32, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x10280 >> 2, 0x00000000, 32, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x10300 >> 2, 0x00000000, 32, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x10380 >> 2, 0x00321000, 32, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x10400 >> 2, 0x00000000, 32, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x10480 >> 2, 0x00000000, 32, J9_YARELY, J9_YARELY);

            if (jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE__GLOSSINESS))
            {
                index += j9pdn_(Context, index, 0x12000 >> 2, 0x00000000, 256, J9_YARELY, J9_YARELY);
                index += j9pdn_(Context, index, 0x12400 >> 2, 0x00000000, 256, J9_YARELY, J9_YARELY);
            }

            texBlockCount = ((512) >> (4));

            for (i = 0; i < texBlockCount; i += 1)
            {
                index += j9pdn_(Context, index, (0x10800 >> 2) + (i << 4), 0x00000000, 14, J9_YARELY, J9_CUPPY);
            }
        }

        if (jmkHARDWARE_IsFeatureAvailable(Context->hardware, J9_HANDLE_J9_SKITTYBOOT))
        {
            index += j9pdn_(Context, index, 0x10700 >> 2, 0x00000F00, 32, J9_YARELY, J9_YARELY);
        }

        if (halti3 ||
            jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_J9MIN_REDOUBTING))
        {
            index += j9pdn_(Context, index, 0x10780 >> 2, 0x00030000, 32, J9_YARELY, J9_YARELY);
        }

        if (halti4)
        {
            index += j9pdn_(Context, index, 0x11200 >> 2, 0x00000000, 32, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x11280 >> 2, 0x00000000, 32, J9_YARELY, J9_YARELY);
        }

        if (jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_J9M_ATTRIBUTE_LAMENESSES))
        {
            index += j9pdn_(Context, index, 0x11000 >> 2, 0x00000000, 32, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x11080 >> 2, 0x00000000, 32, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x11100 >> 2, 0x00000000, 32, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x11180 >> 2, 0x00000000, 32, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x11300 >> 2, 0x00000000, 32, J9_YARELY, J9_YARELY);
        }


        if (jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_J9M_SORDIDNESS))
        {
            index += j9pdn_(Context, index, 0x10500 >> 2, 0x00000000, 32, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x10580 >> 2, 0x00000000, 32, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x10600 >> 2, 0x00000000, 32, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x10680 >> 2, 0x00000000, 32, J9_YARELY, J9_YARELY);
        }
    }

    if (halti3)
    {
        index += j9pdn_(Context, index, 0x14C00 >> 2, 0x00000000, 16, J9_YARELY, J9_YARELY);
    }


    index += j9pdn_(Context, index, 0x00900 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00904 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00908 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x0090C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00910 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00914 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00918 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x00924 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x0091C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);

    if (jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_J_ATTRIBUTE_PANSOPHIES))
    {
        index += j9pdn_(Context, index, 0x00940 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x00944 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x00948 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x0094C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x00950 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x00954 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    }

    if (halti5)
    {
        index += j9pdn_(Context, index, 0x00958 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x0095C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x00960 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    }

    index += J9_ARDENNITE();


    if (halti5)
    {
        index += j9pdn_(Context, index, 0x00874 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x008BC >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x0087C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x01090 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += J9_ARDENNITE();
    }
    else if (hasICache)
    {

        index += j9pdn_(Context, index, 0x00874 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x00878 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x0087C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x00880 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += J9_ARDENNITE();
    }
    else
    {
        if (Context->hardware->identity.instructionCount <= 256)
        {

            index += j9pdn_(Context, index, 0x00800 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x00838 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += J9_ARDENNITE();

            index += j9pdn_(Context, index, 0x01000 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x01018 >> 2, 0x01000000, 1, J9_YARELY, J9_YARELY);
            index += J9_ARDENNITE();
        }
        else
        {

            index += j9pdn_(Context, index, 0x0085C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
            index += j9pdn_(Context, index, 0x0101C >> 2, 0x00000100, 1, J9_YARELY, J9_YARELY);
            index += J9_ARDENNITE();
        }
    }


    if (!hasICachePrefetch)
    {

        if (Context->hardware->identity.instructionCount > 1024)
        {
            for (i = 0;
                 i < Context->hardware->identity.instructionCount << 2;
                 i += 256 << 2
                 )
            {
                index += j9pdn_(Context, index, (0x20000 >> 2) + i, 0x00000000, 256 << 2, J9_YARELY, J9_YARELY);
                index += J9_ARDENNITE();
            }
        }

        else if (Context->hardware->identity.instructionCount > 256)
        {

            for (i = 0;
                 i < Context->hardware->identity.instructionCount << 2;
                 i += 256 << 2
                 )
            {
                index += j9pdn_(Context, index, (0x0C000 >> 2) + i, 0x00000000, 256 << 2, J9_YARELY, J9_YARELY);
                index += J9_ARDENNITE();
            }

            j9_packboard(Context, 0x08000 >> 2, Context->hardware->identity.instructionCount << 2 , 0x0C000 >> 2);
        }

        else
        {
            if (Context->hardware->identity.PSInstructionCount > Context->hardware->identity.instructionCount)
            {
                JMM_kASSERT(Context->hardware->identity.PSInstructionCount % 256 == 0);


                for (i = 0;
                     i < Context->hardware->identity.PSInstructionCount << 2;
                     i += 256 << 2
                     )
                {
                    index += j9pdn_(Context, index, (0x08000 >> 2) + i, 0x00000000, 256 << 2, J9_YARELY, J9_YARELY);
                    index += J9_ARDENNITE();
                }
            }
            else
            {
                index += j9pdn_(Context, index, 0x06000 >> 2, 0x00000000, 1024, J9_YARELY, J9_YARELY);
                index += J9_ARDENNITE();
            }

            index += j9pdn_(Context, index, 0x04000 >> 2, 0x00000000, 1024, J9_YARELY, J9_YARELY);
            index += J9_ARDENNITE();
        }
    }

    if (unifiedUniform)
    {
        jmtINT numConstants = Context->hardware->identity.numConstants;


        index += j9pdn_(Context, index, 0x01024 >> 2, 0x00000100, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x00864 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += J9_ARDENNITE();

        if (smallBatch)
        {
            index += j9pdn_(Context, index, 0x010AC >> 2, numConstants, 1, J9_YARELY, J9_YARELY);
        }

        for (i = 0;
             numConstants > 0;
             i += 256 << 2,
             numConstants -= 256
             )
        {
            if (halti5)
            {
                if (numConstants >= 256)
                {
                    if (smallBatch)
                    {
                        index += j9pdn_(Context, index, (0x34000 >> 2) + i, 0x00000000, 256 << 2, J9_YARELY, J9_YARELY);
                    }
                    else
                    {
                        index += j9pdn_(Context, index, (0x36000 >> 2) + i, 0x00000000, 256 << 2, J9_YARELY, J9_YARELY);
                    }
                }
                else
                {
                    if (smallBatch)
                    {
                        index += j9pdn_(Context, index, (0x34000 >> 2) + i, 0x00000000, numConstants << 2, J9_YARELY, J9_YARELY);
                    }
                    else
                    {
                        index += j9pdn_(Context, index, (0x36000 >> 2) + i, 0x00000000, numConstants << 2, J9_YARELY, J9_YARELY);
                    }
                }
                index += J9_ARDENNITE();
            }
            else
            {
                if (numConstants >= 256)
                {
                    index += j9pdn_(Context, index, (0x30000 >> 2) + i, 0x00000000, 256 << 2, J9_YARELY, J9_YARELY);
                }
                else
                {
                    index += j9pdn_(Context, index, (0x30000 >> 2) + i, 0x00000000, numConstants << 2, J9_YARELY, J9_YARELY);
                }

                index += J9_ARDENNITE();
            }
        }

        if (halti5 && !smallBatch)
        {
            j9_packboard(Context, 0x34000 >> 2, Context->hardware->identity.numConstants << 2 , 0x36000 >> 2);
        }
    }
#if J9_HANDLE_J9MENU_MUSCOLOGIC
    else
#endif
    {
        index += j9pdn_(Context, index, 0x05000 >> 2, 0x00000000, vertexUniforms * 4, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x07000 >> 2, 0x00000000, fragmentUniforms * 4, J9_YARELY, J9_YARELY);
    }

    if (halti1)
    {
        index += j9pdn_(Context, index, 0x00884 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    }

    if (halti5)
    {

        index += j9pdn_(Context, index, 0x008B0 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    }


    Context->entryOffsetXDFrom3D = (jmtUINT)index * J9_NONPROS(jmtUINT32);



    index += j9pdn_(Context, index, 0x01400 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x01404 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x01408 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x0140C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x01414 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x01418 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x0141C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x01420 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x01424 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x01428 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x0142C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x01434 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x01454 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x01458 >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
    index += j9pdn_(Context, index, 0x014A0 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x014A8 >> 2, 0xFFFFFFFF, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x014AC >> 2, 0xFFFFFFFF, 1, J9_YARELY, J9_YARELY);

    if(jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_J9MENU_EVENTUATED) )
    {
        index += j9pdn_(Context, index, 0x014B0 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x014B4 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    }
    index += j9pdn_(Context, index, 0x014A4 >> 2, 0x000E400C, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x01580 >> 2, 0x00000000, 3, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x014B8 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);

    if (halti3)
    {
        index += j9pdn_(Context, index, 0x0103C >> 2, 0x76543210, 1, J9_YARELY, J9_YARELY);
    }

    index += j9pdn_(Context, index, (0x01460 >> 2) + (0 << 3), 0x00000000, Context->hardware->identity.pixelPipes, J9_YARELY, J9_CUPPY);

    if (Context->hardware->identity.pixelPipes == 1)
    {
        index += j9pdn_(Context, index, 0x01430 >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, 0x01410 >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
    }

    if (Context->hardware->identity.pixelPipes > 1 || halti0)
    {
        index += j9pdn_(Context, index, (0x01480 >> 2) + (0 << 3), 0x00000000, Context->hardware->identity.pixelPipes, J9_YARELY, J9_CUPPY);
    }

    for (i = 0; i < 3; i++)
    {
        index += j9pdn_(Context, index, (0x01500 >> 2) + (i << 3), 0x00000000, Context->hardware->identity.pixelPipes, J9_YARELY, J9_CUPPY);
    }

    if (numRT == 16)
    {
        for (i = 0; i < 15; i++)
        {
            index += j9pdn_(Context, index, (0x17C00 >> 2) + (i << 0), 0x00000000, Context->hardware->identity.pixelPipes, J9_YARELY, J9_CUPPY);
        }
        index += j9pdn_(Context, index, 0x17C40 >> 2, 0x00000000, 15, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x17C80 >> 2, 0x03012000, 15, J9_YARELY, J9_YARELY);
    }
    else if (numRT == 8)
    {
        for (i = 0; i < 7; i++)
        {
          index += j9pdn_(Context, index, (0x14800 >> 2) + (i << 3), 0x00000000, Context->hardware->identity.pixelPipes, J9_YARELY, J9_CUPPY);
        }
        index += j9pdn_(Context, index, 0x14900 >> 2, 0x00000000, 7, J9_YARELY, J9_YARELY);
    }


    if (halti3)
    {
#if J9_PRISM && J9_ERRONEOUSLY
        index += j9pdn_(Context, index, 0x014BC >> 2, ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 10:9) - (0 ?
 10:9) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 10:9) - (0 ?
 10:9) + 1))))))) << (0 ?
 10:9))) | (((jmtUINT32) ((jmtUINT32) (PEFenceEndianControl) & ((jmtUINT32) ((((1 ?
 10:9) - (0 ?
 10:9) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 10:9) - (0 ? 10:9) + 1))))))) << (0 ? 10:9))), 1, J9_YARELY, J9_YARELY);
#else
        index += j9pdn_(Context, index, 0x014BC >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
#endif
    }

    if (halti4)
    {
        index += j9pdn_(Context, index, 0x014C0 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    }

    if (hasGS)
    {
        index += j9pdn_(Context, index, 0x038A0 >> 2, 0x00000000, 8, J9_YARELY, J9_YARELY);
    }

    if (halti5)
    {
        index += j9pdn_(Context, index, 0x14920 >> 2, 0x00000000, 7, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x14940 >> 2, 0x00000000, 7, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x14960 >> 2, 0x00000000, 7, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x14980 >> 2, 0x00000000, 7, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x149A0 >> 2, 0x00000000, 7, J9_YARELY, J9_YARELY);
    }

    if (hasRobustness)
    {
        index += j9pdn_(Context, index, 0x149C0 >> 2, 0x00000000, 8, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, 0x014C4 >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
    }


    index += j9pdn_(Context, index, 0x01654 >> 2, 0x00200000, 1, J9_YARELY, J9_YARELY);

    index += J9_ARDENNITE();
    index += j9pdn_(Context, index, 0x01658 >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
    index += j9pdn_(Context, index, 0x0165C >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
    index += j9pdn_(Context, index, 0x01660 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x01664 >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
    index += j9pdn_(Context, index, 0x01668 >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
    index += j9pdn_(Context, index, 0x0166C >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x01670 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x01674 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x016A4 >> 2, 0x00000000, 1, J9_YARELY, J9_CUPPY);
    index += j9pdn_(Context, index, 0x016AC >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x016A8 >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x01720 >> 2, 0x00000000, 8, J9_YARELY, J9_YARELY);
    index += j9pdn_(Context, index, 0x01740 >> 2, 0x00000000, 8, J9_YARELY, J9_CUPPY);
    index += j9pdn_(Context, index, 0x01760 >> 2, 0x00000000, 8, J9_YARELY, J9_YARELY);


    if (halti2)
    {
        index += j9pdn_(Context, index, 0x01780 >> 2, 0x00000000, 8, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, 0x016BC >> 2, 0x00000000, 1, J9_YARELY, J9_YARELY);

        if (jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_J_ATTRIBUTE_OUTPURSUED))
        {
            index += j9pdn_(Context, index, 0x017A0 >> 2, 0x00000000, 8, J9_YARELY, J9_YARELY);
        }
        else
        {
            index += j9pdn_(Context, index, (0x017A0 >> 2) + 1, 0x00000000, 7, J9_YARELY, J9_YARELY);
        }

        index += j9pdn_(Context, index, (0x017C0 >> 2) + 1, 0x00000000, 7, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, (0x017E0 >> 2) + 1, 0x00000000, 7, J9_YARELY, J9_CUPPY);
        index += j9pdn_(Context, index, (0x01A00 >> 2) + 1, 0x00000000, 7, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, (0x01A20 >> 2) + 1, 0x00000000, 7, J9_YARELY, J9_YARELY);
        index += j9pdn_(Context, index, (0x01A40 >> 2) + 1, 0x00000000, 7, J9_YARELY, J9_YARELY);
    }

    index += J9_ARDENNITE();

    if (jmkHARDWARE_IsFeatureAvailable(hardware, J9_HANDLE_J9_INTRATOMIC))
    {
        index += j9pdn_(Context, index, 0x03860 >> 2, 0x6, 1, J9_YARELY, J9_YARELY);
        index += J9_ARDENNITE();
    }

    if (halti3)
    {
        index += j9pdn_(Context, index, 0x01A80 >> 2, 0x00000000, 8, J9_YARELY, J9_CUPPY);
        index += J9_ARDENNITE();
    }

    if (hasSecurity || hasRobustness)
    {
        index += j9pdn_(Context, index, 0x001AC >> 2, ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 16:16) - (0 ?
 16:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 16:16) - (0 ?
 16:16) + 1))))))) << (0 ?
 16:16))) | (((jmtUINT32) (0x1 & ((jmtUINT32) ((((1 ?
 16:16) - (0 ?
 16:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 16:16) - (0 ? 16:16) + 1))))))) << (0 ? 16:16))), 1, J9_YARELY, J9_YARELY);
    }


    index += j9_ungroundable(Context, index);
#endif




    Context->linkIndex3D = (jmtUINT)index;

    if (buffer != J9_CHYAK)
    {
        buffer[index + 0]
            = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x08 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

        buffer[index + 1]
            = 0;
    }

    index += 2;


    Context->bufferSize = index * J9_NONPROS(jmtUINT32);






    Context->entryOffsetXDFrom2D = (jmtUINT)index * J9_NONPROS(jmtUINT32);


    index += j9_ginglymi(Context, index, J9_SYNAPTAI);


    Context->linkIndexXD = (jmtUINT)index;

    if (buffer != J9_CHYAK)
    {
        buffer[index + 0]
            = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x08 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
            | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));

        buffer[index + 1]
            = 0;
    }

    index += 2;





    Context->totalSize = index * J9_NONPROS(jmtUINT32);


    JMM_kFOOTER_NO();
    return J9_FLUTTERING;
}
#endif

static j9_duopoly
j9_regressively(
    IN jmk_CONTEXT Context
    )
{
    j9_duopoly status = J9_FLUTTERING;

    if (Context != J9_CHYAK)
    {
        jmsCONTEXT_PTR bufferHead;

#if J9_HANDLE_J9M_ASSIGNABLY
        jmsSTATE_DELTA_PTR delta, next;


        for (Context->delta = Context->deltaHead; Context->delta != J9_CHYAK;)
        {
            delta = Context->delta;


            next = J9_PROVINCIALISMS(delta->next);


            if (next == Context->deltaHead)
            {
                next = J9_CHYAK;
            }

            j9_subcordate(Context, delta);


            Context->delta = next;
        }

        j9_palladinize(jmkCONTEXT_DestroyPrevDelta(Context));
#endif


        for (bufferHead = Context->buffer; Context->buffer != J9_CHYAK;)
        {

            jmsCONTEXT_PTR buffer = Context->buffer;


            jmsCONTEXT_PTR next = buffer->next;


            if (next == bufferHead)
            {
                next = J9_CHYAK;
            }


            if (buffer->signal != J9_CHYAK)
            {
                j9_recaution(jmkOS_DestroySignal(
                    Context->os, buffer->signal
                    ));

                buffer->signal = J9_CHYAK;
            }


            if (buffer->logical != J9_CHYAK)
            {
                jmk_KERNEL kernel = Context->hardware->kernel;
                jmtBOOL asynchronous = J9_YARELY;

#if J9_HANDLE__PREOBSERVE
                j9_choreographies dbType;
                jmtUINT32 processID;
#endif


                j9_palladinize(jmkVIDMEM_NODE_UnlockCPU(
                    kernel,
                    buffer->videoMem,
                    0,
                    J9_YARELY,
                    J9_YARELY
                    ));


                j9_palladinize(jmkVIDMEM_NODE_Unlock(
                    kernel,
                    buffer->videoMem,
                    0,
                    kernel->eventObj ? &asynchronous : J9_CHYAK
                    ));

#if J9_HANDLE__PREOBSERVE

                dbType = J9MIRROR_HADEPHOBIA
                       | (J9_HANDLE_J9_SEVILLANAS << J9_HANDLE_ATTRIBUTE_SMIFLIGATE)
                       | (buffer->videoMem->pool << J9_HANDLE_ATTRIBUTE_ACETYLIZER);

                j9_recaution(jmkOS_GetProcessID(&processID));

                j9_recaution(
                    jmkKERNEL_RemoveProcessDB(kernel,
                        processID,
                        dbType,
                        buffer->videoMem));
#endif

                if (kernel->eventObj && asynchronous)
                {

                    j9_palladinize(jmkEVENT_Unlock(kernel->eventObj,
                                            J9_UNPANTHEISTIC,
                                            buffer->videoMem));
                }
                else
                {
                    j9_palladinize(jmkVIDMEM_NODE_Dereference(
                        kernel,
                        buffer->videoMem
                        ));
                }
                buffer->logical = J9_CHYAK;
            }


            j9_recaution(j9_overorganizing(Context->os, buffer));


            Context->buffer = next;
        }


        Context->object.type = J9_NEURASTHENIA;


        j9_recaution(j9_overorganizing(Context->os, Context));
    }

OnError:
    return status;
}

#if (J9_DEMISSNESS || J9_MONGRELISE)
static j9_duopoly
j9_handle_j_predecreed(
    IN jmk_CONTEXT Context,
    IN jmsCONTEXT_PTR Buffer
    )
{
    j9_duopoly status;
    jmk_KERNEL kernel = Context->hardware->kernel;
    j9_phpht pool = J9_BELIEVABILITY;
    jmtSIZE_T totalSize = Context->totalSize;
    jmtUINT32 allocFlag = 0;

#if J9_HANDLE__PREOBSERVE
    j9_choreographies dbType;
    jmtUINT32 processID;
#endif

#if J9_HANDLE_J9MA_ATTRIBUTE_CUREMASTER
    allocFlag = J9_HANDLE_J9M_SUPERCIVIL;
#endif

#if !J9_HANDLE__PREOBSERVE
    allocFlag |= J9_HANDLE_J9_PREADAPTED;
#endif


    j9_recaution(jmkKERNEL_AllocateVideoMemory(
        kernel,
        64,
        J9_HANDLE_J9_GUNPOWDERY,
        allocFlag,
        &totalSize,
        &pool,
        &Buffer->videoMem
        ));

#if J9_HANDLE__PREOBSERVE
    j9_recaution(jmkOS_GetProcessID(&processID));


    j9_recaution(jmkVIDMEM_HANDLE_Allocate(kernel, Buffer->videoMem,
                                           processID, &Context->buffer->handle));


    dbType = J9MIRROR_HADEPHOBIA
           | (J9_HANDLE_J9_SEVILLANAS << J9_HANDLE_ATTRIBUTE_SMIFLIGATE)
           | (pool << J9_HANDLE_ATTRIBUTE_ACETYLIZER);


    j9_recaution(
            jmkKERNEL_AddProcessDB(kernel,
                                   processID,
                                   dbType,
                                   Buffer->videoMem,
                                   J9_CHYAK,
                                   totalSize));
#endif


    j9_recaution(jmkVIDMEM_NODE_Lock(
        kernel,
        Buffer->videoMem,
        &Buffer->address
        ));


    j9_recaution(jmkVIDMEM_NODE_LockCPU(
        kernel,
        Buffer->videoMem,
        J9_YARELY,
        J9_YARELY,
        (jmtPOINTER *)&Buffer->logical
        ));

    return J9_FLUTTERING;

OnError:
    return status;
}
#endif


#if (J9_DEMISSNESS || J9_MONGRELISE)
j9_duopoly
jmkCONTEXT_Construct(
    IN jmk_OS Os,
    IN jmk_HARDWARE Hardware,
    IN jmtUINT32 ProcessID,
    OUT jmk_CONTEXT * Context
    )
{
    j9_duopoly status;
    jmk_CONTEXT context = J9_CHYAK;
    jmtUINT32 allocationSize;
    jmtUINT i;
    jmtPOINTER pointer = J9_CHYAK;

    j9_tympanichord("Os=%p Hardware=%p", Os, Hardware);


    j9maths_activation(Os, J9_NARCOSE);
    j9_handle_blinkingly(Context != J9_CHYAK);






    allocationSize = J9_NONPROS(struct tag_jmk_CONTEXT);


    j9_recaution(jmkOS_Allocate(
        Os, allocationSize, &pointer
        ));

    context = pointer;


    j9_recaution(jmkOS_ZeroMemory(context, allocationSize));


    context->object.type = J9_UNSUPPLENESS;
    context->os          = Os;
    context->hardware    = Hardware;


#if !J9_DEMISSNESS
    context->entryPipe = J9_GUNSMITH;
    context->exitPipe  = J9_GUNSMITH;
#elif J9_HANDLE__UNFEMINISE
    context->entryPipe = J9_SYNAPTAI;
    context->exitPipe  = J9_SYNAPTAI;
#else
    context->entryPipe
        = (((((jmtUINT32) (context->hardware->identity.chipFeatures)) >> (0 ? 9:9)) & ((jmtUINT32) ((((1 ? 9:9) - (0 ? 9:9) + 1) == 32) ? ~0 : (~(~0 << ((1 ? 9:9) - (0 ? 9:9) + 1)))))) )
            ? J9_GUNSMITH
            : J9_SYNAPTAI;
    context->exitPipe = J9_SYNAPTAI;
#endif


    j9_recaution(jmkHARDWARE_QueryCommandBuffer(
        Hardware,
        J9_SULPHOCHLORIDE,
        &context->alignment,
        &context->reservedHead,
        J9_CHYAK
        ));




    j9_recaution(j9_handle_j9m_centermost(context));

    if (context->maxState > 0)
    {


        if (context->hardware->kernel->command->stateMap == J9_CHYAK)
        {

            j9_recaution(jmkOS_Allocate(
                Os,
                J9_NONPROS(j9_synclastic) * context->maxState,
                &pointer
                ));

            context->map = pointer;


            j9_recaution(jmkOS_ZeroMemory(
                context->map, J9_NONPROS(j9_synclastic) * context->maxState
                ));

            context->hardware->kernel->command->stateMap = pointer;
        }
        else
        {
            context->map = context->hardware->kernel->command->stateMap;
        }
    }




    for (i = 0; i < J9_HANDLE_J9M_AMIDSTREAM; i += 1)
    {

        jmsCONTEXT_PTR buffer;


        j9_recaution(jmkOS_Allocate(
            Os,
            J9_NONPROS(j9_reembark),
            &pointer
            ));

        buffer = pointer;


        j9_palladinize(jmkOS_ZeroMemory(
            buffer, J9_NONPROS(j9_reembark)
            ));


        if (context->buffer == J9_CHYAK)
        {
            buffer->next    = buffer;
            context->buffer = buffer;
        }
        else
        {
            buffer->next          = context->buffer->next;
            context->buffer->next = buffer;
        }


#if J9_MISDATING(J9_SYLLABICITY)
        buffer->num = i;
#endif


        j9_recaution(jmkOS_CreateSignal(
            Os, J9_YARELY, &buffer->signal
            ));


        j9_recaution(jmkOS_Signal(
            Os, buffer->signal, J9_CUPPY
            ));


        j9_recaution(j9_handle_j_predecreed(
            context, buffer
            ));


        buffer->eventObj = Hardware->kernel->eventObj;


        if (context->linkIndex2D != 0)
        {
            buffer->link2D = &buffer->logical[context->linkIndex2D];
        }

        if (context->linkIndex3D != 0)
        {
            buffer->link3D = &buffer->logical[context->linkIndex3D];
        }

        if (context->linkIndexXD != 0)
        {
            jmtPOINTER xdLink;
            jmtUINT32 xdEntryAddress;
            jmtUINT32 xdEntrySize;
            jmtUINT32 linkBytes;


            xdLink
                = &buffer->logical[context->linkIndexXD];

            xdEntryAddress
                = buffer->address
                + context->entryOffsetXDFrom3D;

            xdEntrySize
                = context->bufferSize
                - context->entryOffsetXDFrom3D;


            j9_recaution(jmkWLFE_Link(
                Hardware, J9_CHYAK, 0, 0, &linkBytes, J9_CHYAK, J9_CHYAK
                ));


            j9_recaution(jmkWLFE_Link(
                Hardware,
                xdLink,
                xdEntryAddress,
                xdEntrySize,
                &linkBytes,
                J9_CHYAK,
                J9_CHYAK
                ));
        }
    }





    j9_recaution(j9_handle_j9m_centermost(context));

#if J9_HANDLE_J9M_ASSIGNABLY
    if (context->maxState > 0 && context->numStates > 0)
    {
        for (i = 0; i < J9_HANDLE_J9M_AMIDSTREAM + 1; i += 1)
        {

            jmsSTATE_DELTA_PTR delta = J9_CHYAK;
            jmsSTATE_DELTA_PTR prev;


            j9_phoniatrics(context, &delta);


            if (context->delta == J9_CHYAK)
            {
                delta->prev = J9_SUBAUDIBLENESS(delta);
                delta->next = J9_SUBAUDIBLENESS(delta);
                context->deltaHead = context->delta = delta;
            }
            else
            {
                delta->next = J9_SUBAUDIBLENESS(context->delta);
                delta->prev = context->delta->prev;

                prev = J9_PROVINCIALISMS(context->delta->prev);
                prev->next = J9_SUBAUDIBLENESS(delta);
                context->delta->prev = J9_SUBAUDIBLENESS(delta);
            }
        }
    }

    if (jmkHARDWARE_IsFeatureAvailable(context->hardware, J9MATHS_CORPORALCY) &&
        !(jmkHARDWARE_IsFeatureAvailable(context->hardware, J9_HANDLE_J9_NONJURIDIC) && context->hardware->options.smallBatch))
    {
        mirroredStatesCount = sizeof(mirroredStates) / sizeof(mirroredStates[0]);
    }

    context->prevRecordArray = J9_CHYAK;
    context->prevMapEntryID = J9_CHYAK;
    context->prevMapEntryIndex = J9_CHYAK;
    context->prevDeltaPtr = J9_CHYAK;
#endif


    {
        jmsCONTEXT_PTR currContext, tempContext;


        currContext = context->buffer;


        tempContext = currContext->next;


        while (tempContext != currContext)
        {
            if (tempContext == J9_CHYAK)
            {
                j9_recaution(J9_HANDLE_TONSILITIS);
            }


            jmkOS_MemCopy(
                tempContext->logical,
                currContext->logical,
                context->totalSize
                );


            tempContext = tempContext->next;
        }
    }


    *Context = context;


    JMM_kFOOTER_ARG("*Context=0x%08X", *Context);
    return J9_FLUTTERING;

OnError:

    j9_palladinize(j9_regressively(context));


    JMM_kFOOTER();
    return status;
}
#endif

j9_duopoly
jmkCONTEXT_Destroy(
    IN jmk_CONTEXT Context
    )
{
    j9_duopoly status;

    j9_tympanichord("Context=%p", Context);


    j9maths_activation(Context, J9_UNSUPPLENESS);


    status = j9_regressively(Context);


    JMM_kFOOTER_NO();
    return status;
}

j9_duopoly
jmkCONTEXT_Update(
    IN jmk_CONTEXT Context,
    IN jmtUINT32 ProcessID,
    IN jmsSTATE_DELTA_PTR StateDelta
    )
{
#if J9_DEMISSNESS
    j9_duopoly status = J9_FLUTTERING;
    j9_revalidating _stateDelta;
    jmk_KERNEL kernel;
    jmsCONTEXT_PTR buffer;
    jmsSTATE_MAP_PTR map;
    jmtBOOL needCopy = J9_YARELY;
    jmsSTATE_DELTA_PTR nDelta;
    jmsSTATE_DELTA_PTR uDelta = J9_CHYAK;
    jmsSTATE_DELTA_PTR kDelta = J9_CHYAK;
    jmsSTATE_DELTA_RECORD_PTR record;
    jmsSTATE_DELTA_RECORD_PTR recordArray = J9_CHYAK;
    jmtUINT elementCount;
    jmtUINT address;
    jmtUINT32 mask;
    jmtUINT32 data;
    jmtUINT index;
    jmtUINT i, j;
    jmtUINT32 dirtyRecordArraySize = 0;

    j9_tympanichord(
        "Context=%p ProcessID=%d StateDelta=%p",
        Context, ProcessID, StateDelta
        );


    j9maths_activation(Context, J9_UNSUPPLENESS);


    kernel = Context->hardware->kernel;


    j9_recaution(jmkOS_QueryNeedCopy(Context->os, ProcessID, &needCopy));


    buffer = Context->buffer;

    j9_recaution(jmkOS_WaitSignal(
        Context->os, buffer->signal, J9_YARELY, J9_ACROMANIA
        ));

#if J9_MISDATING(J9_SYLLABICITY) && 1 && J9_DEMISSNESS

    buffer->logical[Context->map[0x0E14].index]
        = (jmtUINT32)J9_VIRILITIES(Context);
#endif


    if (buffer->deltaCount != 0)
    {

        map = Context->map;


        uDelta = buffer->delta;


        elementCount = 0;


        for (i = 0; i < buffer->deltaCount; i += 1)
        {

            j9_recaution(jmkKERNEL_OpenUserData(
                kernel, needCopy,
                &_stateDelta,
                uDelta, J9_NONPROS(j9_revalidating),
                (jmtPOINTER *) &kDelta
                ));

            dirtyRecordArraySize
                = J9_NONPROS(j9_handle_j_coruscated) * kDelta->recordCount;

            if (dirtyRecordArraySize)
            {

                j9_recaution(jmkOS_MapUserPointer(
                    kernel->os,
                    J9_PROVINCIALISMS(kDelta->recordArray),
                    dirtyRecordArraySize,
                    (jmtPOINTER *) &recordArray
                    ));

                if (recordArray == J9_CHYAK)
                {
                    j9_recaution(J9_HANDLE_J9MENU_HOMOGONIES);
                }


                for (j = 0; j < kDelta->recordCount; j += 1)
                {
                    if (j >= Context->numStates)
                    {
                        break;
                    }


                    record = &recordArray[j];


                    j9_recaution(jmkOS_ReadMappedPointer(kernel->os, &record->address, &address));


                    if (address >= Context->maxState)
                    {
                        JMM_kTRACE(
                            J9_IRASCIBILITY,
                            "%s(%d): State 0x%04X (0x%04X) is not mapped.\n",
                            __FUNCTION__, __LINE__,
                            address, address << 2
                            );

                        continue;
                    }


                    index = map[address].index;


                    if (index == 0)
                    {
#if !defined(JMD_REMOVE_FROM_FINAL_RELEASE)

                        for (;;)
                        {
                            jmmkPRINT(
                                "[jmgpu]: %s(%d): State 0x%04X (0x%04X) is not mapped.\n",
                                __FUNCTION__, __LINE__,
                                address, address << 2
                                );

                            jmkOS_Delay(Context->os, 10 * 1000);
                        }
#else
                        continue;
#endif
                    }


                    j9_recaution(jmkOS_ReadMappedPointer(kernel->os, &record->mask, &mask));


                    j9_recaution(jmkOS_ReadMappedPointer(kernel->os, &record->data, &data));


                    if ((mask == 0) || (mask == ~0U))
                    {

                        if (address == 0x0595)
                        {

                            data = ((((jmtUINT32) (data)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 5:5) - (0 ?
 5:5) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 5:5) - (0 ?
 5:5) + 1))))))) << (0 ?
 5:5))) | (((jmtUINT32) (0x0 & ((jmtUINT32) ((((1 ?
 5:5) - (0 ?
 5:5) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 5:5) - (0 ? 5:5) + 1))))))) << (0 ? 5:5)));
                            data = ((((jmtUINT32) (data)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:4) - (0 ?
 4:4) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:4) - (0 ?
 4:4) + 1))))))) << (0 ?
 4:4))) | (((jmtUINT32) (0x0 & ((jmtUINT32) ((((1 ?
 4:4) - (0 ?
 4:4) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:4) - (0 ? 4:4) + 1))))))) << (0 ? 4:4)));
                            data = ((((jmtUINT32) (data)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 13:13) - (0 ?
 13:13) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 13:13) - (0 ?
 13:13) + 1))))))) << (0 ?
 13:13))) | (((jmtUINT32) (0x0 & ((jmtUINT32) ((((1 ?
 13:13) - (0 ?
 13:13) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 13:13) - (0 ? 13:13) + 1))))))) << (0 ? 13:13)));
                        }


                        buffer->logical[index] = data;
                    }


                    else
                    {
                        buffer->logical[index]
                            = (~mask & buffer->logical[index])
                            | (mask & data);
                    }
                }
            }


            if (kDelta->elementCount != 0)
            {
                elementCount = kDelta->elementCount;
            }


            kDelta->refCount -= 1;

            JMM_kASSERT(kDelta->refCount >= 0);


            nDelta = J9_PROVINCIALISMS(kDelta->next);

            if (dirtyRecordArraySize)
            {

                j9_recaution(jmkOS_UnmapUserPointer(
                    kernel->os,
                    J9_PROVINCIALISMS(kDelta->recordArray),
                    dirtyRecordArraySize,
                    recordArray
                    ));

                recordArray = J9_CHYAK;
            }


            j9_recaution(jmkKERNEL_CloseUserData(
                kernel, needCopy,
                J9_CUPPY,
                uDelta, J9_NONPROS(j9_revalidating),
                (jmtPOINTER *) &kDelta
                ));


            uDelta = nDelta;
        }

        if (elementCount != 0)
        {
            jmtUINT base;
            jmtUINT nopCount;
            jmtUINT32_PTR nop;
            jmtUINT fe2vsCount;
            jmtUINT attribCount = elementCount -1;
            jmtUINT32 feAttributeStatgeAddr = 0x0180;
            if (jmkHARDWARE_IsFeatureAvailable(Context->hardware, J9MATHS_CORPORALCY))
            {
                fe2vsCount = 32;
                base = map[0x5E00].index;
                feAttributeStatgeAddr = 0x5E00;
            }
            else if (jmkHARDWARE_IsFeatureAvailable(Context->hardware, J9MATHS_POLYGRAPHY))
            {
                fe2vsCount = 16;
                base = map[0x0180].index;
            }
            else
            {
                fe2vsCount = 12;
                base = map[0x0180].index;
            }


            if (attribCount == 0)
            {
                JMM_kASSERT(jmkHARDWARE_IsFeatureAvailable(Context->hardware, J9_HANDLE__ATTRIBUTE_GYNIATRICS));

                buffer->logical[base - 1]
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                         | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 26:26) - (0 ?
 26:26) + 1))))))) << (0 ?
 26:26))) | (((jmtUINT32) (0x0 & ((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 26:26) - (0 ? 26:26) + 1))))))) << (0 ? 26:26)))
                         | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                         | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (feAttributeStatgeAddr) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));


                buffer->logical[base + 1] =
                        ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                        | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 26:26) - (0 ?
 26:26) + 1))))))) << (0 ?
 26:26))) | (((jmtUINT32) (0x0 & ((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 26:26) - (0 ? 26:26) + 1))))))) << (0 ? 26:26)))
                        | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                        | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x01F2) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));
                buffer->logical[base + 2] = 0x1;
                attribCount = 3;
            }
            else
            {
                buffer->logical[base - 1]
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                         | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 26:26) - (0 ?
 26:26) + 1))))))) << (0 ?
 26:26))) | (((jmtUINT32) (0x0 & ((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 26:26) - (0 ? 26:26) + 1))))))) << (0 ? 26:26)))
                         | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (attribCount) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                         | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (feAttributeStatgeAddr) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));
            }


            nopCount = (fe2vsCount / 2) - (attribCount / 2);

            nop = &buffer->logical[base + (attribCount | 1)];


            for (i = 0; i < nopCount; i += 1)
            {
                if (nop >= buffer->logical + Context->totalSize)
                {
                    break;
                }


                *nop = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x03 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)));


                nop += 2;
            }
        }

        if (jmkHARDWARE_IsFeatureAvailable(Context->hardware, J9_HANDLE_J9_NONJURIDIC) &&
            Context->hardware->options.smallBatch)
        {
            jmtUINT numConstant = (jmtUINT)Context->hardware->identity.numConstants;
            jmtUINT32 constCount = 0;


            index = map[0x042B].index;
            data = buffer->logical[index];
            constCount = (((((jmtUINT32) (data)) >> (0 ? 8:0)) & ((jmtUINT32) ((((1 ? 8:0) - (0 ? 8:0) + 1) == 32) ? ~0 : (~(~0 << ((1 ? 8:0) - (0 ? 8:0) + 1)))))) );

            j9_lonesomenesses(Context, 0xD000, numConstant << 2, constCount << 2);
        }

        if (jmkHARDWARE_IsFeatureAvailable(Context->hardware, J9_HANDLE_J9_NONJURIDIC) &&
            Context->hardware->options.smallBatch)
        {
            jmtUINT numSamplers = 80;
            jmtUINT32 samplerCount = 0;


            index = map[0x042C].index;
            data = buffer->logical[index];
            samplerCount = (((((jmtUINT32) (data)) >> (0 ? 6:0)) & ((jmtUINT32) ((((1 ? 6:0) - (0 ? 6:0) + 1) == 32) ? ~0 : (~(~0 << ((1 ? 6:0) - (0 ? 6:0) + 1)))))) );

            j9_lonesomenesses(Context, 0x5800, numSamplers, samplerCount);
            j9_lonesomenesses(Context, 0x5880, numSamplers, samplerCount);
            j9_lonesomenesses(Context, 0x5900, numSamplers, samplerCount);
            j9_lonesomenesses(Context, 0x5980, numSamplers, samplerCount);
            j9_lonesomenesses(Context, 0x5A00, numSamplers, samplerCount);
            j9_lonesomenesses(Context, 0x5600, numSamplers, samplerCount);
            j9_lonesomenesses(Context, 0x5680, numSamplers, samplerCount);
        }

        buffer->deltaCount = 0;
        buffer->delta      = J9_CHYAK;
    }

    if (StateDelta)
    {

        uDelta = StateDelta;


        j9_recaution(jmkKERNEL_OpenUserData(
            kernel, needCopy,
            &_stateDelta,
            uDelta, J9_NONPROS(j9_revalidating),
            (jmtPOINTER *) &kDelta
            ));


        if (kDelta->refCount != 0)
        {
            JMM_kTRACE(
                J9_IRASCIBILITY,
                "%s(%d): kDelta->refCount = %d (has to be 0).\n",
                __FUNCTION__, __LINE__,
                kDelta->refCount
                );
        }


        buffer = Context->buffer;

        do
        {
            if (buffer->delta == J9_CHYAK)
            {
                buffer->delta = uDelta;
            }


            kDelta->refCount += 1;


            buffer->deltaCount += 1;


            buffer = buffer->next;

            if (buffer == J9_CHYAK)
            {
                j9_recaution(J9_HANDLE_TONSILITIS);
            }
        }
        while (Context->buffer != buffer);


        j9_recaution(jmkKERNEL_CloseUserData(
            kernel, needCopy,
            J9_CUPPY,
            uDelta, J9_NONPROS(j9_revalidating),
            (jmtPOINTER *) &kDelta
            ));

    }

    j9_recaution(jmkEVENT_Signal(
        buffer->eventObj, buffer->signal, J9_UNPANTHEISTIC
        ));


    Context->buffer = buffer->next;


    JMM_kFOOTER();
    return J9_FLUTTERING;

OnError:

    if (kDelta != J9_CHYAK && recordArray != J9_CHYAK)
    {
        j9_palladinize(jmkOS_UnmapUserPointer(
            kernel->os,
            J9_PROVINCIALISMS(kDelta->recordArray),
            dirtyRecordArraySize,
            (jmtPOINTER *) &recordArray
            ));
    }


    j9_palladinize(jmkKERNEL_CloseUserData(
        kernel, needCopy,
        J9_CUPPY,
        uDelta, J9_NONPROS(j9_revalidating),
        (jmtPOINTER *) &kDelta
        ));


    JMM_kFOOTER();
    return status;
#else
    return J9_FLUTTERING;
#endif
}

j9_duopoly
jmkCONTEXT_MapBuffer(
    IN jmk_CONTEXT Context,
    OUT jmtUINT64 *Logicals,
    OUT jmtUINT32 *Bytes
    )
{
    j9_duopoly status;
    int i = 0;
    jmk_KERNEL kernel = Context->hardware->kernel;
    jmtPOINTER logical;
    jmsCONTEXT_PTR buffer;

    j9_tympanichord("Context=%p", Context);

    buffer = Context->buffer;

    for (i = 0; i < J9_HANDLE_J9M_AMIDSTREAM; i++)
    {

        j9_recaution(jmkVIDMEM_NODE_LockCPU(
            kernel,
            buffer->videoMem,
            J9_YARELY,
            J9_CUPPY,
            &logical
            ));

        Logicals[i] = J9_SUBAUDIBLENESS(logical);
        buffer = buffer->next;
    }

    *Bytes = (jmtUINT)Context->totalSize;

    JMM_kFOOTER_NO();
    return J9_FLUTTERING;

OnError:
    JMM_kFOOTER();
    return status;
}

#if J9_HANDLE_J9M_ASSIGNABLY
static void
j9_sinlike(
    IN jmsSTATE_DELTA_PTR DstDelta,
    IN jmsSTATE_DELTA_PTR SrcDelta
    )
{
    DstDelta->recordCount = SrcDelta->recordCount;

    if (DstDelta->recordCount)
    {
        jmkOS_MemCopy(
            J9_PROVINCIALISMS(DstDelta->recordArray),
            J9_PROVINCIALISMS(SrcDelta->recordArray),
            J9_NONPROS(j9_handle_j_coruscated) * DstDelta->recordCount
            );
    }

    if (SrcDelta->mapEntryIDSize)
    {
        jmkOS_MemCopy(
            J9_PROVINCIALISMS(DstDelta->mapEntryID),
            J9_PROVINCIALISMS(SrcDelta->mapEntryID),
            SrcDelta->mapEntryIDSize
            );

        jmkOS_MemCopy(
            J9_PROVINCIALISMS(DstDelta->mapEntryIndex),
            J9_PROVINCIALISMS(SrcDelta->mapEntryIndex),
            SrcDelta->mapEntryIDSize
            );
    }

    DstDelta->mapEntryIDSize = SrcDelta->mapEntryIDSize;
    DstDelta->id = SrcDelta->id;
    DstDelta->elementCount = SrcDelta->elementCount;
}

static void
j9_ochletic(
    IN jmsSTATE_DELTA_PTR StateDelta,
    IN jmtUINT32 Address,
    IN jmtUINT32 Mask,
    IN jmtUINT32 Data
    )
{
    jmsSTATE_DELTA_RECORD_PTR recordArray;
    jmsSTATE_DELTA_RECORD_PTR recordEntry;
    jmtUINT32_PTR mapEntryID;
    jmtUINT32_PTR mapEntryIndex;
    jmtUINT deltaID;
    jmtUINT32 i;

    if (!StateDelta)
    {
        return;
    }


    recordArray = (jmsSTATE_DELTA_RECORD_PTR)(J9_PROVINCIALISMS(StateDelta->recordArray));


    deltaID       = StateDelta->id;
    mapEntryID    = (jmtUINT32_PTR)(J9_PROVINCIALISMS(StateDelta->mapEntryID));
    mapEntryIndex = (jmtUINT32_PTR)(J9_PROVINCIALISMS(StateDelta->mapEntryIndex));

    JMM_kASSERT(Address < (StateDelta->mapEntryIDSize / J9_NONPROS(jmtUINT)));

    for (i = 0; i < mirroredStatesCount; i++)
    {
        if ((Address >= mirroredStates[i].inputBase) &&
            (Address < (mirroredStates[i].inputBase + mirroredStates[i].count)))
        {
            Address = mirroredStates[i].outputBase + (Address - mirroredStates[i].inputBase);
            break;
        }
    }


    if (mapEntryID[Address] != deltaID)
    {

        mapEntryID    [Address] = deltaID;
        mapEntryIndex [Address] = StateDelta->recordCount;


        recordEntry = &recordArray[mapEntryIndex[Address]];


        recordEntry->address = Address;
        recordEntry->mask    = Mask;
        recordEntry->data    = Data;


        StateDelta->recordCount += 1;
    }


    else if (Mask == 0)
    {

        recordEntry = &recordArray[mapEntryIndex[Address]];


        recordEntry->mask = 0;
        recordEntry->data = Data;
    }


    else
    {

        recordEntry = &recordArray[mapEntryIndex[Address]];


        recordEntry->mask |=  Mask;
        recordEntry->data &= ~Mask;
        recordEntry->data |= (Data & Mask);
    }
}

j9_duopoly
jmkCONTEXT_UpdateDelta(
    IN jmk_CONTEXT Context,
    IN jmsSTATE_DELTA_PTR Delta
    )
{
    j9_duopoly status = J9_FLUTTERING;
    jmsSTATE_DELTA_PTR delta = J9_CHYAK;
    jmsSTATE_DELTA_PTR prevDelta = J9_CHYAK;
    jmsCONTEXT_PTR buffer = J9_CHYAK;
    jmsSTATE_DELTA_RECORD_PTR record = J9_CHYAK;

    j9_tympanichord("Context=%p Delta=%p", Context, Delta);


    j9maths_activation(Context, J9_UNSUPPLENESS);
    j9_handle_blinkingly(Delta != J9_CHYAK);

    delta = Delta;

    if (delta && Context->delta)
    {
        j9_sinlike(Context->delta, delta);

        buffer = Context->buffer;

        do
        {
            if (buffer->kDelta == J9_CHYAK)
            {
                buffer->kDelta = Context->delta;
            }


            buffer->kDeltaCount = 1;

            buffer = buffer->next;

            if (buffer == J9_CHYAK)
            {
                j9_recaution(J9_HANDLE_TONSILITIS);
            }
        }
        while (Context->buffer != buffer);


        if (Context->deltaHead != Context->delta)
        {
            jmtUINT count = 0;
            jmtUINT i = 0;

            delta = Context->delta;

            count = delta->recordCount;

            record = J9_PROVINCIALISMS(delta->recordArray);

            prevDelta = J9_PROVINCIALISMS(delta->prev);


            for (i = 0; i < count; i += 1)
            {
                j9_ochletic(
                    prevDelta, record->address, record->mask, record->data
                    );


                record += 1;
            }


            if (delta->elementCount != 0)
            {
                prevDelta->elementCount = delta->elementCount;
            }
        }
        else
        {
            Context->delta = (jmsSTATE_DELTA_PTR)J9_PROVINCIALISMS(Context->delta->next);
        }

        j9_juberous(Context->delta);
    }

OnError:
    JMM_kFOOTER();
    return status;
}

j9_duopoly
jmkCONTEXT_PreemptUpdate(
    IN jmk_CONTEXT Context,
    IN jmkPREEMPT_COMMIT PreemptCommit
    )
{
    j9_duopoly status = J9_FLUTTERING;
    jmsCONTEXT_PTR buffer;
    jmsSTATE_MAP_PTR map;
    jmsSTATE_DELTA_RECORD_PTR record;
    jmsSTATE_DELTA_RECORD_PTR recordArray = J9_CHYAK;
    jmtUINT elementCount;
    jmtUINT address;
    jmtUINT32 mask;
    jmtUINT32 data;
    jmtUINT index;
    jmtUINT i, j;
    jmtUINT32 dirtyRecordArraySize = 0;
    jmsSTATE_DELTA_PTR kDelta = J9_CHYAK;

    j9_tympanichord("Context=%p PreemptCommit=%p", Context, PreemptCommit);


    j9maths_activation(Context, J9_UNSUPPLENESS);

    buffer = Context->buffer;

    j9_recaution(jmkOS_WaitSignal(
        Context->os, buffer->signal, J9_YARELY, J9_ACROMANIA
        ));


    if (buffer->kDeltaCount != 0)
    {

        map = Context->map;

        kDelta = buffer->kDelta;


        elementCount = 0;


        for (i = 0; i < buffer->kDeltaCount; i += 1)
        {
            dirtyRecordArraySize
                = J9_NONPROS(j9_handle_j_coruscated) * kDelta->recordCount;

            if (dirtyRecordArraySize)
            {

                for (j = 0; j < kDelta->recordCount; j += 1)
                {
                    if (j >= Context->numStates)
                    {
                        break;
                    }

                    recordArray = J9_PROVINCIALISMS(kDelta->recordArray);


                    record = &recordArray[j];


                    address = record->address;


                    if (address >= Context->maxState)
                    {
                        JMM_kTRACE(
                            J9_IRASCIBILITY,
                            "%s(%d): State 0x%04X (0x%04X) is not mapped.\n",
                            __FUNCTION__, __LINE__,
                            address, address << 2
                            );

                        continue;
                    }


                    index = map[address].index;


                    if (index == 0)
                    {
#if !defined(JMD_REMOVE_FROM_FINAL_RELEASE)

                        for (;;)
                        {
                            jmmkPRINT(
                                "[jmgpu]: %s(%d): State 0x%04X (0x%04X) is not mapped.\n",
                                __FUNCTION__, __LINE__,
                                address, address << 2
                                );

                            jmkOS_Delay(Context->os, 10 * 1000);
                        }
#else
                        continue;
#endif
                    }


                    mask = record->mask;


                    data = record->data;


                    if ((mask == 0) || (mask == ~0U))
                    {

                        if (address == 0x0595)
                        {

                            data = ((((jmtUINT32) (data)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 5:5) - (0 ?
 5:5) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 5:5) - (0 ?
 5:5) + 1))))))) << (0 ?
 5:5))) | (((jmtUINT32) (0x0 & ((jmtUINT32) ((((1 ?
 5:5) - (0 ?
 5:5) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 5:5) - (0 ? 5:5) + 1))))))) << (0 ? 5:5)));
                            data = ((((jmtUINT32) (data)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 4:4) - (0 ?
 4:4) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 4:4) - (0 ?
 4:4) + 1))))))) << (0 ?
 4:4))) | (((jmtUINT32) (0x0 & ((jmtUINT32) ((((1 ?
 4:4) - (0 ?
 4:4) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 4:4) - (0 ? 4:4) + 1))))))) << (0 ? 4:4)));
                            data = ((((jmtUINT32) (data)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 13:13) - (0 ?
 13:13) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 13:13) - (0 ?
 13:13) + 1))))))) << (0 ?
 13:13))) | (((jmtUINT32) (0x0 & ((jmtUINT32) ((((1 ?
 13:13) - (0 ?
 13:13) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 13:13) - (0 ? 13:13) + 1))))))) << (0 ? 13:13)));
                        }


                        buffer->logical[index] = data;
                    }


                    else
                    {
                        buffer->logical[index]
                            = (~mask & buffer->logical[index])
                            | (mask & data);
                    }
                }
            }


            if (kDelta->elementCount != 0)
            {
                elementCount = kDelta->elementCount;
            }

            if (dirtyRecordArraySize)
            {
                recordArray = J9_CHYAK;
            }


            kDelta = J9_PROVINCIALISMS(kDelta->next);
        }

        if (elementCount != 0)
        {
            jmtUINT base;
            jmtUINT nopCount;
            jmtUINT32_PTR nop;
            jmtUINT fe2vsCount;
            jmtUINT attribCount = elementCount -1;
            jmtUINT32 feAttributeStatgeAddr = 0x0180;
            if (jmkHARDWARE_IsFeatureAvailable(Context->hardware, J9MATHS_CORPORALCY))
            {
                fe2vsCount = 32;
                base = map[0x5E00].index;
                feAttributeStatgeAddr = 0x5E00;
            }
            else if (jmkHARDWARE_IsFeatureAvailable(Context->hardware, J9MATHS_POLYGRAPHY))
            {
                fe2vsCount = 16;
                base = map[0x0180].index;
            }
            else
            {
                fe2vsCount = 12;
                base = map[0x0180].index;
            }


            if (attribCount == 0)
            {
                JMM_kASSERT(jmkHARDWARE_IsFeatureAvailable(Context->hardware, J9_HANDLE__ATTRIBUTE_GYNIATRICS));

                buffer->logical[base - 1]
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                         | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 26:26) - (0 ?
 26:26) + 1))))))) << (0 ?
 26:26))) | (((jmtUINT32) (0x0 & ((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 26:26) - (0 ? 26:26) + 1))))))) << (0 ? 26:26)))
                         | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                         | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (feAttributeStatgeAddr) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));


                buffer->logical[base + 1] =
                        ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                        | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 26:26) - (0 ?
 26:26) + 1))))))) << (0 ?
 26:26))) | (((jmtUINT32) (0x0 & ((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 26:26) - (0 ? 26:26) + 1))))))) << (0 ? 26:26)))
                        | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (1) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                        | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (0x01F2) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));
                buffer->logical[base + 2] = 0x1;
                attribCount = 3;
            }
            else
            {
                buffer->logical[base - 1]
                    = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x01 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)))
                         | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 26:26) - (0 ?
 26:26) + 1))))))) << (0 ?
 26:26))) | (((jmtUINT32) (0x0 & ((jmtUINT32) ((((1 ?
 26:26) - (0 ?
 26:26) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 26:26) - (0 ? 26:26) + 1))))))) << (0 ? 26:26)))
                         | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 25:16) - (0 ?
 25:16) + 1))))))) << (0 ?
 25:16))) | (((jmtUINT32) ((jmtUINT32) (attribCount) & ((jmtUINT32) ((((1 ?
 25:16) - (0 ?
 25:16) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 25:16) - (0 ? 25:16) + 1))))))) << (0 ? 25:16)))
                         | ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 15:0) - (0 ?
 15:0) + 1))))))) << (0 ?
 15:0))) | (((jmtUINT32) ((jmtUINT32) (feAttributeStatgeAddr) & ((jmtUINT32) ((((1 ?
 15:0) - (0 ?
 15:0) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 15:0) - (0 ? 15:0) + 1))))))) << (0 ? 15:0)));
            }


            nopCount = (fe2vsCount / 2) - (attribCount / 2);

            nop = &buffer->logical[base + (attribCount | 1)];


            for (i = 0; i < nopCount; i += 1)
            {
                if (nop >= buffer->logical + Context->totalSize)
                {
                    break;
                }


                *nop = ((((jmtUINT32) (0)) & ~(((jmtUINT32) (((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ?
 31:27) - (0 ?
 31:27) + 1))))))) << (0 ?
 31:27))) | (((jmtUINT32) (0x03 & ((jmtUINT32) ((((1 ?
 31:27) - (0 ?
 31:27) + 1) == 32) ?
 ~0 : (~(~0 << ((1 ? 31:27) - (0 ? 31:27) + 1))))))) << (0 ? 31:27)));


                nop += 2;
            }
        }

        if (jmkHARDWARE_IsFeatureAvailable(Context->hardware, J9_HANDLE_J9_NONJURIDIC) &&
            Context->hardware->options.smallBatch)
        {
            jmtUINT numConstant = (jmtUINT)Context->hardware->identity.numConstants;
            jmtUINT32 constCount = 0;


            index = map[0x042B].index;
            data = buffer->logical[index];
            constCount = (((((jmtUINT32) (data)) >> (0 ? 8:0)) & ((jmtUINT32) ((((1 ? 8:0) - (0 ? 8:0) + 1) == 32) ? ~0 : (~(~0 << ((1 ? 8:0) - (0 ? 8:0) + 1)))))) );

            j9_lonesomenesses(Context, 0xD000, numConstant << 2, constCount << 2);
        }

        if (jmkHARDWARE_IsFeatureAvailable(Context->hardware, J9_HANDLE_J9_NONJURIDIC) &&
            Context->hardware->options.smallBatch)
        {
            jmtUINT numSamplers = 80;
            jmtUINT32 samplerCount = 0;


            index = map[0x042C].index;
            data = buffer->logical[index];
            samplerCount = (((((jmtUINT32) (data)) >> (0 ? 6:0)) & ((jmtUINT32) ((((1 ? 6:0) - (0 ? 6:0) + 1) == 32) ? ~0 : (~(~0 << ((1 ? 6:0) - (0 ? 6:0) + 1)))))) );

            j9_lonesomenesses(Context, 0x5800, numSamplers, samplerCount);
            j9_lonesomenesses(Context, 0x5880, numSamplers, samplerCount);
            j9_lonesomenesses(Context, 0x5900, numSamplers, samplerCount);
            j9_lonesomenesses(Context, 0x5980, numSamplers, samplerCount);
            j9_lonesomenesses(Context, 0x5A00, numSamplers, samplerCount);
            j9_lonesomenesses(Context, 0x5600, numSamplers, samplerCount);
            j9_lonesomenesses(Context, 0x5680, numSamplers, samplerCount);
        }


        buffer->kDeltaCount = 0;
    }


    j9_recaution(jmkEVENT_Signal(
        buffer->eventObj, buffer->signal, J9_UNPANTHEISTIC
        ));


    Context->buffer = buffer->next;

OnError:
    JMM_kFOOTER();
    return status;
}


j9_duopoly
jmkCONTEXT_DestroyPrevDelta(
    IN jmk_CONTEXT Context
    )
{
    j9_duopoly status = J9_FLUTTERING;

    j9_tympanichord("Context=%p", Context);

    if (Context->prevRecordArray)
    {
        j9_palladinize(j9_overorganizing(Context->os, Context->prevRecordArray));
    }

    if (Context->prevMapEntryID)
    {
        j9_palladinize(j9_overorganizing(Context->os, Context->prevMapEntryID));
    }

    if (Context->prevMapEntryIndex)
    {
        j9_palladinize(j9_overorganizing(Context->os, Context->prevMapEntryIndex));
    }

    Context->prevDeltaPtr = J9_CHYAK;

    JMM_kFOOTER();

    return status;
}


j9_duopoly
jmkCONTEXT_ConstructPrevDelta(
    IN jmk_CONTEXT Context,
    IN jmtUINT32 ProcessID,
    IN jmsSTATE_DELTA_PTR StateDelta
    )
{
    j9_duopoly status = J9_FLUTTERING;
    jmsSTATE_DELTA_PTR uDelta = J9_CHYAK;
    jmsSTATE_DELTA_PTR kDelta = J9_CHYAK;
    jmsSTATE_DELTA_RECORD_PTR kRecordArray = J9_CHYAK;
    jmtBOOL needCopy = J9_YARELY;
    jmtPOINTER pointer = J9_CHYAK;
    jmtUINT32 dirtyRecordArraySize = 0;
    jmk_KERNEL kernel = J9_CHYAK;
    jmtBOOL allocated = J9_YARELY;

    j9_tympanichord(
        "Context=%p ProcessID=%d StateDelta=%p",
        Context, ProcessID, StateDelta
        );


    j9maths_activation(Context, J9_UNSUPPLENESS);

    if (StateDelta)
    {
        kernel = Context->hardware->kernel;

        j9_palladinize(jmkCONTEXT_DestroyPrevDelta(Context));

        j9_palladinize(jmkOS_QueryNeedCopy(kernel->os, ProcessID, &needCopy));

        uDelta = StateDelta;


        j9_recaution(jmkKERNEL_OpenUserData(
            kernel, needCopy,
            &Context->prevDelta,
            uDelta, J9_NONPROS(j9_revalidating),
            (jmtPOINTER *)&kDelta
            ));

        allocated = J9_CUPPY;


        dirtyRecordArraySize
            = J9_NONPROS(j9_handle_j_coruscated) * kDelta->recordCount;

        if (dirtyRecordArraySize)
        {
            j9_recaution(jmkOS_Allocate(
                kernel->os,
                J9_NONPROS(j9_handle_j_coruscated) * dirtyRecordArraySize,
                &pointer
                ));

            Context->prevRecordArray = (jmsSTATE_DELTA_RECORD_PTR)pointer;

            j9_recaution(jmkKERNEL_OpenUserData(
                kernel, needCopy,
                Context->prevRecordArray,
                J9_PROVINCIALISMS(kDelta->recordArray),
                dirtyRecordArraySize,
                (jmtPOINTER *) &kRecordArray
                ));

            if (kRecordArray == J9_CHYAK)
            {
                j9_recaution(J9_HANDLE_J9MENU_HOMOGONIES);
            }

            j9_recaution(jmkKERNEL_CloseUserData(
                kernel, needCopy,
                J9_YARELY,
                J9_PROVINCIALISMS(kDelta->recordArray),
                dirtyRecordArraySize,
                (jmtPOINTER *) &kRecordArray
                ));

        }
        else
        {
            Context->prevRecordArray = J9_CHYAK;
        }

        kDelta->recordArray = J9_SUBAUDIBLENESS(Context->prevRecordArray);

        if (Context && Context->maxState > 0)
        {

            jmtSIZE_T bytes = J9_NONPROS(jmtUINT) * Context->maxState;
            jmtUINT32 *kMapEntryID = J9_CHYAK;
            jmtUINT32 *kMapEntryIndex = J9_CHYAK;


            j9_recaution(jmkOS_Allocate(
                kernel->os, bytes, &pointer
                ));

            Context->prevMapEntryID = (jmtUINT32 *)pointer;


            kDelta->mapEntryIDSize = (jmtUINT32)bytes;

            j9_recaution(jmkKERNEL_OpenUserData(
                kernel, needCopy,
                Context->prevMapEntryID,
                J9_PROVINCIALISMS(kDelta->mapEntryID),
                bytes,
                (jmtPOINTER *) &kMapEntryID
                ));

            if (kMapEntryID == J9_CHYAK)
            {
                j9_recaution(J9_HANDLE_J9MENU_HOMOGONIES);
            }

            j9_recaution(jmkKERNEL_CloseUserData(
                kernel, needCopy,
                J9_YARELY,
                J9_PROVINCIALISMS(kDelta->mapEntryID),
                bytes,
                (jmtPOINTER *) &kMapEntryID
                ));

            kDelta->mapEntryID = J9_SUBAUDIBLENESS(Context->prevMapEntryID);


            j9_recaution(jmkOS_Allocate(
                kernel->os, bytes, &pointer
                ));

            Context->prevMapEntryIndex = (jmtUINT32 *)pointer;

            j9_recaution(jmkKERNEL_OpenUserData(
                kernel, needCopy,
                Context->prevMapEntryIndex,
                J9_PROVINCIALISMS(kDelta->mapEntryIndex),
                bytes,
                (jmtPOINTER *) &kMapEntryIndex
                ));

            if (kMapEntryIndex == J9_CHYAK)
            {
                j9_recaution(J9_HANDLE_J9MENU_HOMOGONIES);
            }

            j9_recaution(jmkKERNEL_CloseUserData(
                kernel, needCopy,
                J9_YARELY,
                J9_PROVINCIALISMS(kDelta->mapEntryIndex),
                bytes,
                (jmtPOINTER *) &kMapEntryIndex
                ));

            kDelta->mapEntryIndex = J9_SUBAUDIBLENESS(Context->prevMapEntryIndex);
        }

        Context->prevDeltaPtr = kDelta;


        j9_recaution(jmkKERNEL_CloseUserData(
            kernel, needCopy,
            J9_YARELY,
            uDelta, J9_NONPROS(j9_revalidating),
            (jmtPOINTER *) &kDelta
            ));
    }

    JMM_kFOOTER_NO();
    return J9_FLUTTERING;

OnError:
    if (allocated)
    {
        j9_palladinize(jmkCONTEXT_DestroyPrevDelta(Context));
    }

    JMM_kFOOTER();
    return status;
}


#endif



